#!/usr/bin/perl
#
# Simple Perl program reproducing MineSet's 'remove_col' functionality
# 1/4/1999 by William H. Hsu
#

$TRUE = 1;
$FALSE = 0;

$ARGC = $#ARGV + 2;
if (($ARGC < 2) || ($ARGC > 3))
{
  print "Usage: REMCOLS.perl dataset-name {-n}\n";
  exit;
}

if ((3 == $ARGC) && ($ARGV[1] eq "-n"))
{
  $removeComplement = $TRUE;
}
else
{
  $removeComplement = $FALSE;
}

$remColFile = $ARGV[0] . ".remcols";
$dataFile = $ARGV[0] . ".data";
$namesFile = $ARGV[0] . ".names";

# Read each column deletion specification from the .remcols file
# ### LATER: hook this up with the MLC++/MineSet schema compiler
open (fpRemCol, $remColFile);

($N, %nameAssoc) = &filterNamesFile ($namesFile);

print "Processing " . $N . " columns...\n";

# by default, mark all columns as included in the bit vector
for ($col = 0; $col < $N; $col++)
{
  if (FALSE == $removeComplement)
  {
    $includeName[$col] = 1;		# default: retain column
  }
  else
  {
    $includeName[$col] = 0;		# default: omit column
  }
}

while ($line = <fpRemCol>)
{
  $colName = &getColName ($line);
  $nameIndex = $nameAssoc{$colName};
  if (FALSE == $removeComplement)
  {
     print "Removing column " . $colName . ", index " . $nameIndex . "\n";
     $includeName[$nameIndex] = 0;		# mark as deleted
  }
  else
  {
     print "Retaining column " . $colName . ", index " . $nameIndex . "\n";
     $includeName[$nameIndex] = 1;		# mark as included
  }
}

close (fpNewCol);

# reprint the REMAINING data columns and names
&printDataCols ($dataFile, @includeName);
&printNames ($namesFile, @includeName);

# scan the list of names and insert them into an associative array (hash)
sub filterNamesFile
{
  local ($namesFile) = @_;
  local ($i) = 0;

  open (fpNames, $namesFile);

  while ($line = <fpNames>)
  {
    @tokens = split (" ", $line);
    $isDeclaration = $tokens[0] =~ /^[a-z]+[a-z0-9]*\:/i;
    if ($isDeclaration)
    {
      @extractedNames = split (":", $tokens[0]);
      $key = $extractedNames[0];
      $filteredNames{$key} = $i;
      $i++;
    }
  }

  close (fpNames);

  return ($i, %filteredNames);
}

# extract the name of the new column
sub getColName
{
  local ($line) = @_;
  @lineContents = split (" ", $line);
  return $lineContents[0];
}

# reprint the .data file, exlcluding the named columns
sub printDataCols
{
  local ($dataFile, @includeName) = @_;
  local ($tempDataFile) = $dataFile . "\.temp";
  local ($pointProcessed) = 0;

  open (fpData, $dataFile);
  open (fpNewData, "\>" . $tempDataFile);

  while ($dataPoint = <fpData>)
  {
    # pass the column indices to a function that takes a row of data and an
    #   inclusion-exclustion bit vector and filters the data by the bit vector
    $newDataPoint = &printDataPoint ($dataPoint, @includeName);
    print fpNewData $newDataPoint;

    $pointsProcessed++;
    if (0 == ($pointsProcessed % 1000))
    {
      print $pointsProcessed . " data points processed\n";
    }
  }

  close ($fpData);
  close ($fpNewData);

  system "mv $tempDataFile $dataFile";
}

# print a single data point with the specified elements removed
#   and return the string
sub printDataPoint
{
  local ($dataPoint, @includeName) = @_;
  local (@dataValues) = split (/\,|\./, $dataPoint);
  local ($N) = $#dataValues - 1; 
  local ($newLine) = "";

  # check for inclusion of each element
  for ($i = 0; $i < $N; $i++)
  {
    $newLine = $newLine . ($includeName[$i] ? $dataValues[$i] . "\," : "");
  }
 
  $newLine = $newLine . $dataValues[$N] . "\.\n";

  return $newLine;
}

# reprint the .names (schema) file, excluding the named rows
sub printNames
{
  local ($namesFile, @includeName) = @_;
  local ($i) = 0;
  local ($isDeclaration);
  local (@tokens);
  local ($tempNamesFile) = $namesFile . "\.temp";

  open (fpNames, $namesFile);
  open (fpNewNames, "\>" . $tempNamesFile);

  while ($line = <fpNames>)
  {
    @tokens = split (" ", $line);
    $isDeclaration = $tokens[0] =~ /^[a-z]+[a-z0-9]*\:/i;
    if ($isDeclaration)
    {
      print fpNewNames (($includeName[$i] == 1) ? $line : "");
      $i++;
    }
    else                                        # comment or blank line
    {
      print fpNewNames $line;
    }
  }

  close (fpNames);
  close (fpNewNames);

  system "mv $tempNamesFile $namesFile";
}
