// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AMInducer_h
#define _AMInducer_h 1

#include <BaseInducer.h>
#include <Inducer.h>
#include <MLCStream.h>
#include <LogOptions.h>


class AMInducer : public BaseInducer {
private:
   NO_DEFAULT_OPS(AMInducer);

   // Member data
   // inducer options.  -1 selects AM default
   Bool preserveWorkarea;
   Bool verbose;
   int numHiddenNodes;
   Real learningRate;
   Real inertia;
   int convTestIterations;
   int convTestPasses;
   Real convTestBound;
   int maxIterations;
   Real errorBound;
   MString existingRun;

   // Private methods
   MString create_am_workarea();
   void remove_am_workarea(const MString& workareaName,
			   const MString& prevDir);
   void write_aspirin_file(const MString& fileStem,
			   const SchemaRC& schema);
   void write_am_data(const MString& fileStem,
		      const InstanceList& trainList,
		      const InstanceList& testList);
   void compile_aspirin_file(const MString& aspirinName);
   void train_am_model(const MString& inducerName,
		       const MString& trainName,
		       const MString& modelName);
   CatTestResult *run_am_model(const MString& inducerName,
			       const MString& trainName,
			       const MString& modelName,
			       InstanceList& trainingSet,
			       const InstanceList& testSet);

   
public:
   AMInducer(const MString& description);
   virtual ~AMInducer();
   
   virtual int class_id() const { return AM_INDUCER; }

   // batchmode operations
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testList);
   virtual CatTestResult *train_and_perf(InstanceList *trainingSet,
					 const InstanceList& testSet);
   virtual Bool supports_full_testing() { return TRUE; }
   
   virtual void set_user_options(const MString& prefix);

   // specific option functions
   Bool get_preserve_workarea() const { return preserveWorkarea; }
   void set_preserve_workarea(Bool p) { preserveWorkarea = p; }

   Bool get_verbose() const { return verbose; }
   void set_verbose(Bool v) { verbose = v; }
   
   int get_num_hidden_nodes() const { return numHiddenNodes; }
   void set_num_hidden_nodes(int n) { numHiddenNodes = n; }

   Real get_learning_rate() const { return learningRate; }
   void set_learning_rate(Real a) { learningRate = a; }

   Real get_inertia() const { return inertia; }
   void set_inertia(Real i) { inertia = i; }

   void get_conv_test_info(int& iterations, int& passes, Real& bound) const
   { iterations = convTestIterations;  passes = convTestPasses;
   bound = convTestBound; }
   void set_conv_test_info(int iterations, int passes, Real bound)
   { convTestIterations = iterations; convTestPasses = passes;
   convTestBound = bound; }

   int get_max_iterations() const { return maxIterations; }
   void set_max_iterations(int m) { maxIterations = m; }

   Real get_error_bound() const { return errorBound; }
   void set_error_bound(Real e) { errorBound = e; }

   MString get_existing_run() const { return existingRun; }
   void set_existing_run(const MString& r) { existingRun = r; }
};

#endif





