// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef Array2_h
#define Array2_h

template <class Element>
class Array2 {
   NO_COPY_CTOR(Array2);
protected:  // Protected in case we want to derive a DynamicArray2 from Array2.
   // Protected member data 
   int numRows;
   int numCols;
   int startRow;
   int startCol; 
   Element *elements;

   // Protected methods
   void alloc_array(int startRow, int startCol, 
			 int numRows, int numCols); 
   void free() { delete [] elements; }
   void copy(const Array2& source); // used by copy constructor
   Bool equal_given_that_bounds_are_equal(const Array2& rhs) const;
public:
   Array2(const Array2& source, CtorDummy);
   Array2(int numRows, int numCols);
   Array2(int numRows, int numCols, const Element& initialValue);
   Array2(int startRow, int startCol, int numRows,
	  int numCols);
   Array2(int startRow, int startCol, int numRows,
	  int numCols, const Element& initialValue);
   virtual ~Array2() { free(); }
   Array2& operator= (const Array2& source);
   // Compares two arrays.  TRUE is returned only if the two Array2s have
   //   the same size and starting rows and columns.
   Bool bounds_are_equal(const Array2& rhs) const
      {return ((size() == rhs.size()) &&
	       (num_rows() == rhs.num_rows()) &&
	       (num_cols() == rhs.num_cols()) &&
	       (start_row() == rhs.start_row()) &&
	       (start_col() == rhs.start_col()));
      }
   Bool operator==(const Array2& rhs) const
      {return (bounds_are_equal(rhs) &&
	       equal_given_that_bounds_are_equal(rhs));}
   Bool operator!=(const Array2& rhs) const
      {return (!(operator==(rhs)));}
   void init_values(const Element& initalValue);
   int num_rows () const { return numRows;} 
   int num_cols ()  const { return numCols; }
   int start_row () const { return startRow; }
   int start_col () const { return startCol; }
   int high_row() const { return startRow + numRows - 1; }
   int high_col() const { return startCol + numCols - 1; }
   const Element& max_in_row(int row, int& index) const;
   const Element& min_in_row(int row, int& index) const;
   const Element& max_in_col(int col, int& index) const;
   const Element& min_in_col(int col, int& index) const;
   int size () const { return numRows*numCols; }
   Element& index(int i);
   const Element& index(int i) const;
   // The following two methods return the same data as the index()
   //   methods--but the fast_index() methods do NOT do ANY error checking.
   //   They are provided for speed--not safety.
   Element& fast_index(int i) {return elements[i];}
   const Element& fast_index(int i) const {return elements[i];}
   Element& operator()(int row,int col);
   const Element& operator()(int row, int col) const {
      DBG(if (row < startRow || row >= numRows + startRow)
	  err<<"Array2<>::operator() const: row reference " << row <<
	  " is out of bounds (" << startRow << ".." << startRow + numRows - 1
	  << ')' << fatal_error;

          if (col < startCol || col >= numCols + startCol)
	  err<<"Array2<>::operator() const: column reference " << col <<
	  " is out of bounds (" << startCol << ".." << startCol + numCols - 1
	  << ')' << fatal_error); 
  
      return elements[(row-startRow)*numCols + (col-startCol)];
   }
   void sum_rows(Array<Element>& returnColumn) const;
   void sum_cols(Array<Element>& returnRow) const;

   Element total_sum() const;

   void display(MLCOStream& stream = Mcout) const;
};

template <class Element> 
   Bool bounds_are_equal(const Array2<Element>& lhs,
			 const Array2<Element>& rhs)
   {return (lhs.bounds_are_equal(rhs));}
template <class Element> 
   Bool operator==(const Array2<Element>& lhs,
		   const Array2<Element>& rhs)
   {return lhs.operator==(rhs);}
template <class Element> 
   Bool operator!=(const Array2<Element>& lhs,
		   const Array2<Element>& rhs) 
   {return (lhs.operator!=(rhs));}

#ifndef PC_MSVC_AMBIG
#endif
template <class Element>
class PtrArray2: public Array2<Element> {
   NO_COPY_CTOR2(PtrArray2,Array2<Element>(1, 1));
public:
   PtrArray2(int numRows, int numCols, Bool init=TRUE);
   PtrArray2(int startRow, int startCol, int numRows,
	     int numCols, Bool init=TRUE);
   ~PtrArray2();          
   // Disabled by default
   PtrArray2<Element>& operator= (const PtrArray2<Element>& source);
};

template <class Element>
DECLARE_DISPLAY(Array2<Element>)

template <class Element>
DECLARE_DISPLAY(PtrArray2<Element>)

#ifdef COMPILE_TIME_TEMPLATES
#include "Array2.ct"
#endif
   
#endif

