// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AssocGenerator_h
#define _AssocGenerator_h 1

#include <SmartInstCtr.h>
#include <DisplayPref.h>
#include <DynamicArray.h>
#include <AssocRule.h>

class AssocGenerator {
   NO_DEFAULT_OPS(AssocGenerator);

   // Member data
   SmartInstCounter counter;
   DynamicArray<AssocRule> rules;
   Real minPrevalence;
   Real minPredictability;

   // Private methods
   void add_one_way_rule(int attrA, NominalVal valA,
			 int attrB, NominalVal valB,
			 Real prevalence, Real predictability,
			 Real expectedPredictability);
   
public:
   AssocGenerator(const InstanceList& il);
   ~AssocGenerator() { }

   void gen_association_rules();

   const Array<AssocRule>& get_rules() const { return rules; }
   void reset_rules();
   
   void display(MLCOStream& stream = Mcout,
		const DisplayPref& pref = defaultDisplayPref) const;

   void set_user_options(const MString& prefix = EMPTY_STRING);
   
   void set_min_prevalence(Real p);
   Real get_min_prevalence() const { return minPrevalence; }

   void set_min_predictability(Real p);
   Real get_min_predictability() const { return minPredictability; }
};
   
DECLARE_DISPLAY(AssocGenerator)

#endif
