// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AssocRule_h
#define _AssocRule_h 1

#include <DisplayPref.h>

class SchemaRC;
class IntArray;

class AssocRule {
   NO_COPY_CTOR(AssocRule);

   // Member data
   SchemaRC *schema;
   IntArray *lhsAttrs;
   IntArray *lhsAttrVals;
   IntArray *rhsAttrs;
   IntArray *rhsAttrVals;

   Real prevalence;
   Real predictability;Real
   expectedPredictability;

   // Private methods
   void unset();
   void clear();   
public:
   void OK(int level = 0) const;
   AssocRule();
   AssocRule(const SchemaRC& sch, IntArray*& lAttrs, IntArray*& lAttrVals,
	     IntArray*& rAttrs, IntArray*& rAttrVals, Real prev,
	     Real pred, Real expPred);
   AssocRule& operator=(AssocRule& rhs);
   ~AssocRule() { clear(); }

   void assign_data(AssocRule& source);

   void display(MLCOStream& stream = Mcout,
		const DisplayPref& pref = defaultDisplayPref) const;

   const SchemaRC& get_schema() const;
   const IntArray& get_lhs_attrs() const;
   const IntArray& get_lhs_attr_vals() const;
   const IntArray& get_rhs_attrs() const;
   const IntArray& get_rhs_attr_vals() const;
   Real get_prevalence() const;
   Real get_predictability() const;
   Real get_expected_predictability() const;
};

DECLARE_DISPLAY(AssocRule)

#endif
