// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AttrCategorizer_h
#define _AttrCategorizer_h 1

#include <NodeCategorizer.h>
#include <DisplayPref.h>

// Persistent helper for AttrCategorizer
class PerAttrCategorizer_ : public PerNodeCategorizer_ {
   NO_DEFAULT_OPS(PerAttrCategorizer_);
public:
   int attrNum;

   PerAttrCategorizer_()
      : attrNum(UNDEFINED_INT) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


// AttrCategorizer categorizes according to the value of a single
// attribute.
class AttrCategorizer : public NodeCategorizer {
   NO_DEFAULT_OPS(AttrCategorizer);

   // Member data
   AttrInfo *attrInfo;
   int attrNum;
public:
   AttrCategorizer(const SchemaRC& schema, int attrNum,
                   const MString& dscr);
   AttrCategorizer(const AttrCategorizer& source, CtorDummy);
   AttrCategorizer(const PerAttrCategorizer_& data, const SchemaRC& schema);
   
   virtual ~AttrCategorizer() { delete attrInfo; }

   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp=defaultDisplayPref) const;
   // Returns a pointer to a deep copy of this AttrCategorizer
   virtual Categorizer* clone() const;
   // Returns the class id
   virtual int class_id() const { return CLASS_ATTR_CATEGORIZER; }

   virtual Bool operator==(const Categorizer& rhs) const;
   int attr_num() const { return attrNum; }

   virtual AugCategory branch(const InstanceRC& inst) const;
   virtual AugCategory *get_edge_label(int branchNum) const;
   
   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);

   // make persistent
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerAttrCategorizer_; }
   virtual void make_persistent(PerCategorizer_& data) const;

   virtual Bool supports_persistence() const { return TRUE; }

   int tester_split_bin(const InstanceRC inst) const // for testers only
   { return branch(inst); }
};


#endif
