// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#error-----------use RealAndLabelColumn.h instead of AttrColumn.h
#ifndef _AttrColumn_h
#define _AttrColumn_h 1

#include <Array.h>
#include <Array2.h>

class AttrLabelPair {
public:
   // a compact representation is preferrable, one might even pack
   // both the value and the label into 4 bytes in the future
   StoredReal value;
   int label;

   // operators needed for sorting
   Bool operator<(const AttrLabelPair& other) const
   { return value < other.value; };
   Bool operator==(const AttrLabelPair& other) const
   { return value == other.value; };
   Bool operator>(const AttrLabelPair& other) const
   { return value > other.value; };

   void display(MLCOStream& stream = Mcout) const;
};

DECLARE_DISPLAY(AttrLabelPair)

class AttrColumn {
   // @@ do we want to make add_pair public instead of friending
   // @@ everyone here?
   friend class InstanceList;	// needed to call add_pair;
   friend class AttributeColumn;// needed for same reason (for FC)
   int numPairs;		// total number of pairs in the array
   int numUnknown;
   Array<AttrLabelPair>* data;	// the pairs; we can not use DynamicArray
				// since we will have to split it later
   Array<int> labCounts;	// how many of each label
   Array<int> unknownLabCounts;	// how many of each label are unknown
   NO_COPY_CTOR(AttrColumn);
   void add_pair(StoredReal val, int lab); // only InstanceList can do this
   void add_unknown(int attrNum); // only InstanceList can do this

public:
   AttrColumn* split(int splitIndex);
   AttrColumn(int maxLength, int numLabels);
   AttrColumn(const AttrColumn& source, CtorDummy)
      : numPairs(source.numPairs),
	data(new Array<AttrLabelPair>(*source.data, ctorDummy)),
	labCounts(source.labCounts, ctorDummy),
	unknownLabCounts(source.unknownLabCounts, ctorDummy)
   { }
   ~AttrColumn() { delete data; };
   void OK() const;
   
   int get_num_instances() const { return numPairs; };
   int get_num_labels() const { return labCounts.size(); };
   int get_num_unknowns() const { return numUnknown; };
   // return the AttrLabelPair block for efficiency
   AttrLabelPair& index(int index) { return data->index(index); };
   const AttrLabelPair& index(int index) const { return data->index(index); }
   void sort() { data->truncate(numPairs); data->sort(); };
   // a shortcut again for the efficiency reasons
   int init_dist_counts(Array2<int>& splitAndLabelDist,
			Array<int>& laabelDist) const;
   void display(MLCOStream& stream = Mcout) const;
};

DECLARE_DISPLAY(AttrColumn)     

#endif


