// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AttrEqCategorizer_h
#define _AttrEqCategorizer_h 1

#include <Categorizer.h>
#include <DisplayPref.h>
#include <InstanceList.h>


// AttrEqCategorizer categorizes according to the value of a single
// attribute.
class AttrEqCategorizer : public Categorizer {
   NO_DEFAULT_OPS(AttrEqCategorizer);

   // Member data
   int attrNum;
   const NominalVal attrVal;
   Bool  separateUnknowns; // On unknown, return unknown or simply not-equal
   
   static const MString equal;
   static const MString notEqual;
   static const MString unknown;
public:
   AttrEqCategorizer(const SchemaRC& schema, int attrNum,
		     NominalVal attrVal, const MString& dscr,
		     Bool separateUnknown);
   AttrEqCategorizer(const AttrEqCategorizer& source, CtorDummy);
   virtual ~AttrEqCategorizer() {};
   virtual AugCategory categorize(const InstanceRC&) const;
   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp=defaultDisplayPref) const;
   // Returns a pointer to a deep copy of this AttrEqCategorizer
   virtual Categorizer* clone() const;
   // Returns the class id
   virtual int class_id() const { return CLASS_ATTR_EQ_CATEGORIZER; }
   int attr_num() const { return attrNum; }

   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);

   virtual InstanceListArray* split_instance_list(const InstanceList& il)
      const;
   
   virtual Bool operator==(const Categorizer& rhs) const;
};


#endif
