// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.


#ifndef _AttrOrder_h
#define _AttrOrder_h 1

#include <Array.h>
#include <InstanceList.h>
#include <MString.h>

class AttrOrder {
public:
   enum OrderType { sequential, user, mutualInfo };

private:
   NO_COPY_CTOR(AttrOrder);

   // Member data
   Array<int> *order;
   OrderType orderType;
   Bool orderSet;

   // Private helper methods
   void set_order_from_user(const MString&, const InstanceList&);
   void set_order_from_minfo(const LogOptions& logOptions,const InstanceList&);

   void copy(const AttrOrder& source);
   void destroy();
public:
   AttrOrder() { orderSet = FALSE; order = NULL; orderType = mutualInfo; }
   AttrOrder(const AttrOrder& source, CtorDummy);
   AttrOrder& operator=(const AttrOrder& source);
   ~AttrOrder() { destroy(); }
   void init();

   const Array<int>& get_order() const;
   void set_order(const Array<int>& array);

   Bool order_set() const { return orderSet;}
   OrderType get_order_type() const { return orderType; }
   void set_order_type(OrderType type) { orderType = type; }
   void set_user_options(const MString& preFix);

   const Array<int>& compute_order(const LogOptions& logOptions,
				   const InstanceList& instList,
				   const MString preFix = EMPTY_STRING);
   virtual Bool operator==(const AttrOrder &rhs) const ;
   Bool operator!=(const AttrOrder &rhs) const 
      {return ! ((*this)== rhs);}
};
#endif

