// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AttrSubsetCat_h
#define _AttrSubsetCat_h 1

#include <NodeCategorizer.h>
#include <LogOptions.h>
#include <MStringRC.h>
#include <Persistence.h>
#include <DisplayPref.h>

class PerAttrSubsetCat_ : public PerNodeCategorizer_ {
public:
   NO_DEFAULT_OPS(PerAttrSubsetCat_);
   int attrNum;
   int numSubsets;
   Array<int> *whichSubset;
   Array<MString> *subsetString;

   PerAttrSubsetCat_()
      : attrNum(UNDEFINED_INT), numSubsets(UNDEFINED_INT),
      whichSubset(NULL), subsetString(NULL) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


class AttrSubsetCat : public NodeCategorizer {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(AttrSubsetCat);

   // Member data
   Array<NominalVal> *whichSubset; // subset number this NominalVal belongs to
   Array<MStringRC> subsetString;
   int attrNum;
   const int numSubsets;
public:
   AttrSubsetCat(const SchemaRC& sch, int attrNum, int numSubsets,
		 Array<NominalVal>*& whichSub, const MString& dscr,
		 const Array<NominalVal>& zeroes);
   AttrSubsetCat(const AttrSubsetCat& source, CtorDummy);
   AttrSubsetCat(const PerAttrSubsetCat_& data,
		 const SchemaRC& schema);
   virtual ~AttrSubsetCat() { DBG(OK()); delete whichSubset; }
   virtual void OK(int level = 0) const;
   virtual int class_id() const { return CLASS_ATTR_SUBSET_CATEGORIZER; }

   virtual AugCategory branch(const InstanceRC& instance) const;
   virtual AugCategory *get_edge_label(int branchNum) const;
   
   int attr_num() const { return attrNum; }
   const MString& subset_name(int which) const
   { return *subsetString[which].read_rep(); }
   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp=defaultDisplayPref) const;
   virtual Categorizer* clone() const;
   virtual Bool operator==(const Categorizer& rhs) const;
   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);

   // persistence
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerAttrSubsetCat_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   virtual Bool supports_persistence() const { return TRUE; }

};


#endif
