// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _AttributeColumn_h
#define _AttributeColumn_h 1

#include <Attribute.h>
#include <RealAndLabelColumn.h>

typedef AttrValue_ (*AttrOp)(const AttrValue_& lhs, const AttrInfo& lInfo,
			  const AttrValue_& rhs, const AttrInfo& rInfo,
			  const AttrInfo& returnInfo);

class AttributeColumn {
   NO_DEFAULT_OPS(AttributeColumn);

   // Member data
   const AttrInfo& attrInfo;
   Array<AttrValue_> values;
public:
   void OK(int level = 0) const;
   
   AttributeColumn(const AttrInfo& info, int numValues);
   AttributeColumn(const AttributeColumn& rhs, CtorDummy);
   ~AttributeColumn();
   
   RealAndLabelColumn *build_attr_column(const AttributeColumn& labels,
					 const Array<Real>& weights,
					 int numLables) const;
	
   Array2<Real> *count_with_labels(const AttributeColumn& labels,
				   const Array<Real>& weights) const;
   
   AttrValue_& index(int i) { return values.index(i); }
   AttrValue_ index(int i) const { return values.index(i); }
   
   const AttrInfo& attr_info() const { return attrInfo; }
   int get_num_instances() const { return values.size(); }

   void column_op(AttrOp op, const AttributeColumn& lhs,
		  const AttributeColumn& rhs);

   static AttrValue_ apply_op(AttrOp op,
			      const AttrValue_& lhs, const AttrInfo& lInfo,
			      const AttrValue_& rhs, const AttrInfo& rInfo,
			      const AttrInfo& returnInfo);
   
   // You can't use "AttrOp" to declare functions in the header, at least
   // not safely.  The macro accomplishes the same task.
   #define DECLARE_ATTR_OP(name) \
      static AttrValue_ name(const AttrValue_& lhs, const AttrInfo& lInfo,\
			  const AttrValue_& rhs, const AttrInfo& rInfo,\
			  const AttrInfo& returnInfo)
   DECLARE_ATTR_OP(add);
   DECLARE_ATTR_OP(subtract);
   DECLARE_ATTR_OP(multiply);
   DECLARE_ATTR_OP(equiv);
   DECLARE_ATTR_OP(divide);

};

#endif
