// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _BFSearch_h
#define _BFSearch_h 1

#include <SSSearch.h>

template <class LocalInfo, class GlobalInfo>
class BFSearch : public StateSpaceSearch<LocalInfo, GlobalInfo> {
   NO_DEFAULT_OPS(BFSearch);
protected:
   // Protected member data
   int maxNonImprovingExpansions;
   Real minExpansionImprovement;
   Real maxFitness;
   int beamWidth;
   Bool prune;
   
   int numNonImprovingExpansions;
   NodePtr bestNode;
   DblLinkList<NodePtr> openList;
   DblLinkList<NodePtr> closedList;

   // Protected methods
   virtual Bool terminate();
public:
   BFSearch() { set_defaults(); }
   virtual ~BFSearch() {}

   virtual void set_user_options(const MString& stem);
   virtual void set_defaults();

   virtual void set_beam_width(int width);
   virtual void set_prune(Bool doPrune) { prune = doPrune; }
   virtual void set_max_non_improving_expansions(int maxNIE)
      { maxNonImprovingExpansions = maxNIE; }
   virtual void set_min_expansion_improvement(Real minEI)
      { minExpansionImprovement = minEI; }
   virtual void set_max_fitness(Real maxF)
   { maxFitness = maxF; }

   virtual const State<LocalInfo, GlobalInfo>&
      search(State<LocalInfo, GlobalInfo>* initialState, GlobalInfo *gInfo,
	     const MString& displayFileName = EMPTY_STRING);
};

#ifdef COMPILE_TIME_TEMPLATES
#include "BFSearch.ct"
#endif

#endif


