// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _Bagging_h
#define _Bagging_h 1

#include <Categorizer.h>
#include <DisplayPref.h>
#include <CatDist.h>
#include <Array.h>


class BaggingCat : public Categorizer {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(BaggingCat);

   // Member data (also see public data)
   // we need a pointer here to get an ownership from outside.
   PtrArray<Categorizer *> * catSet;
   Array<Real> *weightSet;
   Real avgWeight;
   Bool useAboveAvgWeight;
   Array<int>  numCorrect;  // statistics: how many did you get correct
   int numCategorize;       // number of categorizations done
   Bool useScoring;
   
public:

   // Public data
   static const Bool defaultUseAboveAvgWeight;
   
   virtual void OK(int level = 0)const;

   // @@ Modify this to start using scoring
   BaggingCat( const MString& dscr,
	       PtrArray<Categorizer *>*& cats,
	       Array<Real>*& weights,
	       Bool inUseScoring = FALSE);
   
   BaggingCat( const BaggingCat& source,
	       const CtorDummy /* dummyArg */);

   virtual ~BaggingCat();

   virtual const PtrArray<Categorizer *> & get_categorizer_set() const
       {return *catSet;}
   virtual const Array<Real> & get_weight_set() const
       {return *weightSet;}   

   virtual AugCategory categorize(const InstanceRC& inst) const;
   virtual CatDist *score(const InstanceRC&) const;
   virtual Bool supports_scoring() const {
      return useScoring;
   }

   virtual void display_struct(MLCOStream& stream,
			       const DisplayPref& dp) const;

   virtual void display(MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const;

   virtual Categorizer* clone() const;

   
   virtual int class_id() const { return CLASS_BAGGING_CATEGORIZER; }

   virtual Bool operator==(const Categorizer& rhs) const;

   virtual void set_use_above_avg_weight(Bool val) {
      useAboveAvgWeight = val; }
   virtual Bool get_use_above_avg_weight() const { return useAboveAvgWeight; }
   
};

DECLARE_DISPLAY(BaggingCat)

#endif




