// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _BaggingInd_h
#define _BaggingInd_h

#include <Inducer.h>
#include <InstanceList.h>
#include <StackingInd.h>
#include <MRandom.h>
#include <BaggingCat.h>

class BaggingInd : public Inducer {
   RAND_OPTIONS;
   NO_DEFAULT_OPS(BaggingInd);
public:
   enum CombinationType { normal, stacker };
private:
   
   // Member data (also see public data)
   Categorizer* categorizer;
   StackingInd* stackerInducer;

   // Options.
   int numReplication;
   Real proportion;
   Inducer *mainInducer;
   Bool unifWeights;
   Bool useTrainingError;
   Bool useAboveAvgWeight;
   Bool withReplacement;
   //Bool showPerfPerTrial;
   //Bool showPerfPostTrial;
   Bool withNormalization;
   Bool withPerturbation;
   Bool useScoring;
   Bool useBackfit;
   Real perturbStdDev;
   Real perturbSkewFactor;
   unsigned perturbSeed;
   unsigned int randomSeed;
   CombinationType combinationType;


public:
   // Public data
   static const int defaultNumReplication;
   static const Real defaultProportion;
   static Inducer *defaultMainInducer; // Modified by set_main_inducer
   static const Bool defaultUnifWeights;
   static const Bool defaultUseTrainingError;
   static const Bool defaultUseAboveAvgWeight;
   static const Bool defaultWithReplacement;
   static const Bool defaultUseScoring;
   static const Bool defaultUseBackfit;
   static const Bool defaultWithPerturbation;
   static const Bool defaultWithNormalization;
   static const Real defaultPerturbStdDev;
   static const Real defaultPerturbSkewFactor;
   static const unsigned defaultPerturbSeed;
   static const unsigned int defaultRandomSeed;
   static const BaggingInd::CombinationType defaultCombinationType;

   // Public methods
   BaggingInd(const MString& dscr);
   virtual ~BaggingInd();
   virtual int class_id() const { return BAGGING_INDUCER; }
   
   virtual void OK(int level = 0) const;
   virtual void train();
   // Real train_and_test(InstanceList* trainingSet,
   //	       const InstanceList& testSet);
   virtual Bool was_trained(Bool fatal_on_false = TRUE) const;
   virtual const Categorizer& get_categorizer() const;
   virtual Categorizer *release_categorizer();
   Bool can_cast_to_inducer() const;
   
   // options.
   virtual void set_num_replication(int num = defaultNumReplication);
   virtual int get_num_replication() const { return numReplication; }
   virtual void set_proportion(Real val = defaultProportion);
   virtual Real get_proportion() const { return proportion; }   
   virtual void set_main_inducer(Inducer* mainInd = defaultMainInducer);
   virtual const Inducer* get_main_inducer() const { return mainInducer; }
   virtual void set_unif_weights(Bool val) { unifWeights = val; }
   virtual Bool get_unif_weights() const { return unifWeights; }
   virtual void set_user_options(const MString& prefix);
   virtual void set_use_above_avg_weight(Bool val) { useAboveAvgWeight = val; }
   virtual Bool get_use_above_avg_weight() const {
      return useAboveAvgWeight; }
   virtual void set_with_replacement(Bool val) { withReplacement = val; }
   virtual Bool get_with_replacement() const { return withReplacement; }
   virtual void set_with_normalization(Bool val) { withNormalization = val; }
   virtual Bool get_with_normalization() const { return withNormalization; }
   virtual void set_with_perturbation(Bool val) { withPerturbation = val; }
   virtual Bool get_with_perturbation() const { return withPerturbation; }
   virtual void set_perturb_std_dev(Real val) { perturbStdDev = val; }
   virtual Real get_perturb_std_dev() const { return perturbStdDev; }   
   virtual void set_perturb_skew_factor(Real val) { perturbSkewFactor = val; }
   virtual Real get_perturb_skew_factor() const { return perturbSkewFactor; }
   virtual void set_perturb_seed(unsigned val) { perturbSeed = val; }
   virtual unsigned get_perturb_seed() const { return perturbSeed; }      
   virtual void set_combination_type(BaggingInd::CombinationType val)
   { combinationType = val; }
   virtual CombinationType get_combination_type() const
   { return combinationType; }
   virtual void set_use_scoring(Bool val) { useScoring = val; }
   virtual Bool get_use_scoring() const { return useScoring; }
   virtual void set_use_backfit(Bool val) { useBackfit = val; }
   virtual Bool get_use_backfit() const { return useBackfit; }   
};
#endif
