// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _BaseInducer_h
#define _BaseInducer_h 1

#include <class_id.h>

#define CONST_INDUCER         INDUCER_ID_BASE +  1
#define TDDT_INDUCER          INDUCER_ID_BASE +  2
#define TABLE_INDUCER         INDUCER_ID_BASE +  3
#define C45_INDUCER           INDUCER_ID_BASE +  4
#define FSS_INDUCER           INDUCER_ID_BASE +  5
#define OODG_INDUCER          INDUCER_ID_BASE +  6
#define NAIVE_BAYES_INDUCER   INDUCER_ID_BASE +  7
#define NULL_INDUCER          INDUCER_ID_BASE +  8
#define IB_INDUCER            INDUCER_ID_BASE +  9
#define ONER_INDUCER          INDUCER_ID_BASE + 10
#define DISC_TAB_INDUCER      INDUCER_ID_BASE + 11
#define DISC_NB_INDUCER       INDUCER_ID_BASE + 12
#define DF_INDUCER            INDUCER_ID_BASE + 13
#define LIST_HOODG_INDUCER    INDUCER_ID_BASE + 14
#define WINNOW_INDUCER        INDUCER_ID_BASE + 15
#define ID3_INDUCER           INDUCER_ID_BASE + 16
#define PERCEPTRON_INDUCER    INDUCER_ID_BASE + 17
#define LAZY_DT_INDUCER       INDUCER_ID_BASE + 18
#define HOODG_INDUCER         INDUCER_ID_BASE + 19
#define PERF_EST_INDUCER      INDUCER_ID_BASE + 20
#define BAGGING_INDUCER       INDUCER_ID_BASE + 21
#define ENTROPY_ODG_INDUCER   INDUCER_ID_BASE + 22
#define LIST_ODG_INDUCER      INDUCER_ID_BASE + 23
#define C45R_INDUCER          INDUCER_ID_BASE + 24
#define ORDER_FSS_INDUCER     INDUCER_ID_BASE + 25
#define DISC_SEARCH_INDUCER   INDUCER_ID_BASE + 26
#define C45AP_INDUCER         INDUCER_ID_BASE + 27
#define CatDT_INDUCER         INDUCER_ID_BASE + 28
#define CF_INDUCER            INDUCER_ID_BASE + 29
#define TABLE_CAS_INDUCER     INDUCER_ID_BASE + 30
#define WEIGHT_SEARCH_INDUCER INDUCER_ID_BASE + 31
#define PROJECT_INDUCER       INDUCER_ID_BASE + 32
#define T2_INDUCER            INDUCER_ID_BASE + 33
#define STACKING_INDUCER      INDUCER_ID_BASE + 34
#define SGI_DT_INDUCER        INDUCER_ID_BASE + 35
#define FCF_INDUCER           INDUCER_ID_BASE + 36
#define AM_INDUCER            INDUCER_ID_BASE + 37
#define PEBLS_INDUCER         INDUCER_ID_BASE + 38
#define RIPPER_INDUCER        INDUCER_ID_BASE + 39
#define OC1_INDUCER           INDUCER_ID_BASE + 40
#define COODG_INDUCER         INDUCER_ID_BASE + 41
#define CN2_INDUCER           INDUCER_ID_BASE + 42
#define AHA_IB_INDUCER        INDUCER_ID_BASE + 43
#define CART_INDUCER          INDUCER_ID_BASE + 44
#define C50_INDUCER           INDUCER_ID_BASE + 45
#define ODT_INDUCER           INDUCER_ID_BASE + 46
#define CLUSTER_INDUCER       INDUCER_ID_BASE + 47
#define BOOSTER_INDUCER       INDUCER_ID_BASE + 48
#define DDT_INDUCER           INDUCER_ID_BASE + 49


#include <LogOptions.h>
#include <AugCategory.h>
#include <InstanceList.h>

class InstanceList;
class Categorizer;
class Inducer;
class IncrInducer;
class CatTestResult;

extern const MString DEFAULT_DATA_EXT;
extern const MString DEFAULT_NAMES_EXT;
extern const MString DEFAULT_TEST_EXT;

class BaseInducer { // ABC
   LOG_OPTIONS;
   NO_DEFAULT_OPS(BaseInducer);

   // Member data (also see protected data)
   const MString dscr;   
protected:
   // Protected member data
   InstanceList *TS;
   
public:
   virtual void OK(int level = 0) const {(void)level;}
   BaseInducer(const MString& description) : dscr(description) {TS=NULL;}
   BaseInducer(const BaseInducer& source, CtorDummy ctorDummy);
   virtual ~BaseInducer();

   // Returns the class id
   virtual int class_id() const = 0;

   const MString& description() const {return dscr;};
   // read_data reads the training set.  The file name suffices for the data
  // file and names file default to ".data" and ".names"
  // respectively.  If the files have a different prefix, give an
  // empty file name, and give the full file name as suffix.
   virtual void read_data(const MString& file,
			  const MString& namesExtension = DEFAULT_NAMES_EXT,
			  const MString& dataExtension = DEFAULT_DATA_EXT);

   virtual InstanceList* assign_data(InstanceList*& newTS);

   // release_data gives you back the InstanceList.
   virtual InstanceList* release_data();
   virtual Bool has_data(Bool fatalOnFalse = TRUE) const;
   const InstanceList& instance_list() const;
   // change the representaion ownership of refcounted objects
   PTHR_DECL(void change_ownership() { TS->change_ownership(); })
   // These are non-const becuase we temporarily need to store the trainingSet
   // We need temporary ownership of trainingSet, but return it.
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testSet) = 0;
   virtual CatTestResult *train_and_perf(InstanceList* trainingSet,
					 const InstanceList& testSet);

   // Determines if this Inducer supports full testing (i.e.
   // supports the train_and_perf/ train_and_perf_files functions)
   virtual Bool supports_full_testing() { return FALSE; }
   
   // Convenience function that reads the files in.
   // This isn't called train_and_test (the obvious choice) because derived
   //    classes will hide this function if they only define the InstanceList
   //    version (scope is different).
   virtual Real train_and_test_files(const MString& fileStem,
			     const MString& namesExtension = DEFAULT_NAMES_EXT,
			     const MString& dataExtension = DEFAULT_DATA_EXT,
			     const MString& testExtension = DEFAULT_TEST_EXT);
   virtual Bool can_cast_to_inducer() const;
   virtual Inducer& cast_to_inducer();
   virtual Bool can_cast_to_incr_inducer() const;
   virtual IncrInducer& cast_to_incr_inducer();

   // Weight functions
   virtual void normalize_weights(Real normFactor);
};

#endif
