// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _BoosterInducer_h
#define _BoosterInducer_h 1
#include <Array.h>
#include <Inducer.h>
#include <GetOption.h>
#include <BaseInducer.h>
#include <StackingInd.h>
#include <InstanceList.h>

class BoosterInducer : public Inducer { 
  NO_DEFAULT_OPS(BoosterInducer);
public:
   enum WeightType { adaBoost, probAdaBoost, random, x4 };
   enum CombinationType { ada, stacker };
private:
   // Member data
   Categorizer *categorizer;
   Inducer* inducer;
   StackingInd* stackerInducer;
   int numTrials;
   WeightType weightType;
   CombinationType combinationType;
   unsigned randomWeightSeed;
   unsigned boostSampleSeed;
   Bool useSampling;
   Bool stopAtZero;
   //Bool showPerfPerTrial;
   //Bool showPerfPostTrial;
   Bool dumpMineset;
   MString minesetName;
   Bool useBackfit;
   Bool trashHighWeights;
   Real trashHighWeightThreshold;
   int maxResamples;
   Real minWeight;
   Real maxWeight;
   Bool dumpLists;
   Bool useScoring;
   Bool allowZeros;
   Bool perturb;
   Real perturbSD;

   // Private methods
   void run_boosting(const InstanceList &newData,
		     PtrArray<Categorizer*> *&subCategorizers,
		     Array<Real> *&subWeights,
		     const InstanceList *testData);
   void truncate_to_trial(int trial,
			  PtrArray<Categorizer*> *&subCategorizers,
			  Array<Real> *&subWeights);
   void reset_weights(const InstanceList &origData,
		      InstanceList *&weightedData,
		      Real &totalWeight);		      
   Bool update_weights(const InstanceList &origData,
		       Real error, Categorizer *&cat,
		       MRandom &weightRandom, Array<int> &errors,
		       int &trial,
		       Array<Real> *&subWeights,
		       InstanceList *&weightedData,
		       Real &totalWeight);
   void update_ada_weights(const Categorizer &cat, Real beta,
			   InstanceList &weightedData);
   void update_prob_ada_weights(const Categorizer &cat, Real beta,
			   InstanceList &weightedData);
   void update_random_weights(MRandom &weightRandom,
			      InstanceList &weightedData);
   void update_x4_weights(const Categorizer &cat,
			  Array<int> &errors,
			  InstanceList &weightedData);
   Bool print_end_weights(const InstanceList &weightedData) const;
   void dump_mineset_list(const InstanceList &instList, int trial)
      const;
public:
   BoosterInducer(const MString& descr);
   virtual ~BoosterInducer();
   void train();
   Bool was_trained(Bool fatalOnFalse = TRUE) const;
   const Categorizer& get_categorizer() const;
   virtual Categorizer* release_categorizer();

   int class_id() const { return BOOSTER_INDUCER;}
   Real train_and_test(InstanceList* trainingSet,
		       const InstanceList& testSet);

   virtual void display(MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const;
   virtual void display_options(MLCOStream& stream,
				const DisplayPref& dp) const;
      
   Bool can_cast_to_inducer() const;
   virtual Bool supports_full_testing() { return FALSE; }
   void set_user_options(const MString& prefix);
   void set_inducer(Inducer*& inducer);
   Inducer* release_inducer();
   void set_stacker_inducer(StackingInd*& inducer);
   StackingInd* release_stacker_inducer();
   int get_num_trials() const
   { return numTrials; }
   void set_num_trials(int inTrials)
   { numTrials = inTrials; }
   WeightType get_weight_type() const
   { return weightType; }
   void set_weight_type(WeightType inWeightType)
   { weightType = inWeightType; }
   CombinationType get_combination_type() const
   { return combinationType; }
   void set_combination_type(CombinationType inCombinationType)
   { combinationType = inCombinationType; }
   int get_random_weight_seed() const
   { return randomWeightSeed; }
   void set_random_weight_seed(int inSeed)
   { randomWeightSeed = inSeed; }
   int get_boost_sample_seed() const
   { return boostSampleSeed; }
   void set_boost_sample_seed(int inSeed)
   { boostSampleSeed = inSeed; }
   Bool get_use_sampling() const
   { return useSampling; }
   void set_use_sampling(Bool inBool)
   { useSampling = inBool; }
   Bool get_stop_at_zero() const
   { return stopAtZero; }
   void set_stop_at_zero(Bool inBool)
   { stopAtZero = inBool; }
};

DECLARE_DISPLAY(BoosterInducer)

#endif










