// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _Bootstrap_h
#define _Bootstrap_h 1

#include <Array.h>
#include <Array2.h>
#include <LogOptions.h>
#include <MRandom.h>
#include <PerfEstimator.h>
#include <InstanceList.h>

class BaseInducer;
class PerfData;

class Bootstrap : public PerfEstimator {

   NO_DEFAULT_OPS(Bootstrap);

public:
   // we want to make this enum public, but we have to declare it before
   // all private/protected declarations which depend on it.
   // Hence the unusual public/private/protected ordering.
   enum BootstrapType { simple = 0, fractional = 2 };

private:
   // Member data (also see protected data)
   int numTimes;
   BootstrapType bsType;
   Real bsFract;

protected:
   // Protected data
   static const int BootstrapDefaultNumTimes;
   static const BootstrapType BootstrapDefaultType;
   static const Real BootstrapDefaultFraction;

   Real apparentError;

   // Protected methods
   virtual Real train_and_test(BaseInducer& inducer,
			       InstanceList* trainList,
  			       const InstanceList& testList,
			       const MString& dumpSuffix,
			       PerfData *localPerfData);
   

public:
   Bootstrap(int nTimes = BootstrapDefaultNumTimes,
	     const BootstrapType type = BootstrapDefaultType,
	     Real fraction = BootstrapDefaultFraction);
   
   // options
   void set_times(int num);
   void set_type(BootstrapType whichType);
   void set_fraction(Real fract);
   int get_times() const { return numTimes; }
   BootstrapType get_type() const { return bsType; }
   Real get_fraction() const { return bsFract; }

   // override description and error estimation functions
   virtual MString description() const;  
   virtual Real estimate_performance(BaseInducer& inducer,
				     InstanceList& dataList);
   virtual Real estimate_performance(BaseInducer& inducer,
				     const MString& filestem);

};

#endif 
