// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _C45Inducer_h
#define _C45Inducer_h 1

#include <Inducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <DecisionTree.h>
#include <TmpFileName.h>
#include <LogOptions.h>

// These are useful as global functions.  In fact, they provide
//   more functionality than the inducer above.
// The first parameter should be set to the command line that
//   will execute C4.5.
Bool run_c45(const LogOptions& logOptions, const MString& c45Pgm, 
	    const InstanceList& trainingSet, const InstanceList& testList,
	    Real& pruneAcc, Real& noPruneAcc, Real& estimateAcc,
	    int& noPruneSize, int& pruneSize);


// This version is intended to be used if you're running C45 a lot.
// Profiling showed that file creation (creat) and deletion (unlink) were
//   taking up to 40% of the time in some cases, so this version takes open
//   streams, writes to them, and truncates.
Bool run_c45(const LogOptions& logOptions, const MString& c45Pgm,
	    const InstanceList& trainingList, const InstanceList& testList,
	    Real& pruneAcc, Real& noPruneAcc, Real& estimateAcc,
	    int& noPruneSize, int& pruneSize, const MString& tmpFileStem,
            MLCOStream& namesStream, MLCOStream& dataStream,
	    MLCOStream& testStream);

class C45Inducer : public BaseInducer {
   NO_DEFAULT_OPS(C45Inducer);

   // Member data (also see public data)
   Bool usePrunedTree;
   Bool cleanupNeeded; // If we run C4.5, we need a cleanup of some files it
		       // generates.
   MString pgmName;
   MString pgmFlags;
   
   PtrArray<TmpFileName *> *tempNames;
   MLCOStream* namesStream;
   MLCOStream* dataStream;
   MLCOStream* testStream;

   // Private methods
   void init_files();
   void end_files();

public:
   // Public data
   static const MString defaultPgmName;
   static const MString defaultPgmFlags;

   // Methods
   C45Inducer(const MString& description, 
	      const MString& theFlags = defaultPgmFlags,
	      Bool usePruned = TRUE,
	      const MString& aPgmName = defaultPgmName);
   C45Inducer(const C45Inducer&, CtorDummy ctorDummy);
   virtual ~C45Inducer();
   virtual int class_id() const { return C45_INDUCER; }
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testSet);
   virtual void set_pgm_flags(const MString &flags) { pgmFlags = flags; }
   virtual const MString& file_stem() { return tempNames->index(0)->get_stem();}
   const MString& get_flags() {return pgmFlags;}
};


#endif








