// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _C45RInducer_h
#define _C45RInducer_h 1

#include <Inducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <DecisionTree.h>
#include <TmpFileName.h>

// These are useful as global functions.  In fact, they provide
//   more functionality than the inducer above.
// The first parameter should be set to the command line that
//   will execute C4.5.
Bool run_c45R(const LogOptions& logOptions, const MString& c45RPgm, 
 	      const InstanceList& trainingSet,
	      const InstanceList& testList, int& numErr, int& numRules,
	      int& numConjuntions);


// This version is intended to be used if you're running C45 a lot.
// Profiling showed that file creation (creat) and deletion (unlink) were
//   taking up to 40% of the time in some cases, so this version takes open
//   streams, writes to them, and truncates.
Bool run_c45R(const LogOptions& logOptions, const MString& c45Pgm1,
	      const MString& c45Pgm2, const InstanceList& trainingList,
	      const InstanceList& testList, int& numErr, int& numRules,
	      int& numConjuntions, MString& tmpFileStem,
	      MLCOStream& namesStream, MLCOStream& dataStream,
	      MLCOStream& testStream);

class C45RInducer : public BaseInducer {
   NO_DEFAULT_OPS(C45RInducer);

   // Member data (also see public data)
   MString pgmName1;
   MString pgmFlags1;
   MString pgmName2;
   MString pgmFlags2;

   PtrArray<TmpFileName *> *tempNames;
   MLCOStream* namesStream;
   MLCOStream* dataStream;
   MLCOStream* testStream;

   // Private methods
   void init_files();
   void end_files();
public:
   // Public data
   static const MString defaultPgmName1;
   static const MString defaultPgmName2;
   static const MString defaultPgmFlags1;
   static const MString defaultPgmFlags2;

   // methods
   C45RInducer(const MString& description, 
	      const MString& theFlags1 = defaultPgmFlags1,
	      const MString& theFlags2 = defaultPgmFlags2,
	      const MString& aPgmName1 = defaultPgmName1,
	      const MString& aPgmName2 = defaultPgmName2);
   virtual ~C45RInducer();
   virtual int class_id() const { return C45R_INDUCER; }

   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testSet);
};


#endif








