// This file is a concatination of defns.h, global.c, and hooks.c.



//@@ defns.h

/*************************************************************************/
/*									 */
/*		Definitions used in C5.0				 */
/*              ------------------------				 */
/*									 */
/*************************************************************************/


#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>


/*************************************************************************/
/*									 */
/*		Constants, macros etc.					 */
/*									 */
/*************************************************************************/


#define	 Nil	   0		/* null pointer */
#define	 false	   0
#define	 true	   1
#define	 None	   -1 
#define	 Epsilon   1E-4

#define  IGNORE	   1		/* special attribute status: do not use */
#define  DISCRETE  2		/* ditto: collect values as data read */
#define  ASK	   3		/* ditto: used by predict */

#define	 Unknown   1.5777218104420236e-30	/* unlikely value! */

#define	 BrDiscr   1
#define	 BrThresh  2
#define	 BrSubset  3

#define  Alloc(N,T)		(T *) Pmalloc((N)*sizeof(T))
#define  AllocZero(N,T)		(T *) Pcalloc(N, sizeof(T))
#define  Realloc(V,N,T)		V = (T *) Prealloc(V, (N)*sizeof(T))

#define	 Bit(b)			(1 << (b))
#define	 In(b,s)		((s[(b) >> 3]) & Bit((b) & 07))
#define	 ClearBits(n,s)		memset(s,0,n)
#define	 CopyBits(n,f,t)	memcpy(t,f,n)
#define	 SetBit(b,s)		(s[(b) >> 3] |= Bit((b) & 07))

#define	 ForEach(v,f,l)		for(v=f ; v<=l ; ++v) 

#define  Discrete(a)		(MaxAttVal[a])
#define  Continuous(a)		(! MaxAttVal[a])

#define Verbosity(d,s)


/*************************************************************************/
/*									 */
/*		Type definitions					 */
/*									 */
/*************************************************************************/


typedef  unsigned char	Boolean, BranchType, *Set;
typedef	 char		*String;

typedef  int	ItemNo;		/* data item number */
typedef  float	ItemCount;	/* count of (partial) items */

typedef  int	ClassNo,	/* class number, 1..MaxClass */
		DiscrValue,	/* discrete attribute value (0 = ?) */
		Attribute;	/* attribute number, 1..MaxAtt */


typedef  union  _attribute_value
	 {
	    DiscrValue	_discr_val;
	    float	_cont_val;
	 }
	 AttValue, *Description;

#define  CVal(Case,Attribute)   Case[Attribute]._cont_val
#define  DVal(Case,Attribute)   Case[Attribute]._discr_val
#define  Class(Case)		(*Case)._discr_val
#define  Weight(Case)		(*(Case-1))._cont_val



typedef  struct _treerec	*Tree;
typedef  struct _treerec
	 {
	    BranchType	NodeType;
	    ClassNo	Leaf;		/* best class at this node */
	    ItemCount	Items,		/* no of items at this node */
			*ClassDist,	/* class distribution of items */
	    		Errors;		/* no of errors at this node */
	    Attribute	Tested; 	/* attribute referenced in test */
	    int		Forks;		/* number of branches at this node */
	    float	Cut,		/* threshold for continuous attribute */
		  	Lower,		/* lower limit of soft threshold */
		  	Upper;		/* upper limit ditto */
	    Set         *Subset;	/* subsets of discrete values  */
	    Tree	*Branch;	/* Branch[x] = subtree for outcome x */
	 }
	 TreeRec;


typedef  int	RuleNo;			/* rule number */

typedef  struct _condrec
	 {
	    BranchType	NodeType;	/* test type (see tree nodes) */
	    Attribute	Tested;		/* attribute tested */
	    int		Forks;		/* possible branches */
	    float	Cut;		/* threshold (if relevant) */
	    Set		Subset;		/* subset (if relevant) */
	    int		TestValue;	/* specified outcome of test */
	 }
	 CondRec, *Condition;


typedef  struct _rulerec
	 {
	    RuleNo	RNo;		/* rule number */
	    int		TNo,		/* trial number */
	    		Size;		/* number of conditions */
	    Condition	*Lhs;		/* conditions themselves */
	    ClassNo	Rhs;		/* class given by rule */
	    ItemCount	Cover;		/* number of cases covered by rule */
	    float	Error;		/* estimated error rate */
	    int		Vote;		/* unit = 0.001 */
	 }
	 RuleRec, *CRule;


typedef struct _rulesetrec
	 {
	    RuleNo	SNRules;	/* number of rules */
	    CRule	*SRule;		/* rules */
	    ClassNo	SDefault;	/* default class for this ruleset */
	 }
	 RuleSetRec, *CRuleSet;


/*************************************************************************/
/*									 */
/*		Function prototypes					 */
/*									 */
/*************************************************************************/

Boolean	    ReadName(FILE *f, String s);
void	    GetNames(FILE *Nf);
int	    Which(String Val, String *List, int First, int Last);
String	    CopyString(String S);
Description GetDescription(FILE *Df);
void	    GetMCosts(FILE *f);
int	    SelectMax(float [], int, int);

ClassNo	    TreeClassify(Description CaseDesc, Tree DecisionTree);
float	    Interpolate(Tree T, float Val);
ClassNo	    RuleClassify(Description CaseDesc, CRuleSet RS);
Boolean	    Matches(CRule R, Description Case);
ClassNo	    BoostClassify(Description CaseDesc, int MaxTrial, Boolean UseRules);
ClassNo	    SelectClass(ClassNo Default, float TotWeight, Boolean UseCosts);
void	    FindLeaf(Description CaseDesc, Tree T, Tree PT, float Wt);
Boolean	    Satisfies(Description CaseDesc, Condition OneCond);
Tree	    GetTree(String Extension);
Tree	    InTree();
void	    CheckFile(String Extension, Boolean Write);
void	    RecoverDiscreteNames();
void	    StreamIn(String S, int n);
void	    *Pmalloc(unsigned Bytes);
void	    *Prealloc(void *Present, unsigned Bytes);
void	    *Pcalloc(unsigned Number, unsigned Size);
double	    KRandom();
void	    Error(int ErrNo, String S1, String S2);
FILE *	    GetFile(String Extension, String RW);
CRuleSet    GetRules(String Extension);




//@@ global.c

/*************************************************************************/
/*									 */
/*		General data for C5.0					 */
/*		---------------------					 */
/*									 */
/*************************************************************************/


	Attribute	ClassAtt=0;	/* attribute to use as class */
	int		MaxAtt,		/* max att number */
			MaxClass;	/* max class number */
	DiscrValue	*MaxAttVal;	/* number of values for each att */
	char		*SpecialStatus;	/* special att treatment */
	String		*ClassName,	/* class names */
		  	*AttName,	/* att names */
		  	**AttValName;	/* att value names */
	float		Confidence,	/* set by classify() */
			*Vote=Nil,	/* total votes for classes */
			**MCost=Nil;	/* misclass cost [pred][real] */
	Tree		*Pruned=Nil;	/* decision trees */
	CRuleSet	*RuleSet;	/* rulesets */
	FILE		*TRf=0;		/* file pointer for tree and rule i/o */
	char		*FileName="?",	/* file stem */
			Fn[500];	/* file name with extension */


/*************************************************************************/
/*									 */
/*		Global parameters for C5.0				 */
/*		--------------------------				 */
/*									 */
/*************************************************************************/


	int		TRIALS=1;	/* number of classifiers */

	Boolean		BOOST=0,	/* true if boosting invoked */
			RULES=0,	/* true if use rulesets */
			PROBTHRESH=0;	/* true if to use soft thresholds */


/*************************************************************************/
/*									 */
/*	  Global data for C5.0 used for building decision trees		 */
/*	  -----------------------------------------------------		 */
/*									 */
/*************************************************************************/


int
	Trial;		/* trial number for boosting */

float
	*ClassSum;	/* class weights during classification */


/*************************************************************************/
/*									 */
/*	Global data for constructing and applying rules			 */
/*      -----------------------------------------------			 */
/*									 */
/*************************************************************************/


CRule	*Rule;		/* current rules */

RuleNo	NRules;		/* number of rules */

int	RuleSpace;	/* space currently allocated for rules */

// CRuleSet *RuleSet;	/* rulesets */   //@@ This is a redeclaration

ClassNo	Default;	/* default class associated with ruleset */




//@@ hooks.c

/*************************************************************************/
/*									 */
/*	Get names of classes, attributes and attribute values		 */
/*	-----------------------------------------------------		 */
/*									 */
/*************************************************************************/


#define  Space(s)	(s == ' ' || s == '\n' || s == '\t')
#define  SkipComment	while ( ( c = getc(f) ) != '\n' )

int	Delimiter;



/*************************************************************************/
/*									 */
/*	Read a name from file f into string s, setting Delimiter.	 */
/*									 */
/*	- Embedded periods are permitted, but periods followed by space	 */
/*	  characters act as delimiters.					 */
/*	- Embedded spaces are permitted, but multiple spaces are	 */
/*	  replaced by a single space.					 */
/*	- Any character can be escaped by '\'.				 */
/*	- The remainder of a line following '|' is ignored.		 */
/*									 */
/*************************************************************************/


Boolean ReadName(FILE *f, String s)
/*      --------  */
{
    register char *Sp=s;
    register int  c;

    /*  Skip to first non-space character  */

    while ( ( c = getc(f) ) == '|' || Space(c) )
    {
	if ( c == '|' ) SkipComment;
    }

    /*  Return false if no names to read  */

    if ( c == EOF )
    {
	Delimiter = EOF;
	return false;
    }

    /*  Read in characters up to the next delimiter  */

    while ( c != ':' && c != ',' && c != '\n' && c != '|' && c != EOF )
    {
	if ( c == '.' )
	{
	    if ( ( c = getc(f) ) == '|' || Space(c) ) break;
	    *Sp++ = '.';
	}

	if ( c == '\\' )
	{
	    c = getc(f);
	}

	*Sp++ = c;

	if ( c == ' ' )
	{
	    while ( ( c = getc(f) ) == ' ' )
		;
	}
	else
	{
	    c = getc(f);
	}
    }

    if ( c == '|' ) SkipComment;
    Delimiter = c;

    /*  Strip trailing spaces  */

    while ( Space(*(Sp-1)) ) Sp--;

    *Sp++ = '\0';
    return true;
}



/*************************************************************************/
/*									 */
/*	Read names of classes, attributes and legal attribute values.	 */
/*	On completion, names are stored in:				 */
/*	  ClassName	-	class names				 */
/*	  AttName	-	attribute names				 */
/*	  AttValName	-	attribute value names			 */
/*	with:								 */
/*	  MaxAttVal	-	number of values for each attribute	 */
/*									 */
/*	Other global variables set are:					 */
/*	  MaxAtt	-	maximum attribute number		 */
/*	  MaxClass	-	maximum class number			 */
/*									 */
/*	Note:  until the number of attributes is known, the name	 */
/*	       information is assembled in local arrays			 */
/*									 */
/*************************************************************************/


void GetNames(FILE *Nf)
/*   --------  */
{
    char	Buffer[1000];
    DiscrValue	v;
    int		AttCeiling=100, ClassCeiling=100, ValCeiling;

    /*  Get class names from names file  */

    ClassName = AllocZero(ClassCeiling, String);
    ClassName[0] = "?";
    MaxClass = 0;
    do
    {
	ReadName(Nf, Buffer);

	if ( ++MaxClass >= ClassCeiling)
	{
	    ClassCeiling += 100;
	    ClassName = Realloc(ClassName, ClassCeiling, String);
	}
	ClassName[MaxClass] = CopyString(Buffer);
    }
    while ( Delimiter == ',' );

    /*  Get attribute and attribute value names from names file  */

    AttName = AllocZero(AttCeiling, String);
    MaxAttVal = AllocZero(AttCeiling, DiscrValue);
    AttValName = AllocZero(AttCeiling, String *);
    SpecialStatus = Alloc(AttCeiling, char);

    MaxAtt = 0;
    while ( ReadName(Nf, Buffer) )
    {
	if ( Delimiter != ':' ) Error(1, Buffer, "");

	if ( ++MaxAtt >= AttCeiling )
	{
	    AttCeiling += 100;
	    AttName = Realloc(AttName, AttCeiling, String);
	    MaxAttVal = Realloc(MaxAttVal, AttCeiling, DiscrValue);
	    AttValName = Realloc(AttValName, AttCeiling, String *);
	    SpecialStatus = Realloc(SpecialStatus, AttCeiling, char);
	}

	AttName[MaxAtt] = CopyString(Buffer);
	SpecialStatus[MaxAtt] = Nil;
	MaxAttVal[MaxAtt] = 0;
	ValCeiling = 100;
	AttValName[MaxAtt] = AllocZero(ValCeiling, String);

	do
	{
	    if ( ! ( ReadName(Nf, Buffer) ) ) Error(2, AttName[MaxAtt], "");

	    if ( ++MaxAttVal[MaxAtt] >= ValCeiling )
	    {
		ValCeiling += 100;
		AttValName[MaxAtt] =
		    Realloc(AttValName[MaxAtt], ValCeiling, String);
	    }

	    AttValName[MaxAtt][MaxAttVal[MaxAtt]] = CopyString(Buffer);
	}
	while ( Delimiter == ',' );

	if ( MaxAttVal[MaxAtt] == 1 )
	{
	    /*  Check for special treatment  */

	    if ( ! strcmp(Buffer, "continuous") )
	    {}
	    else
	    if ( ! memcmp(Buffer, "discrete", 8) )
	    {
		SpecialStatus[MaxAtt] = DISCRETE;

		/*  Read max values, reserve space  */

		v = atoi(&Buffer[8]);
		if ( v < 2 )
		{
		    Error(10, AttName[MaxAtt], "");
		}

		AttValName[MaxAtt] = Realloc(AttValName[MaxAtt], v+2, String);
		AttValName[MaxAtt][0] = (char *) (int) v;
	    }
	    else
	    if ( ! strcmp(Buffer, "ignore") )
	    {
		SpecialStatus[MaxAtt] = IGNORE;
	    }
	    else
	    {
		/*  Cannot have only one discrete value for an attribute  */

		Error(3, AttName[MaxAtt], "");
	    }

	    MaxAttVal[MaxAtt] = 0;
	}
    }

    /*  Check whether class is one of the attributes  */

    if ( MaxClass == 1 )
    {
	ClassAtt = Which(ClassName[1], AttName, 1, MaxAtt);

	/*  Class attribute must be defined and must be discrete  */

	if ( ClassAtt <= 0 )
	{
	    Error(11, ClassName[1], "");
	}
	if ( SpecialStatus[ClassAtt] != DISCRETE && ! MaxAttVal[ClassAtt] )
	{
	    Error(12, ClassName[1], "");
	}

	MaxClass  = MaxAttVal[ClassAtt];
	ClassName = AttValName[ClassAtt];
	ClassName[0] = "?";
    }

    fclose(Nf);
}



/*************************************************************************/
/*									 */
/*	Locate value Val in List[First] to List[Last]			 */
/*									 */
/*************************************************************************/


int Which(String Val, String *List, int First, int Last)
/*  -----  */
{
    int	n=First;

    while ( n <= Last && strcmp(Val, List[n]) ) n++;

    return ( n <= Last ? n : First-1 );
}



/*************************************************************************/
/*									 */
/*	Allocate space then copy string into it				 */
/*									 */
/*************************************************************************/


String CopyString(String S)
/*     -----------  */
{
    char *p;

    p = Alloc(strlen(S)+1, char);
    strcpy(p, S);
    return p;
}


/*************************************************************************/
/*									 */
/*  Read a raw case description from file Df.				 */
/*									 */
/*  For each attribute, read the attribute value from the file.		 */
/*  If it is a discrete valued attribute, find the associated no.	 */
/*  of this attribute value (if the value is unknown this is 0).	 */
/*									 */
/*  Returns the Description of the case (i.e. the array of		 */
/*  attribute values).							 */
/*									 */
/*************************************************************************/


Description GetDescription(FILE *Df)
/*          --------------  */
{
    Attribute	Att;
    char	name[500], *endname;
    int		Dv;
    float	Cv;
    Description	Dummy, DVec;

    if ( ReadName(Df, name) )
    {
	Dummy = Alloc(MaxAtt+2, AttValue);
	DVec = &Dummy[1];

        ForEach(Att, 1, MaxAtt)
        {
	    if ( SpecialStatus[Att] == IGNORE )
	    {
		/*  Skip this value  */

		DVal(DVec, Att) = 0;
	    }
	    else
	    if ( Discrete(Att) || SpecialStatus[Att] == DISCRETE )
	    {
	        if ( ! ( strcmp(name, "?") ) )
		{
		    Dv = 0;
		}
	        else
	        {
		    Dv = Which(name, AttValName[Att], 1, MaxAttVal[Att]);
		    if ( ! Dv )
		    {
			if ( SpecialStatus[Att] == DISCRETE )
			{
			    /*  Add value to list  */

			    Dv = ++MaxAttVal[Att];
			    if ( Dv > (int) AttValName[Att][0] )
			    {
				Error(9, AttName[Att], (char *) AttValName[Att][0]);
			    }

			    AttValName[Att][Dv] = CopyString(name);
			}
			else
			{
			    Error(4, AttName[Att], name);
			}
		    }
	        }
	        DVal(DVec, Att) = Dv;
	    }
	    else
	    {
		/*  Continuous value  */

	        if ( ! ( strcmp(name, "?") ) )
		{
		    Cv = Unknown;
		}
	        else
		{
		    Cv = (float)(strtod(name, &endname));
		    if ( endname == name || *endname != '\0' )
			Error(4, AttName[Att], name);
		}
		CVal(DVec, Att) = Cv;
	    }

	    if ( ! ClassAtt || Att < MaxAtt ) ReadName(Df, name);
        }

	if ( ClassAtt )
	{
	    Class(DVec) = DVal(DVec, ClassAtt);
	}
	else
        if ( (Class(DVec) = Which(name, ClassName, 1, MaxClass)) < 0 )
        {
	    Class(DVec) = 0;
        }

	return DVec;
    }
    else
    {
	return Nil;
    }
}



/*************************************************************************/
/*									 */
/*	Retrieve entire decision tree with extension Extension		 */
/*									 */
/*************************************************************************/


Tree GetTree(String Extension)
/*   -------  */
{
    CheckFile(Extension, false);

    return InTree();
}



/*************************************************************************/
/*									 */
/*	Retrieve tree from saved characters				 */
/*									 */
/*************************************************************************/


Tree InTree()
/*   ------  */
{
    Tree	T;
    DiscrValue	v;
    int		Bytes;

    T = (Tree) AllocZero(1, TreeRec);

    StreamIn((char *) &T->NodeType, sizeof(BranchType));
    StreamIn((char *) &T->Leaf, sizeof(ClassNo));
    StreamIn((char *) &T->Items, sizeof(ItemCount));
    StreamIn((char *) &T->Errors, sizeof(ItemCount));

    T->ClassDist = AllocZero(MaxClass+1, ItemCount);
    StreamIn((char *) T->ClassDist, (MaxClass + 1) * sizeof(ItemCount));

    if ( T->NodeType )
    {
	StreamIn((char *) &T->Tested, sizeof(Attribute));
	StreamIn((char *) &T->Forks, sizeof(int));

	switch ( T->NodeType )
	{
	    case BrDiscr:
		break;

	    case BrThresh:
		StreamIn((char *) &T->Cut, sizeof(float));
		StreamIn((char *) &T->Lower, sizeof(float));
		StreamIn((char *) &T->Upper, sizeof(float));
		break;

	    case BrSubset:
		T->Subset = (Set *) AllocZero(T->Forks+1, Set);

		Bytes = (MaxAttVal[T->Tested]>>3) + 1;
		ForEach(v, 1, T->Forks)
		{
		    T->Subset[v] = Alloc(Bytes, unsigned char);
		    StreamIn((char *) T->Subset[v], Bytes);
		}
	}

	T->Branch = AllocZero(T->Forks+1, Tree);
	ForEach(v, 1, T->Forks)
	{
	    T->Branch[v] = InTree();
	}
    }

    return T;
}



/*************************************************************************/
/*									 */
/*	Check whether file is open.  If it is not, open it and		 */
/*	read/write discrete names					 */
/*									 */
/*************************************************************************/


void CheckFile(String Extension, Boolean Write)
/*   ---------  */
{
    static char	*LastExt="";

    if ( ! TRf || strcmp(LastExt, Extension) )
    {
	LastExt = Extension;

	if ( TRf )
	{
	    fprintf(TRf, "\n");
	    fclose(TRf);
	}

	Write = Write;  //@@ avoid never referenced warning

#ifdef WIN32
	if ( ! (TRf = GetFile(Extension, "rb")) ) Error(0, Fn, "");
#else
	if ( ! (TRf = GetFile(Extension, "r")) ) Error(0, Fn, "");
#endif

	StreamIn((char *) &TRIALS, sizeof(int));

	RecoverDiscreteNames();
    }
}



/*************************************************************************/
/*									 */
/*	Recover attribute values read with "discrete N"			 */
/*									 */
/*************************************************************************/


void RecoverDiscreteNames()
/*   --------------------  */
{
    Attribute	Att;
    DiscrValue	v;
    int		Length;

    ForEach(Att, 1, MaxAtt)
    {
	if ( SpecialStatus[Att] != DISCRETE ) continue;

	StreamIn((char *) &MaxAttVal[Att], sizeof(int));

	ForEach(v, 1, MaxAttVal[Att])
	{
	    StreamIn((char *) &Length, sizeof(int));

	    AttValName[Att][v] = Alloc(Length, char);
	    StreamIn(AttValName[Att][v], Length);
	}
    }
}



/*************************************************************************/
/*									 */
/*	Stream characters to/from file TRf from/to an address		 */
/*									 */
/*************************************************************************/


void StreamIn(String S, int n)
/*   --------  */
{
    while ( n-- ) *S++ = getc(TRf);
}



/*************************************************************************/
/*								  	 */
/*	Recover a ruleset						 */
/*								  	 */
/*************************************************************************/


CRuleSet GetRules(String Extension)
/*       --------  */
{
    int		ri, d, Bytes;
    CRuleSet	RS;
    CRule	R;
    Condition	C;

    CheckFile(Extension, false);
    
    RS = Alloc(1, RuleSetRec);

    StreamIn((char *) &RS->SNRules, sizeof(RuleNo));
    StreamIn((char *) &RS->SDefault, sizeof(ClassNo));

    RS->SRule = Alloc(RS->SNRules+1, CRule);

    ForEach(ri, 1, RS->SNRules)
    {
	R = RS->SRule[ri] = Alloc(1, RuleRec);

        StreamIn((char *) &R->RNo, sizeof(int));
        StreamIn((char *) &R->TNo, sizeof(int));
        StreamIn((char *) &R->Size, sizeof(int));

	R->Lhs = Alloc(R->Size+1, Condition);
	ForEach(d, 1, R->Size)
	{
	    C = R->Lhs[d] = Alloc(1, CondRec);

	    StreamIn((char *) &C->NodeType, sizeof(BranchType));
	    StreamIn((char *) &C->Tested, sizeof(Attribute));
	    StreamIn((char *) &C->Forks, sizeof(int));
	    StreamIn((char *) &C->Cut, sizeof(float));
	    if ( C->NodeType == BrSubset )
	    {
		Bytes = (MaxAttVal[C->Tested]>>3) + 1;
		C->Subset = Alloc(Bytes, unsigned char);
		StreamIn((char *) C->Subset, Bytes);
	    }
	    StreamIn((char *) &R->Lhs[d]->TestValue, sizeof(int));
	}
	StreamIn((char *) &R->Rhs, sizeof(ClassNo));
	StreamIn((char *) &R->Cover, sizeof(ItemCount));
	StreamIn((char *) &R->Error, sizeof(float));
	StreamIn((char *) &R->Vote, sizeof(int));
    }

    return RS;
}



/*************************************************************************/
/*                                                              	 */
/*	Categorize a case description using the given decision tree	 */
/*                                                              	 */
/*************************************************************************/


ClassNo TreeClassify(Description CaseDesc, Tree DecisionTree)
/*      ------------  */
{
    ClassNo	c;
    float	Tot=0;

    ForEach(c, 0, MaxClass)
    {
	ClassSum[c] = 0;
    }

    FindLeaf(CaseDesc, DecisionTree, Nil, 1.0);

    ForEach(c, 0, MaxClass)
    {
	Tot += ClassSum[c];
    }

    return SelectClass(1, Tot, MCost != Nil);
}



/*************************************************************************/
/*                                                              	 */
/*	Classify a case description using the given subtree by		 */
/*	adjusting the value ClassSum for each class			 */
/*                                                              	 */
/*************************************************************************/


void FindLeaf(Description CaseDesc, Tree T, Tree PT, float Fraction)
/*   --------  */
{
    DiscrValue	v, Dv;
    ClassNo	c;
    float	Cv;

    switch ( T->NodeType )
    {
        case 0:  /* leaf */

	    if ( T->Items < Epsilon )
	    {
		T = PT;		/* use parent node instead */
	    }

	    /*  Update from ALL classes  */

	    ForEach(c, 1, MaxClass)
	    {
		if ( T->ClassDist[c] )
		{
		    ClassSum[c] += Fraction * (T->ClassDist[c] + 1) /
				   (T->Items + MaxClass);
		}
	    }

	    return;

	case BrDiscr:  /* test of discrete attribute */

	    Dv = DVal(CaseDesc, T->Tested);

	    if ( Dv && Dv <= T->Forks )	/*  Make sure not new discrete value  */
	    {
		FindLeaf(CaseDesc, T->Branch[Dv], T, Fraction);
	    }
	    else
	    {
		ForEach(v, 1, T->Forks)
		{
		    FindLeaf(CaseDesc, T->Branch[v], T,
			     (Fraction * T->Branch[v]->Items) / T->Items);
		}
	    }

	    return;

	case BrThresh:  /* test of continuous attribute */

	    Cv = CVal(CaseDesc, T->Tested);

	    if ( Cv == Unknown )
	    {
		ForEach(v, 1, 2)
		{
		    FindLeaf(CaseDesc, T->Branch[v], T,
			     (Fraction * T->Branch[v]->Items) / T->Items);
		}
	    }
	    else
	    {
		v = ( Cv <= T->Cut ? 1 : 2 );
		FindLeaf(CaseDesc, T->Branch[v], T, Fraction);
	    }

	    return;

	case BrSubset:  /* subset test on discrete attribute  */

	    Dv = DVal(CaseDesc, T->Tested);

	    if ( Dv )
	    {
		ForEach(v, 1, T->Forks)
		{
		    if ( In(Dv, T->Subset[v]) )
		    {
			FindLeaf(CaseDesc, T->Branch[v], T, Fraction);

			return;
		    }
		}
	    }

	    /*  Value unknown or not found in any of the subsets  */

	    ForEach(v, 1, T->Forks)
	    {
		FindLeaf(CaseDesc, T->Branch[v], T,
		         (Fraction * T->Branch[v]->Items) / T->Items);
	    }

	    return;
    }
}



/*************************************************************************/
/*                                                              	 */
/*	Categorize a case description using a ruleset			 */
/*                                                              	 */
/*************************************************************************/


ClassNo RuleClassify(Description CaseDesc, CRuleSet RS)
/*      ------------  */
{
    ClassNo	c;
    float	TotWeight=2.0;
    int		r;
    CRule	R;

    ForEach(c, 1, MaxClass)
    {
	ClassSum[c] = 0;
    }
    ClassSum[0] = ClassSum[RS->SDefault] = 1.0;

    ForEach(r, 1, RS->SNRules)
    {
	R = RS->SRule[r];

	if ( Matches(R, CaseDesc) )
	{
	    ClassSum[R->Rhs] += R->Vote;
	    TotWeight += 1000.0;
	}
    }

    return SelectClass(RS->SDefault, TotWeight, MCost != Nil);
}



/*************************************************************************/
/*									 */
/*	Determine whether the given case description satisfies the	 */
/*	given condition							 */
/*									 */
/*************************************************************************/


Boolean Satisfies(Description CaseDesc, Condition OneCond)
/*      ---------  */
{
    DiscrValue	v;
    float	cv;
    Boolean	Outcome;

    /*  Determine the outcome of this test on this item  */

    switch ( OneCond->NodeType )
    {
	case BrDiscr:  /* test of discrete attribute */

	    v = DVal(CaseDesc, OneCond->Tested);
	    Outcome = ( v == 0 ? -1 : v );
	    break;

	case BrThresh:  /* test of continuous attribute */

	    cv = CVal(CaseDesc, OneCond->Tested);
	    Outcome = ( cv == Unknown ? -1 : cv <= OneCond->Cut ? 1 : 2 );
	    break;

	case BrSubset:  /* subset test on discrete attribute  */

	    v = DVal(CaseDesc, OneCond->Tested);
	    Outcome = ( In(v, OneCond->Subset) ? OneCond->TestValue : 0 );
    }

    return ( Outcome == OneCond->TestValue );
}



/*************************************************************************/
/*								         */
/*	Determine whether a case satisfies all conditions of a rule	 */
/*								         */
/*************************************************************************/


Boolean Matches(CRule R, Description Case)
/*      -------  */
{
    int d;

    ForEach(d, 1, R->Size)
    {
	if ( ! Satisfies(Case, R->Lhs[d]) )
	{
	    return false;
	}
    }

    return true;
}



/*************************************************************************/
/*									 */
/*	Classify a case using boosted tree or rule sequence		 */
/*									 */
/*************************************************************************/


ClassNo BoostClassify(Description CaseDesc, int MaxTrial, Boolean UseRules)
/*	-------------  */
{
    ClassNo	c, Best;
    int		t;
    
    ForEach(c, 0, MaxClass)
    {
	Vote[c] = 0;
    }

    ForEach(t, 0, MaxTrial)
    {
	Best = ( UseRules ? RuleClassify(CaseDesc, RuleSet[t]) :
			    TreeClassify(CaseDesc, Pruned[t]) );
	Vote[Best] += Confidence;
    }

    /*  Copy votes into ClassSum  */

    ForEach(c, 0, MaxClass)
    {
	ClassSum[c] = Vote[c];
    }

    return SelectClass(int(KRandom() * MaxClass + 1), 1.0, MCost != Nil);
    //@@ the int cast is needed for g++
}



/*************************************************************************/
/*									 */
/*	Select the best class to return.  Take misclassification costs	 */
/*	into account if they are defined.				 */
/*									 */
/*************************************************************************/


ClassNo SelectClass(ClassNo Default, float TotWeight, Boolean UseCosts)
/*      -----------  */
{
    ClassNo	c, cc, BestClass;
    float	ExpCost, BestCost=1E38F;

    BestClass = Default;
    ForEach(c, 1, MaxClass)
    {
	if ( UseCosts )
	{
	    ExpCost = 0;
	    ForEach(cc, 1, MaxClass)
	    {
		if ( cc == c ) continue;
		ExpCost += ClassSum[cc] * MCost[c][cc];
	    }

	    if ( ExpCost < BestCost )
	    {
		BestClass = c;
		BestCost  = ExpCost;
	    }
	}
	else
	{
	    if ( ClassSum[c] > ClassSum[BestClass] ) BestClass = c;
	}
    }

    Confidence = ClassSum[BestClass] / TotWeight;

    return BestClass;
}



/*************************************************************************/
/*								   	 */
/*	Interpolate a single value between Lower, Cut and Upper		 */
/*								   	 */
/*************************************************************************/


float Interpolate(Tree T, float Val)
/*    -----------  */
{
    return ( PROBTHRESH != 2 ? (float) (Val <= T->Cut) :
	     Val <= T->Lower ? 1.0F :
	     Val > T->Upper ? 0.0F :
	     Val <= T->Cut ?
		(float)(1 - 0.5 * (Val - T->Lower)
			/ (T->Cut - T->Lower + 1E-6)) :
		(float)(0.5 - 0.5 * (Val - T->Cut)
			/ (T->Upper - T->Cut + 1E-6) ));
}



/*************************************************************************/
/*									 */
/*	Read variable misclassification costs				 */
/*									 */
/*************************************************************************/


void GetMCosts(FILE *Cf)
/*   ---------  */
{
    ClassNo	Pred, Real;
    char	Name[256];
    int		LineNo=0;
    float	Val;

    /*  Set up cost matrices  */

    MCost = Alloc(MaxClass+1, float *);
    ForEach(Pred, 1, MaxClass)
    {
	MCost[Pred] = Alloc(MaxClass+1, float);
	ForEach(Real, 1, MaxClass)
	{
	    MCost[Pred][Real] = ( Pred == Real ? 0.0F : 1.0F );
	}
    }

    /*  Read entries from cost file  */

    while ( ReadName(Cf, Name) )
    {
	LineNo++;
	if ( ! (Pred = Which(Name, ClassName, 1, MaxClass)) )
	{
	    Error(6, Name, (String) LineNo);
	}

	if ( ! ReadName(Cf, Name) ||
	     ! (Real = Which(Name, ClassName, 1, MaxClass)) )
	{
	    Error(6, Name, (String) LineNo);
	}

	if ( fscanf(Cf, "%f", &Val) != 1 || Val < 0 )
	{
	    Error(7, "", (String) LineNo);
	}

	MCost[Pred][Real] = Val;
    }

    fclose(Cf);
}



/*************************************************************************/
/*									 */
/*	This is a specialised form of the getopt utility.		 */
/*									 */
/*************************************************************************/


String	OptArg, Option;


char ProcessOption(int Argc, char *Argv[], char *Options)
/*   -------------  */
{
    int		i;
    static int	OptNo=1;

    if ( OptNo >= Argc ) return '\00';

    if ( *(Option = Argv[OptNo++]) != '-' ) return '?';

    for ( i = 0 ; Options[i] ; i++ )
    {
	if ( Options[i] == Option[1] )
	{
	    OptArg = ( Options[i+1] != '+' ? Nil :
		       Option[2] ? Option+2 :
		       OptNo < Argc ? Argv[OptNo++] : "0" );
	    return Option[1];
	}
    }

    return '?';
}



/*************************************************************************/
/*									 */
/*	Protected memory allocation routines				 */
/*									 */
/*************************************************************************/



void *Pmalloc(unsigned Bytes)
/*    -------  */
{
    void *p=Nil;

    if ( ! Bytes || (p = (void *) malloc(Bytes)) )
    {
	return p;
    }

    Error(8, "", "");

    return Nil;
}



void *Prealloc(void *Present, unsigned Bytes)
/*    --------  */
{
    void *p=Nil;

    if ( ! Bytes ) return Nil;

    if ( ! Present ) return Pmalloc(Bytes);

    if ( (p = (void *) realloc(Present, Bytes)) )
    {
	return p;
    }

    Error(8, "", "");

    return Nil;
}



void *Pcalloc(unsigned Number, unsigned Size)
/*    -------  */
{
    void *p;

    if ( ! Number || (p = (void *) calloc(Number, Size)) )
    {
	return p;
    }

    Error(8, "", "");

    return Nil;
}



/*************************************************************************/
/*									 */
/*	Generate uniform random numbers					 */
/*									 */
/*************************************************************************/


#define	Modify(F,S)	if ( (F -= S) < 0 ) F += 1.0


double KRandom()
/*     -------  */
{
    static double	URD[55];
    static int		Fp=0, Sp=0;
    double		V1, V2;
    int			i, j;

    /*  Initialisation  */

    if ( Fp == Sp )
    {
	Fp = 0;
	Sp = 31;

	V1 = 1.0;
	V2 = 0.314159285;

	ForEach(i, 1, 55)
	{
	    URD[ j = (i * 21) % 55 ] = V1;
	    V1 = V2 - V1;
	    if ( V1 < 0 ) V1 += 1.0;
	    V2 = URD[j];
	}

	ForEach(j, 0, 5)
	{
	    ForEach(i, 0, 54)
	    {
		Modify(URD[i], URD[(i+30) % 55]);
	    }
	}
    }

    Fp = (Fp + 1) % 55;
    Sp = (Sp + 1) % 55;
    Modify(URD[Fp], URD[Sp]);

    return URD[Fp];
}



/*************************************************************************/
/*									 */
/*	Error messages							 */
/*									 */
/*************************************************************************/


void Error(int ErrNo, String S1, String S2)
/*   -----  */
{
    static char Messages=0;
    Boolean	Quit=false;

    printf("\nERROR:  ");
    switch ( ErrNo )
    {
	case 0:
	    printf("cannot open file %s%s\n", S1, S2);
	    Quit = true;
	    break;

	case 1:
	    printf("colon expected after attribute name %s\n", S1);
	    break;

	case 2:
	    printf("unexpected eof while reading attribute %s\n", S1);
	    break;

	case 3:
	    printf("attribute %s has only one value\n", S1);
	    break;

	case 4:
	    printf("illegal value '%s' for attribute %s\n", S2, S1);
	    break;

	case 5:
	    printf("illegal class '%s'\n", S2);
	    break;

	case 6:
	    printf("line %d of cost file: illegal class %s\n",
		    (int) S2, S1);
	    break;

	case 7:
	    printf("line %d of cost file: illegal cost\n", (int) S2);
	    break;

	case 8:
	    printf("unable to allocate sufficient memory\n");
	    Quit = true;
	    break;

	case 9:
	    printf("too many values for %s (max %d)\n", S1, (int) S2);
	    Quit = true;
	    break;

	case 10:
	    printf("illegal number of discrete values for attribute %s\n", S1);
	    Quit = true;
	    break;

	case 11:
	    printf("target attribute %s not found\n", S1);
	    Quit = true;
	    break;

	case 12:
	    printf("target attribute %s is not discrete\n", S1);
	    Quit = true;
	    break;
    }

    if ( ++Messages > 10 )
    {
	printf("Error limit exceeded\n");
	Quit = true;
    }

    if ( Quit )
    {
	exit(1);
    }
}



/*************************************************************************/
/*									 */
/*	Open file with given extension for read/write			 */
/*									 */
/*************************************************************************/


FILE *GetFile(String Extension, String RW)
/*    --------  */
{
    strcpy(Fn, FileName);
    strcat(Fn, Extension);
    return fopen(Fn, RW);
}
