// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _C50Inducer_h
#define _C50Inducer_h 1

#include <Inducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <DecisionTree.h>
#include <TmpFileName.h>
#include <LogOptions.h>
#include <CatTestResult.h>

// The c50Pgm parameter should be set to the command that will execute C5.0.
CatTestResult *run_c50(const LogOptions& logOptions,
		       const MString& c50Pgm,
		       const InstanceList& trainingSet,
		       const InstanceList& testingSet,
		       int minCases = 2,
		       int pruningConfidence = 25,
		       Bool rules = FALSE,
		       Bool subsetting = FALSE,
		       Bool probThresholds = FALSE,
		       Bool boosting = FALSE,
		       int boostingTrials = 10);
		       
class C50Inducer : public BaseInducer {
   // Member data (also see public data)
   MString pgmName;
   int minCases;
   int pruningConfidence;
   Bool rules;
   Bool subsetting;
   Bool probThresholds;
   Bool boosting;
   int boostingTrials;
public:
   // Public data
   static const MString defaultPgmName;
   // Methods
   C50Inducer(const MString& description,
	      int minNumCases = 2,
	      int pruningConf = 25,
	      Bool useRules = FALSE,
	      Bool subset = FALSE,
	      Bool probThres = FALSE,
	      Bool boost = FALSE,
	      int tials = 10,
	      const MString& aPgmName = defaultPgmName);
   virtual ~C50Inducer();
   virtual int class_id() const { return C50_INDUCER; }
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testSet);
   virtual CatTestResult *train_and_perf(InstanceList* trainingSet,
					 const InstanceList& testSet);
   virtual Bool supports_full_testing() { return TRUE; }
};


#endif








