// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _CARTInducer_h
#define _CARTInducer_h 1 

#include <BaseInducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <DecisionTree.h>
#include <TmpFileName.h>
#include <LogOptions.h>
#include <CatTestResult.h>

CatTestResult* run_cart(const LogOptions& logOptions,
			InstanceList& trainingSet,
			const InstanceList& testingSet,
			Bool prune, const MString& settings);

class CARTInducer : public BaseInducer {
   NO_DEFAULT_OPS(CARTInducer);

   // Member data (also see public data)
   Bool prune;
   Bool classProbabilityTree;
   MString options;
public:
   // Public data
   static MString defaultOptions;
   static Bool defaultPrune;

   // Methods
   CARTInducer(const MString& description, 
	       Bool pruning = defaultPrune,
	       const MString& opts = defaultOptions);
   virtual ~CARTInducer();
   virtual int class_id() const { return CART_INDUCER; }
   virtual Real train_and_test(InstanceList* trainingSet,
		 	       const InstanceList& testSet);
   virtual CatTestResult* train_and_perf(InstanceList* trainingSet,
					 const InstanceList& testSet);
   virtual Bool supports_full_testing() { return TRUE; }
   virtual void set_options(const MString &newOptions) {
      options = newOptions; }
};
   
#endif








