// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _CrossValidator_h
#define _CrossValidator_h 1

#include <PerfEstimator.h>

class BaseInducer;
class LabelledInstanceList;
class CatTestResult;

class CrossValidator : public PerfEstimator {
   NO_DEFAULT_OPS(CrossValidator);
   
   // Member data (also see protected and public data)
   int numFolds;
   int numTimes;
protected:
   // Protected member data
   Real fraction;
   // Protected methods
   virtual Real estimate_time_performance(BaseInducer& inducer,
			     InstanceList& dataList, int time, int folds);
public:
   // Public data
   static const int  defaultNumFolds;
   static const int  defaultNumTimes;
   static const int  defaultMaxFolds;        // for auto_set_folds
   static const int  defaultAutoFoldTimes;   // for auto_set_folds
   static const Real defaultStdDevEpsilon;   // for auto_set_folds
   static const Real defaultAccEpsilon;      // for auto_set_folds
   static const int  defaultMaxTimes;        // for auto_estimate
   static const Real defaultAutoStdDev;      // for auto_estimate

   // Methods
   CrossValidator(int nFolds = CrossValidator::defaultNumFolds,
		  int nTimes = CrossValidator::defaultNumTimes);
   virtual ~CrossValidator();
   void set_folds(int num); // negative -> leave num out
   void set_times(int num);
   void set_fraction(Real fract);
   int get_folds() const {return numFolds;}
   int get_times() const  {ASSERT(numTimes > 0); return numTimes;}
   virtual MString description() const;
   virtual Real estimate_performance(BaseInducer& inducer, 
				     InstanceList& dataList);
   // Estimate using dumped files.
   virtual Real estimate_performance(BaseInducer& inducer,
				     const MString& filestem);
   virtual Real auto_estimate_performance(BaseInducer& inducer,
					  InstanceList& dataList,
			     Real desiredStdDev = defaultAutoStdDev,
			     int  maxTimes   = defaultMaxTimes);
   // Auto-settings attempts to set numFolds by an ad-hoc to a reasonable
   //   number such that the variance resulting from the training-set size is
   //   not too big.
   virtual void auto_set_folds(BaseInducer& inducer, InstanceList& dataList,
			       int maxFolds = defaultMaxFolds,
			       int maxTimes = defaultMaxTimes,
			       Real AccEpsilon  = defaultAccEpsilon,
			       Real stdDevEpsilon = defaultStdDevEpsilon);
};

#endif






