// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _CartSplit_h
#define _CartSplit_h 1

#include <PartitionSplit.h>

class ValueProbability {
   NO_COPY_CTOR(ValueProbability);

   // Member data
   Real p;
   NominalVal value;

   // Private methods
   ValueProbability& operator=(const ValueProbability& rhs);

   friend class CartSplit;
   friend Bool operator<(const ValueProbability& lhs,
			 const ValueProbability& rhs);
   friend Bool operator==(const ValueProbability& lhs,
			  const ValueProbability& rhs);
   friend Bool operator!=(const ValueProbability& lhs,
			  const ValueProbability& rhs)
      {return ! (operator==(lhs, rhs));}
public:   // needs to be public for the template instantiator
   ValueProbability() : p(0.0), value(0) { }
};

// Derivation is virtual for CartSplit
class CartSplit : virtual public PartitionSplit {
   NO_DEFAULT_OPS(CartSplit);

   // Private methods
   Real evaluate_division(PartitionEvaluator& evaluator,
			  const Array<ValueProbability>& probs,
			  int splitIndex) const;
   void construct_split(const Array<ValueProbability>& probabilities,
			int bestLastIndex);
public:
   CartSplit() : PartitionSplit() { }
   ~CartSplit() { }
   void subset_split(PartitionEvaluator& evaluator);
};

#endif
