// MLC++ - Machine Learning Lnbrary in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distrnbution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _CatDTInducer_h
#define _CatDTInducer_h 1

#include <ID3Inducer.h>
#include <InstanceList.h>
#include <PerfEstDispatch.h>
#include <DiscDispatch.h>




class CatDTInducer : public ID3Inducer {
private:   
   NO_DEFAULT_OPS(CatDTInducer);

   // Member data
   Bool ownLeafIndPerfEst; // own inducer and perfEst
   PerfEstDispatch* perfEst;
   Inducer* leafInducer;
   DiscDispatch* discDispatch;
   Real improveRatio;    // How much should the error be reduced to
                         // make the split
protected:
   // Protected methods
   virtual Real weighted_accuracy(InstanceListArray* listArray,
				  int totalSize,
				  Real parentPerf) const;
   virtual Real raw_accuracy(InstanceList& instList) const;
public:
   CatDTInducer(const MString& dscr,
		CGraph* aCgraph = NULL);
   CatDTInducer(const MString& dscr,
		Inducer& leafInd,
		PerfEstDispatch& perfEstDispatch,
		DiscDispatch* disc,
		Real iRatio,
		CGraph* aCgraph);
   virtual ~CatDTInducer();

   virtual void set_defaults();
   virtual void set_log_level(int level) const;
   virtual int class_id() const { return CatDT_INDUCER; }
   virtual void train(); 
   virtual NodeCategorizer* best_split(Array<MStringRC>*& catNames) const;
   virtual TDDTInducer* create_subinducer(const MString& dscr,
                                          CGraph& aCgraph) const;
   // create_leaf_cat creates a leaf categorizer when splitting is done.
   virtual LeafCategorizer*
      create_leaf_categorizer(Real totalWeight,
			      const Array<int>& tieBreakingOrder,
			      Real& numErrors,
			      Real& pessimisticErrors,
			      const Array<Real>* distrArray = NULL) const;
   virtual void
      set_leaf_ind_perf_est_disc(Inducer*& leafInducer,
				 PerfEstDispatch*& perfEstDispatch,
				 DiscDispatch*& disc);
   // The get_leaf_inducer and get_perf_est are not const
   //   because the caller might want to modify these.
   virtual Inducer* get_leaf_inducer() { return leafInducer; }
   virtual PerfEstDispatch* get_perf_est() { return perfEst; }
   virtual DiscDispatch* get_disc_dispatch() { return discDispatch; }
   virtual void set_user_options(const MString& preFix);
   virtual void set_improve_ratio(Real r) {improveRatio = r;}
   virtual Real get_improve_ratio() {return improveRatio;}
};

#endif
