// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef CENTROID1DNOMINAL_H_
#define CENTROID1DNOMINAL_H_ 1

#include <MString.h>
#include <Centroid1D.h>
#include <DistribDistFunctor.h>


/* A little struct used to initialize the nominal centroids,
 utilizing the Array sort function.  The struct must be defined
 here to MLTemplates can have access to it. */
struct intAndNominal
{
   Real count;  //@@ This should be weight?
   NominalVal nomVal;
   
   Bool operator< (const intAndNominal& rhs) const
   { return(count < rhs.count); }
   Bool operator== (const intAndNominal& rhs) const
   { return(count==rhs.count); }
};


class PerCentroid1DNominalDistr_ : public PerCentroid1D_
{
public:
   DECLARE_CENTROID_HELPER(CENTROID_1D_NOMINAL_DISTR_ID,
			   Centroid1DNominalDistr,
			   PerCentroid1DNominalDistr_)
   Array<int> *counts;
   Array<Real> *distrib;
   Array<Real> *unknownVal;
   int initVal;   
   //@@ Need to handle distance functor correctly

   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};
   
class Centroid1DNominalDistr : public Centroid1D
{
   Array<int> counts;
   Array<Real> distrib;
   Array<Real> unknownVal;  // default position given to instances
                            // with this attribute unknown
   NominalAttrValue_ initVal;  // default position when there are no instances
                               // associated with this centroid
   Array<Real>* attrCounts;
   DistribDistFunctor* distanceFunctor;

   NO_COPY_CTOR(Centroid1DNominalDistr);
protected:
   virtual void update(const InstanceRC& instance, Real weight);

   virtual PtrArray<Centroid*>* no_initialization( int numCents );
   virtual PtrArray<Centroid*>* uniform_initialization( int numCents );
   virtual PtrArray<Centroid*>* random_initialization( int numCents );

public:
   DECLARE_CENTROID(CENTROID_1D_NOMINAL_DISTR_ID,
		    Centroid1DNominalDistr,
		    PerCentroid1DNominalDistr_)   
   virtual void OK(int level=0) const;

   Centroid1DNominalDistr(const SchemaRC& schema, int attrNum,
			  DistribDistFunctor* distFunctor ); 
   Centroid1DNominalDistr(const Centroid1DNominalDistr& other, CtorDummy); 
   Centroid1DNominalDistr(const SchemaRC& schema,
			  PerCentroid1DNominalDistr_& helper);
   virtual void fill_helper(PersistentHelper_& helper) const;

   virtual Centroid* clone() const;

   virtual ~Centroid1DNominalDistr() { DBG(OK()); }
   virtual Bool operator==(const Centroid& rhs) const;
   Bool operator==( const Centroid1DNominalDistr& rhs) const;

   virtual Bool can_cast_to_1DNominal() const { return TRUE; }
   virtual Centroid1DNominalDistr* cast_to_1DNominal() { return this; }
   virtual const Centroid1DNominalDistr& cast_to_1DNominal() const { return *this; }

   virtual void set_distance_functor( DistribDistFunctor*& newFunctor );

   virtual AttrValue_ attr_val() const;
   virtual const NominalAttrInfo& nominal_attr_info() const {
      return attr_info().cast_to_nominal(); }
   
   virtual NominalVal nominal_val() const;
   virtual NominalVal init_val() const;
   virtual const Array<Real>& unknown_val() const;

   virtual const MString& nominal_str() const;

   virtual void reset();
   virtual void set_init_val( NominalVal newInit );
   virtual void set_unknown_val( const Array<Real>& unknown );

   virtual Bool is_initialized() const {
      return(!attr_info().is_unknown(initVal) ); }

   virtual void display_short(MLCOStream& stream = Mcout) const;
   virtual void display(MLCOStream& stream = Mcout) const; 

   virtual const Array<int>& get_counts() const; 
   virtual const Array<Real>& get_distribution() const;

   virtual PtrArray<Centroid*>* init_centroids( int numCentroids, 
						const InstanceList& instances);

   virtual Real distance(const InstanceRC& inst) const;
   virtual Real distance(const Centroid& other) const;
};                   
DECLARE_DISPLAY(Centroid1DNominalDistr)


class PerCentroid1DNominalMode_ : public PerCentroid1DNominalDistr_
{
public:
   DECLARE_CENTROID_HELPER(CENTROID_1D_NOMINAL_MODE_ID,
			   Centroid1DNominalMode,
			   PerCentroid1DNominalMode_)
   // no new data, so no new read and write
};
   
class Centroid1DNominalMode : public Centroid1DNominalDistr
{
   NO_COPY_CTOR(Centroid1DNominalMode);
public:
   DECLARE_CENTROID(CENTROID_1D_NOMINAL_MODE_ID,
		    Centroid1DNominalMode,
		    PerCentroid1DNominalMode_)

   Centroid1DNominalMode(const SchemaRC& schema, int attrNum);
   Centroid1DNominalMode( const Centroid1DNominalMode& other, CtorDummy );
   Centroid1DNominalMode(const SchemaRC& schema,
			 PerCentroid1DNominalMode_& helper);
   // no new persistent data, so fill_helper is not declared

   virtual Centroid* clone() const;

   virtual ~Centroid1DNominalMode() { DBG(OK()); }

   virtual void display_short(MLCOStream& stream = Mcout) const;
   virtual void display(MLCOStream& stream = Mcout) const;

   virtual Real distance(const InstanceRC& instance) const;
   virtual Real distance(const Centroid& other) const;

};
DECLARE_DISPLAY(Centroid1DNominalMode)


#endif
