// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef CENTROID1DREAL_H_
#define CENTROID1DREAL_H_ 1

#include <MString.h>
#include <Centroid1D.h>

class PerCentroid1DReal_ : public PerCentroid1D_
{
public:
   DECLARE_CENTROID_HELPER(CENTROID_1D_REAL_ID, Centroid1DReal,
			   PerCentroid1DReal_)
   AttrValue_ attrVal;
   AttrValue_ initVal;
   AttrValue_ unknownVal;
   int normMethod;

   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};
   
class Centroid1DReal : public Centroid1D
{
private:
   NO_COPY_CTOR(Centroid1DReal);

   // Member data
   AttrValue_ attrVal;
   AttrValue_ initVal;  // default position when there are no instances
                        // associated with this centroid
   AttrValue_ unknownVal;  // default position given to instances
                           // with this attribute unknown

   InstanceList::NormalizationMethod normMethod;
  
protected:
   // Protected methods
   virtual void update(const InstanceRC& instance, Real weight);

   virtual PtrArray<Centroid*>* no_initialization( int numCents );
   virtual PtrArray<Centroid*>* uniform_initialization( int numCents );
   virtual PtrArray<Centroid*>* random_initialization( int numCents );

public:
   DECLARE_CENTROID(CENTROID_1D_REAL_ID, Centroid1DReal,
			   PerCentroid1DReal_)
   virtual void OK(int level=0) const;

   Centroid1DReal(const SchemaRC& schema, int attrNum);
   Centroid1DReal(const Centroid1DReal& other, CtorDummy);
   Centroid1DReal(const SchemaRC& schema, PerCentroid1DReal_& helper);
   virtual void fill_helper(PersistentHelper_& helper) const;
   virtual Centroid* clone() const;

   virtual ~Centroid1DReal() { DBG(OK()); }
   virtual Bool operator==( const Centroid& rhs) const;
   Bool operator==( const Centroid1DReal& rhs) const;

   virtual Bool can_cast_to_1DReal() const { return TRUE; }
   virtual Centroid1DReal* cast_to_1DReal() { return this; }
   virtual const Centroid1DReal& cast_to_1DReal() const { return *this; }

   virtual AttrValue_ attr_val() const;
   virtual const RealAttrInfo& real_attr_info() const {
      return attr_info().cast_to_real(); }
   
   virtual Real real_val() const;
   virtual Real init_val() const;
   virtual Real unknown_val() const;

   virtual Real normalized_val() const;
   virtual Real normalized_init_val() const;
   virtual Real normalized_unknown_val() const;

   virtual void reset( );
   virtual void set_init_val( Real newInit );
   virtual void set_unknown_val( Real newUnknown );

   virtual Bool is_initialized() const
     { return(!attr_info().is_unknown(initVal)); }

   virtual void display_short(MLCOStream& stream = Mcout) const;
   virtual void display(MLCOStream& stream = Mcout) const;

   virtual PtrArray<Centroid*>* init_centroids( int numCentroids, 
						const InstanceList& il );

   virtual Real distance(const InstanceRC& inst) const;
   virtual Real distance(const Centroid& other) const;
};
DECLARE_DISPLAY(Centroid1DReal)


#endif
