// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _CentroidMultiD_h
#define _CentroidMultiD_h 1

#include <Centroid.h>
#include <Centroid1D.h>
#include <DistribDistFunctor.h>

class PerCentroidMultiD_ : public PerCentroid_ {
public:
   DECLARE_CENTROID_HELPER(CENTROID_MULTID_ID, CentroidMultiD,
			   PerCentroidMultiD_)
   Array<Real> *attrWeights;
   Array<PersistentHelper_ *> *attrCentroids;
   Real sumOfWeights;
   int nominalType;
   Bool manualWeights;
   //@@ need to handle distance functor
   
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};
   
class CentroidMultiD : public Centroid
{
private:
   NO_COPY_CTOR(CentroidMultiD);

   // Member data
   Array<Real> attrWeights; 
   PtrArray<Centroid*> attrCentroids;
   Real sumOfWeights;

   // User options
   NominalType nominalType;
   Bool manualWeights;

   // Distance functor to use for nominal attributes
   DistribDistFunctor* nominalDistFunctor;

protected:
   // Protected methods
   virtual const PtrArray<Centroid*>& attr_centroids() const
      { return attrCentroids; }
   virtual const Centroid& attr_centroid(int i) const
      { return *(attrCentroids[i]); }

   virtual void update(const InstanceRC& instance, Real weight);
   virtual void display_short(MLCOStream& stream = Mcout) const
      { display(stream); }

public:
   DECLARE_CENTROID(CENTROID_MULTID_ID, CentroidMultiD, PerCentroidMultiD_)
   virtual void OK( int /*level */ = 0 ) const;

   CentroidMultiD(const SchemaRC& schema, DistribDistFunctor* nomDistFunct);
   CentroidMultiD(const CentroidMultiD& other, CtorDummy);
   CentroidMultiD(const SchemaRC& schema, PerCentroidMultiD_& helper);
   virtual void fill_helper(PersistentHelper_& helper) const;

   virtual Centroid* clone() const;

   virtual ~CentroidMultiD();

   virtual void set_user_options( const MString& prefix = "" );
   virtual void set_nominal_type( NominalType nomType );

   virtual Bool can_cast_to_multiD() const { return TRUE; }
   virtual CentroidMultiD* cast_to_multiD() { return this; }
   virtual const CentroidMultiD& cast_to_multiD() const { return *this; }

   virtual int num_dimensions() const { return get_schema().num_attr();}

   virtual Bool operator==(const Centroid& rhs) const;
   virtual Bool operator==(const CentroidMultiD& rhs) const;

   virtual void set_centroid( Centroid*& initCent, int position );
   virtual Centroid& get_centroid( int position );
   virtual const Centroid& get_centroid( int position ) const;

   virtual void display(MLCOStream& stream = Mcout) const;
   virtual PtrArray<Centroid*>* init_centroids(int numCentroids, 
					       const InstanceList& instances ); 

   virtual Real distance(const InstanceRC& inst) const;
   virtual Real distance(const Centroid& inst) const;

   virtual void add_to(const InstanceRC& instance);
   virtual void del_from(const InstanceRC& instance);
   virtual void reset();
};


#endif
