// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _ConstCategorizer_h
#define _ConstCategorizer_h 1

#include <Categorizer.h>
#include <DisplayPref.h>
#include <CatDist.h>

// Persistent storage object for Const Categorizers
class PerConstCategorizer_ : public PerCategorizer_ {
   NO_DEFAULT_OPS(PerConstCategorizer_);
public:
   // flag to determine if scoring version is being used
   Bool useScoring;
   
   // Ravelled CatDist information (if scoring is selected)
   int low;
   int size;
   Array<Real> *scores;
   Array<int> *order;

   // category and description (if scoring is not selected)
   int cat;
   MString description;

   PerConstCategorizer_()
      : useScoring(FALSE), low(UNDEFINED_INT), size(UNDEFINED_INT),
	scores(NULL), order(NULL), cat(UNDEFINED_INT),
	description(EMPTY_STRING) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


// Const Categorizer returns a constant category independent of the
// instance.
class ConstCategorizer : public Categorizer {
   NO_DEFAULT_OPS(ConstCategorizer);

   // Member data
   CatDist *predDist;
public:
   virtual void OK(int level = 0) const;

   ConstCategorizer(const MString& dscr, const AugCategory& augCat,
		    const SchemaRC& schema);
   ConstCategorizer(const MString& dscr, CatDist *&catDist,
		    const SchemaRC& schema);
   ConstCategorizer(const ConstCategorizer&, CtorDummy);
   ConstCategorizer(const PerConstCategorizer_& data, const SchemaRC& schema);

   virtual ~ConstCategorizer() { DBG(OK(0)); delete predDist; }
   // ignore instance
   virtual AugCategory categorize(const InstanceRC&) const
      { DBG(OK(1));  return predDist->best_category(); }
   virtual CatDist *score(const InstanceRC&) const
      { DBG(OK(1));  return new CatDist(*predDist, ctorDummy); }
   virtual Bool supports_scoring() const { return TRUE; }
   
   virtual AugCategory get_category() const
      { DBG(OK(1));  return predDist->best_category(); }
   virtual const CatDist& get_cat_dist() const
      { DBG(OK(1));  return *predDist; }
   virtual void display_struct(MLCOStream& stream = Mcout,
			    const DisplayPref& dp = defaultDisplayPref) const;
   // Returns a pointer to a deep copy of this ConstCategorizer
   virtual Categorizer* clone() const;

   virtual void set_used_attr(BoolArray&) const {}
   virtual void project(const BoolArray& mask);
   
   // Returns the class ID
   virtual int class_id() const { return CLASS_CONST_CATEGORIZER; }

   virtual Bool operator==(const Categorizer& rhs) const ;

   // persistence
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerConstCategorizer_;
   }
   virtual void make_persistent(PerCategorizer_&) const;

   virtual Bool supports_persistence() const { return TRUE; }
};

#endif
