// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _Tree_h
#define _Tree_h 1

#include <RootCatGraph.h>
#include <RDGCat.h>
#include <mlcIO.h>
#include <TDDTInducer.h>

class DecisionTree : public RootedCatGraph {
   NO_DEFAULT_OPS(DecisionTree);

   // Private methods
   void convertToTreeVizFormat(MLCOStream& conf, //@@ make virtual
			       MLCOStream& data,
			       const DisplayPref& pref,
			       Bool hasNodeLosses,
			       Bool hasLossMatrix) const;
   void process_XStream_display(const DisplayPref& dp,
				Bool hasNodeLosses = FALSE,
				Bool hasLossMatrix = FALSE) const;
   
public:
   virtual void OK(int level = 0) const;
   DecisionTree(Bool isGraphSparse = FALSE);
   DecisionTree(CGraph& grph, Bool isGraphSparse = FALSE);
   DecisionTree(const DecisionTree& source, CtorDummy);
   DecisionTree(const PerRootedCatGraph_& data,
		NodeInfo*& prototype,
		const Array<NodeInfo *>& nodes,
		Bool isGraphSparse = FALSE) :
      RootedCatGraph(data, prototype, nodes, isGraphSparse) { }
   
   virtual ~DecisionTree();
   virtual CatGraph *clone() const
   { return new DecisionTree(*this, ctorDummy); }
   
   virtual void display(Bool hasNodeLosses,
			Bool hasLossMatrix,
			MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const;
   virtual void display(MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const
   { display(FALSE, FALSE, stream, dp);}
   virtual void delete_subtree(NodePtr node,
			       const NodePtr newNode = NULL);
   void assign_subtree_levels(NodePtr node, int baseLevel); 
   virtual Bool check_node_reachable(const NodePtr source,
				     const NodePtr dest) const;
   virtual void distribute_instances(NodePtr subtree,
				     const InstanceList& il,
				     Real pruningFactor,
				     Real& pessimisticErrors,
				     TDDTInducer::LeafDistType ldType,
				     Real leafDistParameter,
				     const Array<Real> *parentWeightDist,
				     Bool saveOriginalDistr = FALSE);
};

#endif
