// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _DiscCat_h
#define _DiscCat_h 1


#include <InstanceList.h>
#include <RealDiscretizor.h>
#include <EntropyDisc.h>
#include <UniformRangeDisc.h>
#include <OneR.h>

class PerDiscCat_ : public PerCategorizer_ {
   NO_DEFAULT_OPS(PerDiscCat_);
public:
   BoolArray *hasDisc;
   Array<PerRealDiscretizor_> *disc;

   PerDiscCat_()
      : hasDisc(NULL), disc(NULL) {}
   void read(MInStore& in);
   void write(MOutStore& out);
};

   
class DiscCat : public Categorizer{
   NO_DEFAULT_OPS(DiscCat);

   // Member data (also see public data)
   SchemaRC discSchema;
   Categorizer* categorizer;
   PtrArray<RealDiscretizor*>* disc;

   // Private method
   void project_discretizor (const BoolArray& mask);   
public:
   // Public data
   static const Category discDummyCat;
   static const MString discDummyName;
   // Methods
   DiscCat( const MString& description,
	    PtrArray<RealDiscretizor*>*& discretizors, //gets ownership
	    const SchemaRC& discretizedSchema,
	    Categorizer*& cat,
	    const SchemaRC& thisSchema);
   DiscCat(const DiscCat& source, CtorDummy);
   DiscCat(const PerDiscCat_& data, const SchemaRC& schema,
	   Categorizer*& innerCat);
   virtual ~DiscCat();
   virtual void OK(int level = 0) const;
   virtual AugCategory categorize(const InstanceRC& inst) const;
   virtual CatDist *score(const InstanceRC& inst) const;
   virtual Bool supports_scoring() const {
      return categorizer->supports_scoring(); }
   virtual Bool supports_backfit() const {
      return categorizer->supports_backfit(); }
   virtual void backfit(const InstanceList& instList);
   virtual void display_struct(MLCOStream& stream,
		       const DisplayPref& dp) const;
   virtual Categorizer* clone() const;
   virtual int class_id() const { return CLASS_DISC_CATEGORIZER; }
   virtual Bool operator==(const Categorizer& rhs) const;

   virtual const Categorizer& inner_categorizer() const;
   virtual const SchemaRC& inner_schema() const;

   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerDiscCat_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   
   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);
   virtual Categorizer *project_and_set_inner(const BoolArray& mask,
					      Categorizer*& newInner);

   virtual Bool supports_persistence() const {
     ASSERT(categorizer);  return categorizer->supports_persistence(); }

   virtual PtrArray<RealDiscretizor*>* get_real_discretizors() const
   { return disc; }
};

#endif







