// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _DiscNodeCat_h
#define _DiscNodeCat_h 1

#include <NodeCategorizer.h>
#include <RealDiscretizor.h>

class PerDiscNodeCat_ : public PerNodeCategorizer_ {
   NO_DEFAULT_OPS(PerDiscNodeCat_);
public:
   BoolArray *hasDisc;
   Array<PerRealDiscretizor_> *disc;
   
   PerDiscNodeCat_() : hasDisc(NULL), disc(NULL) { }
   void read(MInStore& in);
   void write(MOutStore& out);
};

class DiscNodeCat : public NodeCategorizer {
   NO_DEFAULT_OPS(DiscNodeCat);

   // Member data (also see public data)
   SchemaRC innerSchema;
   PtrArray<RealDiscretizor*> *discretizors;
public:
   // Public data
   static const AugCategory dummyAugCategory;

   // Methods
   virtual void OK(int level = 0) const;

   DiscNodeCat(const SchemaRC& originalSchema, const SchemaRC& newSchema,
	       PtrArray<RealDiscretizor*>*& discVector);
   DiscNodeCat(const DiscNodeCat& source, CtorDummy);
   DiscNodeCat(const PerDiscNodeCat_& data, const SchemaRC& schema,
	       const SchemaRC& innerSch);
   ~DiscNodeCat() { delete discretizors; }

   Categorizer *clone() const { return new DiscNodeCat(*this, ctorDummy); }

   virtual Bool operator==(const Categorizer& rhs) const;
   
   virtual int class_id() const { return CLASS_DISC_NODE_CATEGORIZER; }

   virtual void set_used_attr(BoolArray&) const { } // doesn't use any

   SchemaRC get_inner_schema() const { return innerSchema; }

   AugCategory branch(const InstanceRC& inst) const;  // will abort
   virtual AugCategory *get_edge_label(int branchNum) const; // @@ must deal!
   
   virtual CatDist *score(const InstanceRC& inst) const
   { return NodeCategorizer::score(inst); }
   virtual CatDist *score(const InstanceRC& inst, Bool addLoss) const;
   virtual AugCategory categorize(const InstanceRC& inst) const;

   virtual void display_struct(MLCOStream& stream,
			       const DisplayPref& dp) const;
   
   virtual InstanceListArray* split_instance_list(const InstanceList& il)
      const;

   virtual void distribute_instances(const InstanceList&,
				     Real ,
				     Real &,
				     TDDTInducer::LeafDistType,
				     Real ,
				     const Array<Real> *,
				     Bool)
   { err << "DiscNodeCat::distribute_instances: I don't know how!"
	 << fatal_error; }


   virtual PerCategorizer_ *build_persistent_object() const
   { return new PerDiscNodeCat_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   // @@ We can't be persistent now, because the current mechanism does not
   // allow different schemas among the categorizers in a graph.
   virtual Bool supports_persistence() const { return FALSE; }
};
   
#endif
