// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef DISTRIBDISTFUNCTOR_H_
#define DISTRIBDISTFUNCTOR_H_ 1


class DistribDistFunctor
{
public:
  DistribDistFunctor() {}
  virtual DistribDistFunctor *clone() const = 0;
  virtual ~DistribDistFunctor() {}
  
  virtual Real distance( const Array<Real>& p,
			 const Array<Real>& q ) = 0;
  virtual Real distance( const Array<Real>& q,
			 int position ) = 0;
  Real distance( int position1, int position2 )
            { return(position1 == position2 ? 0.0 : 1.0); }
};

class VectorDistFunctor : public DistribDistFunctor
{
public:
  VectorDistFunctor() {}
  virtual ~VectorDistFunctor() {}
  virtual DistribDistFunctor *clone() const
     { return new VectorDistFunctor; }
   
  virtual Real distance( const Array<Real>& p,
			 const Array<Real>& q );
  virtual Real distance( const Array<Real>& q,
			 int position );
};

class SquaredDiffFunctor : public DistribDistFunctor
{
public:
  SquaredDiffFunctor() {}
  virtual ~SquaredDiffFunctor() {}
  virtual DistribDistFunctor *clone() const
     { return new SquaredDiffFunctor; }

  virtual Real distance( const Array<Real>& p,
			 const Array<Real>& q );
  virtual Real distance( const Array<Real>& q,
			 int position );
};

class AbsoluteDiffFunctor : public DistribDistFunctor
{
public:
  AbsoluteDiffFunctor() {}
  virtual ~AbsoluteDiffFunctor() {}
  virtual DistribDistFunctor *clone() const
     { return new AbsoluteDiffFunctor; }
   
  virtual Real distance( const Array<Real>& p,
			 const Array<Real>& q );
  virtual Real distance( const Array<Real>& q,
			 int position );
};

class KullbackLeiblerFunctor : public DistribDistFunctor 
{
public:
  KullbackLeiblerFunctor() {}
  virtual ~KullbackLeiblerFunctor() {}
  virtual DistribDistFunctor *clone() const
     { return new KullbackLeiblerFunctor; }

  virtual Real distance( const Array<Real>& p,
			 const Array<Real>& q );
  virtual Real distance( const Array<Real>& q,
			 int position );
};

#endif
