// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FCExpression_h
#define _FCExpression_h 1

#include <LogOptions.h>
#include <Array.h>
#include <DblLinkList.h>

class InstanceRC;
class AttributeColumn;
class AttrInfo;
class AttrValue_;
class SchemaRC;

class FCExpression {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(FCExpression);
public:
   // emptyOp is a placeholder; nullOp is for attributes
   enum FCOperator { emptyOp, nullOp, dayOfQuarter, plus, minus, times,
		     divide, nominalEq };
private:
   // Member data
   MString description;
   AttributeColumn *column;
   Bool ownColumn;
   FCOperator myOperator;
   AttrInfo *attrInfo;
   FCExpression *parent;
   int numTerms; // for MDL penalty
   union {
      int attrNum;
      FCExpression *unaryOperand;
      struct {
	 FCExpression *right;
	 FCExpression *left;
      } operands;
   } evalInfo;

   // Private methods
   void init();
   void build_attr_info();
   void build_name();
   void build_column();
   FCExpression *copy_from_below(const FCExpression *oldChild);

   FCExpression(FCExpression *operand, FCOperator op, FCExpression *aParent);
   FCExpression(FCExpression *lhs, FCOperator op, FCExpression *rhs,
		FCExpression *aParent);  
public:
   void OK(int level = 0) const;
   FCExpression();
   FCExpression(int attrNum, AttributeColumn *value);
   FCExpression(const FCExpression& source, CtorDummy);
   FCExpression(const FCExpression& operand, FCOperator op);
   FCExpression(const FCExpression& lhs, FCOperator op,
		const FCExpression& rhs);
   ~FCExpression();

   int num_terms() const { return numTerms; }

   AttributeColumn *release_column();
   void drop_columns();
   
   FCOperator get_operator() const { return myOperator; }
   const AttributeColumn& get_column() const
   { ASSERT(column); return *column; }
   const AttrInfo& attr_info() const { return *attrInfo; }
   
   AttrValue_ evaluate_instance(const InstanceRC& inst) const;

   void leaf_replace(const PtrArray<FCExpression*>& others,
		     DblLinkList<FCExpression*>& children) const;
   
   Bool compatible_with_schema(const SchemaRC& schema,
			       Bool fatalOnFalse = TRUE) const;

   const MString& get_name() const { return description; }
   void display(MLCOStream& stream = Mcout) const;
};
   
Bool operator==(const FCExpression& lhs, const FCExpression &rhs);
Bool operator!=(const FCExpression& lhs, const FCExpression &rhs);

DECLARE_DISPLAY(FCExpression)
#endif
