// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FCExpressionCache_h
#define _FCExpressionCache_h 1

#include <DblLinkList.h>

class FCExpression;
class AttributeColumn;
class AttrInfo;

class FCExpressionValuePair {
   NO_OPERATOR_ASSIGN(FCExpressionValuePair);
public:
   // Public member data
   const FCExpression& expression;
   const AttributeColumn& value;

   // Methods
   FCExpressionValuePair(const FCExpression& expr,
			 const AttributeColumn& val)
      : expression(expr), value(val) { }
   FCExpressionValuePair(const FCExpressionValuePair& source)
      : expression(source.expression), value(source.value) { }
};   

typedef DLLPix<FCExpressionValuePair> EVPix;

// The cache doesn't own anything that it contains (beware)
class FCExpressionCache {
   NO_DEFAULT_OPS(FCExpressionCache);
   DblLinkList<FCExpressionValuePair> expressionValues;
public:
   void OK(int level = 0) const;
   FCExpressionCache() : expressionValues() { }
   ~FCExpressionCache() { }
   
   Bool in_cache(const FCExpression& expression) const;
   const DblLinkList<FCExpressionValuePair>& get_expressions()
   { return expressionValues; }
   const AttributeColumn& get_value(FCExpression& expression);
   void add_value(const FCExpression& expression);
};

#endif
