// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FCExpressionState_h
#define _FCExpressionState_h 1

#include <State.h>
#include <FCExpression.h>

class FCSearchInfo;

typedef State<FCExpression, FCSearchInfo> FCExprSearchState;

class FCExpressionState : public FCExprSearchState {
   NO_DEFAULT_OPS(FCExpressionState);

   // Member data
   Real unadjustedFitness;
   Real parentFitness;

   // Private methods
   void gen_balanced_succ(const FCSearchInfo& info,
			  DblLinkList<FCExprSearchState*>& children) const;

   void gen_unbalanced_succ(const FCSearchInfo& info,
			    const StateSpace<FCExprSearchState>& space,
			    DblLinkList<FCExprSearchState*>& children) const;
public:
   void OK(int level = 0) const;

   FCExpressionState(FCExpression*& initialInfo,
		     const FCSearchInfo& searchInfo,
		     Real parentFit = MIN_EVALUATION)
      : FCExprSearchState(initialInfo, searchInfo),
	unadjustedFitness(MIN_EVALUATION), parentFitness(parentFit) { }
   ~FCExpressionState() { }

   Real eval(FCSearchInfo* info, Bool computeReal = TRUE,
	     Bool computeEstimated = TRUE);

   DblLinkList<FCExprSearchState*>*
   gen_succ(FCSearchInfo *info, StateSpace<FCExprSearchState> *space,
	    Bool computeReal = TRUE);

   Real get_unadjusted_fitness() const { return unadjustedFitness; }
   
   void display_info(MLCOStream& stream = Mcout) const;
   void display_stats(MLCOStream& stream,
		      FCSearchInfo *info) const;
   void display_for_graph(MLCOStream& stream = Mcout) const;
};


#endif
