// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FCSearchInfo_h
#define _FCSearchInfo_h 1

#include <SplitScore.h>

class FCExpression;
class AttributeColumn;

class FCSearchInfo {
   NO_DEFAULT_OPS(FCSearchInfo);

   // Member data (also see public data)
   Real mdlPenalty;
   int maxExpressionSize;
   Real diffSearchProjection;
   Bool balancedSearch;
   Array<Real> *weights;
public:
   // Public member data
   const PtrArray<FCExpression*>& rootExpressions;
   SplitScore splitScore;
   const AttributeColumn& labelValues;
   int numLabels;

   // Methods
   void OK(int label = 0) const;
   FCSearchInfo(const PtrArray<FCExpression*>& roots,
		const AttributeColumn& labels,
		Array<Real> *&weights, int numLabs);
   ~FCSearchInfo();

   const Array<Real>& get_weights() const { DBG(OK()); return *weights; }
   
   void set_mdl_penalty(Real penalty) { mdlPenalty = penalty; }
   Real get_mdl_penalty() const { return mdlPenalty; }
   void set_max_expression_size(int size) { maxExpressionSize = size; }
   int get_max_expression_size() const { return maxExpressionSize; }
   void set_diff_search_projection(Real p) { diffSearchProjection = p; }
   Real get_diff_search_projection() const { return diffSearchProjection; }
   void set_balanced_search(Bool b) { balancedSearch = b; }
   Bool get_balanced_search() const { return balancedSearch; }
};

#endif
