// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FeatureSearch_h
#define _FeatureSearch_h 1

#include <DynamicArray.h>
#include <SchemaRC.h>
#include <BFSearch.h>
#include <FCExpressionState.h>

class InstanceList;
class ConstructOp;

class FeatureSearch {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(FeatureSearch);

   // Member data (also see public data)
   BFSearch<FCExpression, FCSearchInfo> searcher;
   DynamicArray<AttributeColumn*> columns;
   SchemaRC *workingSchema;
   const InstanceList *instList;
   AttributeColumn *labels;

   Real mdlPenalty;
   int maxExpressionSize;
   Real diffSearchProjection;
   MString searchTreevizFile;
   Bool balancedSearch;

   // Private methods
   void free();
public:
   // Public data
   static const Real defaultMdlPenalty;
   static const int defaultMaxExpressionSize;
   static const Real defaultDiffSearchProjection;
   static const Bool defaultBalancedSearch;

   // Methods
   virtual void OK(int level = 0) const;

   FeatureSearch();
   FeatureSearch(const InstanceList& il);
   ~FeatureSearch();

   void initialize(const InstanceList& il);
   
   ConstructOp *add_feature();

   void set_user_options(const MString& prefix);

   void set_mdl_penalty(Real penalty) { mdlPenalty = penalty; }
   Real get_mdl_penalty() const { return mdlPenalty; }
   void set_max_expression_size(int size) { maxExpressionSize = size; }
   int get_max_expression_size() const { return maxExpressionSize; }
   void set_diff_search_projection(Real p) { diffSearchProjection = p; }
   Real get_diff_search_projection() const { return diffSearchProjection; }
   void set_search_treeviz_file(const MString& file)
   { searchTreevizFile = file; }
   const MString& get_search_treeviz_file() const
   { return searchTreevizFile; }
   void set_balanced_search(Bool b) { balancedSearch = b; }
   Bool get_balanced_search() const { return balancedSearch; }
};

#endif
