// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _FeatureSet_h
#define _FeatureSet_h 1

#include <DynamicArray.h>
#include <InstanceList.h>

class FeatureSet : private DynamicArray<int>
{
   // Private methods
   Array<int>& operator=(const Array<int>& rhs) {
      return Array<int>::operator=(rhs);
   }
   DynamicArray<int>& operator=(const DynamicArray<int>& rhs) {
      return DynamicArray<int>::operator=(rhs);
   }
   
public:
   void OK() const;

   FeatureSet();
   FeatureSet(const FeatureSet& rhs, CtorDummy dummy);
   FeatureSet(const SchemaRC& schema);
   FeatureSet(const Array<int>& featureArray);

   Bool operator==(const FeatureSet& rhs) const
   {  if(num_features() != rhs.num_features()) return FALSE;
      else return ::operator==(*this, rhs); }
   Bool operator!=(const FeatureSet& rhs) const
      {return ! ((*this)== rhs);}
   FeatureSet& operator=(const FeatureSet& rhs) {
      DynamicArray<int>::operator=(rhs);
      return *this;
   }
   
   void add_feature(int featureNum);
   Bool instances_equal(const InstanceRC& inst1, const InstanceRC& inst2)
        const;
   Bool contains(int elem) const;
   Bool contains(const FeatureSet& rhs) const;
   void difference(const FeatureSet& rhs, FeatureSet& diff) const;
   void project_down(const FeatureSet& oldProj, const FeatureSet& oldInst,
		     FeatureSet& newProj, FeatureSet& newInst)
        const;
   const Array<int>& get_feature_numbers() const {
         return *this; }
   int num_features() const { return size(); }
   
   void display_instance(const InstanceRC& inst, MLCOStream& stream = Mcout)
        const;
   void display_names(const SchemaRC& schema, MLCOStream& stream = Mcout)
        const;
   void display(MLCOStream& stream = Mcout) const;
   
};

DECLARE_DISPLAY(FeatureSet)

#endif

