// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _GenPix_h
#define _GenPix_h 1

template <class DT, class IT, class CT>		// DT = data type
class GenPix {					// IT = index type
protected:
   // Protected member data (also see public data)
   // CT = container type
   const CT& container;
   IT iter;
public:
   // Public data
   static IT const clearValue;

   // Methods
   GenPix(const CT& initialContainer);
   GenPix(const GenPix<DT,IT,CT>& src);		// copy constructor

   virtual void first() = 0;
   virtual void last() = 0;
   virtual void next() = 0;
   virtual void prev() = 0;
   virtual Bool is_valid(Bool fatalOnFalse = FALSE) const = 0;

   Bool is_clear() const { return iter == clearValue; }
   void set_clear() { iter = clearValue; }
   operator Bool() const { return !is_clear(); }
   Bool operator!() const { return is_clear(); }

   GenPix<DT,IT,CT>& operator=(const GenPix<DT,IT,CT>&);
   Bool operator==(const GenPix<DT,IT,CT>&) const;
   Bool operator!=(const GenPix<DT,IT,CT>&) const;
   // operator* is provided inline, for speed.
   // Returns the value of the element indexed by the pix.
   // There is no non-const version of this operator because a pix should
   //   not have write access to the container class.
   // The container class must have a function "data" which takes an index
   //   (of class IT) and returns the data (of class DT) stored at that index.
   const DT& operator*() const
   {
   // The following doubles the running time for t_ID3Inducer in DEBUGLEVEL 2
   // Since it's fairly rare for pointers to be invalid, we'll ignore it.
   // DBGSLOW(is_valid(TRUE));
	  if (is_clear())
          err << "GenPix<>::operator*(): pix is clear" << fatal_error;
      return container.data(iter);
   }
};

#ifdef COMPILE_TIME_TEMPLATES
#include "GenPix.ct"
#endif

#endif
