// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _GlobalOptions_h
#define _GlobalOptions_h 1

// the GlobalOptions class is a class full of static members.
// It is not meant to be instantiated.
// Static members in this class may be called upon before any nontrivial
// init functions get called.  Therefore, all static members must be
// "statically safe" (see basics.h for details)
class GlobalOptions {
public:

   // Public data
   // useful global super-nuisance options

   // fatal error options
   static Bool dumpCore;
   static Bool keepTemp;
   static Bool echoDone;
   
   // temporary directory for mmaped files
   static MString *tempDir;
   
   // names of external viewing utilities used by MLC++
   static MString *dotUtil;
   static MString *dottyUtil;
   static MString *treevizUtil;
   static MString *evivizUtil;
   static MString *scattervizUtil;
   static MString *rulevizUtil;

   // hash table stats
   static Bool displayHashStats;

   // options file dumping information
   static MString *optionsFileName;
   static MString *optionsFileFormat;

   // stream wrapping information
   static int lineWidth;
   static MString *wrapPrefix;
   static MString *newlinePrefix;

   // pathnames
   static MString *mlcPath;

   // logging/debug information
   static int logLevel;
   static int debugLevel;
   static Bool warnFastDebug;
   static Bool dribble;
   static Bool showLogOrigin;
   static Bool showAllOptions;

   // error rate related options
   static int clampEpsilonMultiplier;
   static int printPerfPrecision;

   // exceptions
   static Bool exceptionsEnabled;

   // predictions
   static Bool allowUnknownPredictions;

   // Number of digits of precision to output for Real and StoredReal types.
   static int realPrecision;
   static int storedRealPrecision;

   // Methods
   // helpful functions
   static void init();      // called early in startup, BEFORE
                            // option server is ready.
   static void reinit();    // called in command-line processing, when
                            // option server is ready
   static void finish();    // called at shutdown time.
};
   
#endif
