// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _Grouper_h
#define _Grouper_h 1

#include <SGIDTInducer.h>

class InstanceList;
class PartitionSplit;
class PartitionEvaluator;

class Grouper {
   NO_DEFAULT_OPS(Grouper);

   // Member data (also see public data)
   PtrArray<Array<int>*> *partitions;
   PtrArray<Array<MString>*> *partitionNames;
   InstanceList *groupedData;
   InstanceList *groupedTest;
   PartitionSplit *splitter;
   int minSetSize;
   int minNumValues;
   SplitScore::SplitScoreCriterion splitBy;

   // Private methods
   PartitionEvaluator *get_partition_evaluator(const InstanceList& data,
					       int attrNum) const;

   Array<int> *find_partition(const InstanceList& data, int attrNum) const;

public:
   // Public data
   static const int defaultMinNumValues;
   static const int defaultMinSetSize;
   static const PartitionMethodType defaultPartitionMethod;
   static const SplitScore::SplitScoreCriterion defaultSplitScoreCriterion;

   // Methods
   Grouper();
   ~Grouper();

   void group(InstanceList *trainingSet, InstanceList *testSet);

   void dump_results(const MString& dumpStem) const;

   const InstanceList& get_grouped_data() const;
   const InstanceList& get_grouped_test() const;
   const Array<int>& get_partition(int attrNum) const;

   InstanceList *release_grouped_data();
   InstanceList *release_grouped_test();
   
   void set_user_options(const MString& prefix);
   int get_min_set_size() const { return minSetSize; }
   int get_min_num_values() const { return minNumValues; }
   SplitScore::SplitScoreCriterion get_split_by() const { return splitBy; }
   void set_min_set_size(int size) { minSetSize = size; }
   void set_min_num_values(int num) { minNumValues = num; }
   void set_split_by(SplitScore::SplitScoreCriterion split) { splitBy = split;}
};

#endif
