// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.


#ifndef IBCategorizer_h
#define IBCategorizer_h

#include <TableCat.h>
#include <InstanceList.h>
#include <InstListMinArray.h>

class IBCategorizer : public TableCategorizer {
private:   
   NO_DEFAULT_OPS(IBCategorizer);

   // Member data
   int kVal;
   Array<Real> *weightVector;
   InstanceList instStore;
   IBInducer::NeighborVote vote;
   IBInducer::NnkValue nnkValue;
protected:
   // Protected methods
   Real distance(const Instance& inst1, const Instance& inst2,
		 Real maxDist) const;
   Category major_category(const InstListMinArray& minArray) const;   
   CatDist *get_scores(const InstListMinArray& minArray) const;  
   void categorize_helper(const InstanceRC& instance,
			  InstListMinArray &minArray) const;
public:

   virtual void OK(int level = 0) const;
   IBCategorizer(const MString& dscr, const InstanceList& instList);
   IBCategorizer(const IBCategorizer& source, CtorDummy);
   virtual ~IBCategorizer();
   virtual AugCategory categorize(const InstanceRC& ) const;
   virtual CatDist *score(const InstanceRC&) const;
   virtual Bool supports_scoring() const {
      return TRUE;
   }
   virtual void display_struct(MLCOStream& stream,
			       const DisplayPref& dp) const ;
   virtual Categorizer* clone() const;
   virtual void set_weights(const Array<Real>& weights);
   virtual const Array<Real>& get_weights() const { 
        ASSERT(weightVector); return *weightVector; }
   int get_k_val() const { return kVal; }
   void set_k_val(int k);
   // Returns the class id
   virtual int class_id() const { return CLASS_IB_CATEGORIZER;};

   virtual void add_instance(const InstanceRC& inst);
   virtual Bool operator==(const Categorizer& rhs) const;
   virtual void set_neighbor_vote(IBInducer::NeighborVote v) { vote = v; }
   virtual IBInducer::NeighborVote get_neighbor_vote() const { return vote; }
   virtual void set_nnk_value(IBInducer::NnkValue v) { nnkValue = v; }
   virtual IBInducer::NnkValue get_nnk_value() const { return nnkValue; }
};
#endif
