// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file "InstCounts.c"


#ifndef _InstCounts_h
#define _InstCounts_h 1

#include <SchemaRC.h>
#include <InstanceRC.h>
#include <UnivHashTable.h>
#include <IntArrayCt.h>

class InstanceList;

class InstCounts : private UniversalHashTable<IntArray, IntArrayAndCount> {
   NO_COPY_CTOR(InstCounts);
   NO_OPERATOR_ASSIGN(InstCounts);

   IntArrayAndCount updateArray;
   
   const IntArray attributes;
   const InstanceList& instances;

   DBG_DECLARE(const InstanceRC firstInst;)
   DBG_DECLARE(const InstanceRC lastInst;)
   DBG_DECLARE(const Real totalWeight;)

   void init_counters();
   void check_query(const IntArray& query) const;
   
public:
   virtual void OK(int level = 0) const;
   
   InstCounts(const IntArray& attrs, const InstanceList& il);
   InstCounts(const InstCounts& source, CtorDummy);
   virtual ~InstCounts() { DBG(OK(0)); }

   // Simple accessors
   const InstanceList& get_instances() const { DBG(OK()); return instances; }
   const IntArray& get_attributes() const { DBG(OK()); return attributes; }
   const IntArray& get_key() const { return attributes; }
   Bool contains_key(const IntArray& key) const
   { return key.size() == attributes.size() && key == attributes; }
   
   virtual void display(MLCOStream& stream) const;
   Real count(const IntArray& vals) const;
   void update(const InstanceRC& instance);
   const IntArray& get_attrs() const;
};

DECLARE_DISPLAY(InstCounts)

#endif
