// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _InstanceOp_h
#define _InstanceOp_h 1

#include <BoolArray.h>

#define CLASS_PROJECT_OP 1
#define CLASS_OP_LIST_OP 2
#define CLASS_CONSTRUCT_OP 3

class Instance;
class Schema;
class InstanceRC;
class SchemaRC;

class InstanceOp {   // ABC
   NO_DEFAULT_OPS(InstanceOp);

   // Member data
   SchemaRC *origSchema;
   SchemaRC *finalSchema;   
protected:
   // Protected methods
   void set_final_schema(const SchemaRC& newFinal);   
public:
   InstanceOp(const SchemaRC& schema);
   InstanceOp(const InstanceOp& op, CtorDummy);
   virtual InstanceOp *clone() const = 0;
   
   virtual InstanceRC apply(const InstanceRC& inst) const = 0;
   virtual SchemaRC apply(const SchemaRC& schema) const = 0;
   virtual int class_id() const = 0;
   virtual void display(MLCOStream& out = Mcout) const = 0;
   
   virtual Bool can_add_operation(const InstanceOp&) const;
   virtual void add_operation(InstanceOp*& op);

   virtual int convert_attr_num(int) const { return -1; }
   virtual SchemaRC& original_schema() const;
   virtual SchemaRC& final_schema() const;
   virtual ~InstanceOp() {}
   // Note:  The following, though pure virtual, has an implementation
   //   (defined in InstanceOp.c).  This implementation MUST be invoked
   //   by the operator== of all derived classes.
   virtual Bool operator==(const InstanceOp& rhs)const  = 0;
   Bool operator!=(const InstanceOp& rhs)const
      {return ! ((*this)== rhs);}
};

DECLARE_DISPLAY(InstanceOp)



class ProjectOp : public InstanceOp {
   NO_DEFAULT_OPS(ProjectOp);

   // Member data
   BoolArray projectMask;
public:
   ProjectOp(const SchemaRC& schema, int attrNum);
   ProjectOp(const SchemaRC& schema, const BoolArray& attrMask);
   ProjectOp(const ProjectOp& op, CtorDummy);

   virtual InstanceOp *clone() const
      { return new ProjectOp(*this, ctorDummy); }
   
   virtual InstanceRC apply(const InstanceRC& inst) const;
   virtual SchemaRC apply(const SchemaRC& schema) const;
   
   virtual int class_id() const { return CLASS_PROJECT_OP; }
   virtual void display(MLCOStream& out = Mcout) const;
   
   virtual Bool can_add_operation(const InstanceOp& op) const;
   virtual void add_operation(InstanceOp*& op);
   virtual int convert_attr_num(int originalNum) const;
   virtual Bool operator==(const InstanceOp& rhs) const;
};


#endif


