// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.
/***************************************************************************
  Description  : IntArray is an Array<int> that offers the appropriate
                  helper functions for UnivHashTable, i.e. length() and
		  operator const unsigned char *.
  Assumptions  :
  Comments     : This was created to allow for smart counters - we're
                  using this as the KeyType for hash table, thus hashing
		  by a set of attributes (both nominal attribute values and
		  attribute numbers).

		 Everything is defined in the header.
  Complexity   :
  Enhancements : 
  History      : Eric Bauer and Clay Kunz                      5/17/96
                   Initial revision (.h)
***************************************************************************/

#ifndef _IntArray_h
#define _IntArray_h 1

#include <Array.h>

class IntArray : public Array<int> {
   NO_COPY_CTOR(IntArray);
public:
   IntArray(const IntArray& source, CtorDummy dummyArg)
      : Array<int>(source, dummyArg) { }
   IntArray(int size)
      : Array<int>(size) { }
   virtual ~IntArray() { }

   int length() const { return sizeof(int)*size(); }

   // get_internal_rep is for universal hashing  
   const unsigned char *get_internal_rep() const {
      return (const unsigned char *)elements;}

   // Needed to avoid ambiguity problems under GNU
   // IntArray::operator= isn't found
   // IntArray::operator<< is calling BoolArray::operator<<
#ifdef GNU
   IntArray& operator=(const IntArray& source)
   { Array<int>::operator=(source); return *this; }
   void display(MLCOStream& stream = Mcout) const
   { Array<int>::display(stream); }
#endif
};

//@@ on the PC, there was no << function for base Arrays.
//@@ add one here for IntArray (make it inline to keep the code
//@@ in the header)
#ifdef PC_MSVC
inline DEF_DISPLAY(IntArray)
#endif

// GNU doesn't find the a version of <<, ==, and !=
#ifdef GNU
inline DEF_DISPLAY(IntArray)
   
static Bool operator==(const IntArray& lhs, const IntArray& rhs)
{ return (const Array<int>&)lhs==(const Array<int>&)rhs;}

static Bool operator!=(const IntArray& lhs, const IntArray& rhs)
{ return (const Array<int>&)lhs!=(const Array<int>&)rhs;}
#endif


#endif



