/*******************************************************************************
+
+  LEDA 3.5
+
+
+  graph_misc.h
+
+
+  Copyright (c) 1995, 1996, 1997  by  LEDA Software GmbH
+  Postfach 151101, 66041 Saarbruecken, Germany
+  All rights reserved.
+ 
*******************************************************************************/


#ifndef LEDA_GRAPH_MISC_H
#define LEDA_GRAPH_MISC_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 350056
#include <LEDA/REDEFINE_NAMES.h>
#endif




//-----------------------------------------------------------------------------
// miscellaneous  (should be members or constructors)
//-----------------------------------------------------------------------------

#include <LEDA/graph.h>
#include <LEDA/node_array.h>
#include <LEDA/edge_array.h>

/*{\Manpage {graph_misc} {} {Miscellaneous Graph Functions} }*/

/*{\Moperations 2 5}*/

extern __exportF bool Is_Simple(const graph& G);
/*{\Mfunc  returns true if $G$ is simple, i.e., has no parallel
           edges, false otherwise. }*/

inline bool Is_Bidirected(const graph& G) { return G.is_bidirected(); }
/*{\Mfuncl  	returns true if every edge has a reversal and 
                false otherwise. }*/

extern __exportF bool Is_Bidirected(const graph& G, edge_array<edge>& rev);
/*{\Mfuncl  	computes for every edge $e = (v,w)$ in $G$ its
		reversal $rev[e] = (w,v)$ in $G$ (nil if
		not present). Returns true if every edge has a
		reversal and false otherwise. }*/

extern __exportF bool Is_Map(const graph& G);
/*{\Mfuncl      tests whether $G$ is a map. }*/	

extern __exportF bool Is_Planar_Map(const graph& G);
/*{\Mfuncl      tests whether $G$ is a planar map. }*/	
  
extern __exportF bool Is_Acyclic(const graph& G);
/*{\Mfunc  returns true if the directed $G$ is acyclic
           and false otherwise. }*/

extern __exportF bool Is_Acyclic(const graph& G, list<edge>& L);
/*{\Mfunc  as above; in addition, constructs a list of edges $L$ 
           whose deletion makes $G$ acyclic. }*/

extern __exportF void Make_Acyclic(const graph& G);
/*{\Mfunc  makes $G$ acyclic by removing all DFS back edges. }*/ 



extern __exportF bool Is_Connected(const graph& G);
/*{\Mfunc  returns true if the undirected graph underlying $G$ is connected
           and false otherwise. }*/


extern __exportF bool Is_Biconnected(const graph& G);
/*{\Mfunc  returns true if the undirected graph underlying $G$ is biconnected
           and false otherwise. }*/


extern __exportF bool Is_Bipartite(const graph& G);
/*{\Mfuncl returns true if $G$ is bipartite and false otherwise. }*/

extern __exportF bool Is_Bipartite(const graph& G, list<node>& A, list<node>& B);
/*{\Mfuncl returns true if $G$ is bipartite and false otherwise. 
           If $G$ is bipartite the two sides are returned in $A$ and $B$,
           respectively. }*/

extern __exportF bool Is_Planar(const graph& G);
/*{\Mfuncl returns true if $G$ is planar and false otherwise. }*/



extern __exportF list<edge> Make_Simple(graph& G);
/*{\Mfunc  makes $G$ simple by removing all but one from each set of
           parallel edges. Returns the list of remaining edges with 
           parallel edges in the original graph. }*/



extern __exportF void Make_Bidirected(graph& G,list<edge>& L);
/*{\Mfuncl 	makes $G$ bidirected by inserting missing reversal
                edges. Appends all inserted edges to list $L$. }*/

extern __exportF list<edge> Make_Bidirected(graph& G);
/*{\Mfuncl 	makes $G$ bidirected by inserting missing reversal
                edges. Returns the list of inserted edges. }*/


extern __exportF void Make_Connected(graph& G,list<edge>& L);
/*{\Mfuncl  makes $G$ connected; appends all inserted 
            edges to list $L$. }*/

extern __exportF list<edge> Make_Connected(graph& G);
/*{\Mfuncl  makes $G$ connected; returns the list of inserted edges. }*/



extern __exportF void Make_Biconnected(graph& G,list<edge>& L);
/*{\Mfuncl  makes $G$ biconnected; appends all inserted 
            edges to list $L$. }*/

extern __exportF list<edge> Make_Biconnected(graph& G);
/*{\Mfuncl  makes $G$ biconnected; returns the list of inserted edges. }*/



extern __exportF list<node> Delete_Loops(graph& G);
/*{\Mfunc returns the list of nodes with self-loops and deletes 
           all self-loops. }*/


extern __exportF int Genus(const graph& G);
/*{\Mfunc computes the genus of $G$. \precond $G$ must be a map. }*/


// for historical reasons:

inline bool compute_correspondence(const graph& G, edge_array<edge>& rev)
{ return Is_Bidirected(G,rev); }

inline void eliminate_parallel_edges(graph& G) { Make_Simple(G); }



#if LEDA_ROOT_INCL_ID == 350056
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif

#endif

