/*******************************************************************************
+
+  LEDA 3.5
+
+
+  iteration.h
+
+
+  Copyright (c) 1995, 1996, 1997  by  LEDA Software GmbH
+  Postfach 151101, 66041 Saarbruecken, Germany
+  All rights reserved.
+ 
*******************************************************************************/


#ifndef LEDA_ITERATION_H
#define LEDA_ITERATION_H

#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 350067
#include <LEDA/REDEFINE_NAMES.h>
#endif


#if defined(__GNUC__) || defined(__ELSE_SCOPE_BUG__)
#define LEDA_FORALL_PREAMBLE
#define LOOP_CAT(x,y) x ## y 
#define LOOP_VAR(y)  LOOP_CAT(LOOP_VAR,y)
#define LOOP_VAR1(y) LOOP_CAT(LOOP_VAR1,y)
#else
#define LEDA_FORALL_PREAMBLE  if (0); else
#define LOOP_VAR(y)  LOOP_VAR
#define LOOP_VAR1(y) LOOP_VAR1
#endif


//------------------------------------------------------------------------------
//   forall_items(it,...)
//   forall_rev_items(it,...)
//------------------------------------------------------------------------------


template<class T>
inline void LOOP_ASSIGN(T& x, void* p) { x = (T)p; }

#define forall_items(x,S)\
LEDA_FORALL_PREAMBLE \
for(void* LOOP_VAR(__LINE__)=(S).first_item();\
LOOP_ASSIGN(x,LOOP_VAR(__LINE__)), LOOP_VAR(__LINE__)=(S).next_item(x), x; )

#define forall_rev_items(x,S)\
LEDA_FORALL_PREAMBLE \
for(void* LOOP_VAR(__LINE__)=(S).last_item();\
LOOP_ASSIGN(x,LOOP_VAR(__LINE__)), LOOP_VAR(__LINE__)=(S).pred_item(x), x; )



//------------------------------------------------------------------------------
//   forall(x,...)
//   forall_rev(x,...)   (Forall)
//   forall_defined(x,...)
//------------------------------------------------------------------------------

#if defined(__hpuxcc__) || defined(__GNUC__)
#define LOOP_ITEM(p) S.make_item(p) 
#else
#define LOOP_ITEM(p) ((T::item)p)
#endif

/*Note to authors: Making code unreadable does not make it any more efficient*/

template<class T>
inline bool LedaLoopSucc(const T& S, void*& p) 
{ if (p) { p = S.next_item(LOOP_ITEM(p)); return true; }
  else return false;
 }

template<class T>
inline bool LedaLoopPred(const T& S, void*& p)
{ if (p) { p = S.pred(LOOP_ITEM(p)); return true; }
  else return false;
 }

template<class T, class var_type>
inline void LedaLoopInf(const T& S, var_type& x, void*& p) 
{ if (p) x = S.inf(LOOP_ITEM(p)); }

template<class T, class var_type>
inline void LedaLoopKey(const T& S, var_type& x, void*& p) 
{ if (p) x = S.key(LOOP_ITEM(p)); }


#define forall(x,S)\
LEDA_FORALL_PREAMBLE \
for(void* LOOP_VAR(__LINE__) = (S).first_item();\
LedaLoopInf(S,x,LOOP_VAR(__LINE__)), LedaLoopSucc(S,LOOP_VAR(__LINE__)); )

#define forall_rev(x,S)\
LEDA_FORALL_PREAMBLE \
for(void* LOOP_VAR(__LINE__) = (S).last();\
LedaLoopInf(S,x,LOOP_VAR(__LINE__)), LedaLoopPred(S,LOOP_VAR(__LINE__)); )

#define forall_defined(x,S)\
LEDA_FORALL_PREAMBLE \
for(void* LOOP_VAR(__LINE__) = (S).first_item();\
LedaLoopKey(S,x,LOOP_VAR(__LINE__)), LedaLoopSucc(S,LOOP_VAR(__LINE__)); )





// Macros for generating STL iterators (experimental)

#define STL_ITERATORS(DTYPE,ITYPE,ETYPE)\
class iterator {\
  friend class DTYPE;\
  DTYPE* L;\
  ITYPE it;\
  iterator(DTYPE* x, ITYPE y) : L(x), it(y)  {}\
public:\
 iterator() : L(0),it(0)   {}\
 iterator& operator++()    { it = L->stl_next_item(it); return *this; }\
 iterator& operator--()    { it = L->stl_pred_item(it); return *this; }\
 iterator  operator++(int) { iterator tmp = *this; ++(*this); return tmp; }\
 iterator  operator--(int) { iterator tmp = *this; --(*this); return tmp; }\
 ETYPE& operator*() const { return (*L)[it]; }\
 friend bool operator==(const iterator& x, const iterator& y)\
 { return x.it == y.it; }\
/* friend bool operator!=(const iterator& x, const iterator& y)\
 { return x.it != y.it; } */\
 operator ITYPE() { return it; }\
 friend ETYPE* value_type(const iterator&) { return (ETYPE*)0; }\
};\
iterator  begin() { return iterator(this,first_item());}\
iterator  end()   { return iterator(this,stl_next_item(last_item()));  }\
\
class const_iterator {\
  friend class DTYPE;\
  const DTYPE* L;\
  ITYPE it;\
  const_iterator(const DTYPE* x, ITYPE y) : L(x), it(y)  {}\
public:\
 const_iterator() : L(0),it(0)   {}\
 const_iterator& operator++()    { it = L->stl_next_item(it); return *this; }\
 const_iterator& operator--()    { it = L->stl_pred_item(it); return *this; }\
 const_iterator  operator++(int) { const_iterator tmp = *this; ++(*this); return tmp; }\
 const_iterator  operator--(int) { const_iterator tmp = *this; --(*this); return tmp; }\
 const ETYPE& operator*() const { return (*L)[it]; }\
 friend bool operator==(const const_iterator& x, const const_iterator& y) \
 { return x.it == y.it; }\
/* friend bool operator!=(const const_iterator& x, const const_iterator& y) \
 { return x.it != y.it; } */\
 operator ITYPE() { return it; }\
 friend ETYPE* value_type(const const_iterator&) { return (ETYPE*)0; }\
};\
const_iterator begin() const { return const_iterator(this,first_item()); }\
const_iterator end()   const { return const_iterator(this,stl_next_item(last_item()));   }


#if LEDA_ROOT_INCL_ID == 350067
#undef LEDA_ROOT_INCL_ID
#include <LEDA/UNDEFINE_NAMES.h>
#endif

#endif
