// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _LeafCategorizer_h
#define _LeafCategorizer_h 1

#include <NodeCategorizer.h>

class PerLeafCategorizer_ : public PerNodeCategorizer_ {
   NO_DEFAULT_OPS(PerLeafCategorizer_);
public:
   PerLeafCategorizer_() { }
   virtual void read(MInStore& in) { PerNodeCategorizer_::read(in); }
   virtual void write(MOutStore& out) { PerNodeCategorizer_::write(out); }
};

class LeafCategorizer : public NodeCategorizer {
   NO_DEFAULT_OPS(LeafCategorizer);

   // Member data
   Categorizer *categorizer;

   // Private methods
   void set_categorizer(Categorizer*& aCategorizer);
protected:
   // Protected methods
   virtual void clear();

public:
   virtual void OK(int level = 0) const;
   LeafCategorizer(Categorizer*& aCategorizer);
   LeafCategorizer(const LeafCategorizer& source, CtorDummy);
   virtual ~LeafCategorizer() { DBG(OK()); clear(); }

   virtual Categorizer* clone() const
   { return new LeafCategorizer(*this, ctorDummy); }

   virtual int class_id() const { return CLASS_LEAF_CATEGORIZER; }

   virtual Bool operator==(const Categorizer& rhs) const;
   
   virtual const Categorizer& get_categorizer() const { return *categorizer; }

   // Functions overridden from NodeCategorizer
   virtual AugCategory branch(const InstanceRC& inst) const;  // will abort
   virtual AugCategory *get_edge_label(int branchNum) const;  // will abort
   
   virtual void distribute_instances(const InstanceList& il,
				     Real pruningFactor,
				     Real &pessimisticErrors,
				     TDDTInducer::LeafDistType ldType,
				     Real leafDistParameter,
				     const Array<Real> *parentWeightDist,
				     Bool saveOriginalDistr);

   virtual AugCategory categorize(const InstanceRC& inst) const
   { return categorizer->categorize(inst); }
   virtual Bool supports_scoring() const { return TRUE; }
   virtual CatDist *score(const InstanceRC& inst) const
   { return NodeCategorizer::score(inst); }
   virtual CatDist *score(const InstanceRC& inst, Bool addLoss) const;

   // Wrapped functions: these we just pass on to the wrapped categorizer.
   int num_categories() const {return categorizer->num_categories();}

   const MString& description() const { return categorizer->description(); }
   void set_description(const MString& val)
   { categorizer->set_description(val); }

   void set_schema(const SchemaRC& sch) { categorizer->set_schema(sch); }
   virtual SchemaRC get_schema() const { return categorizer->get_schema(); }
   
   virtual Real total_weight() const { return categorizer->total_weight(); }

   virtual void build_distr(const InstanceList& instList)
   { categorizer->build_distr(instList); }

   virtual void set_distr(const Array<Real>& val)
   { categorizer->set_distr(val); }
   
   virtual void add_distr(const Array<Real>& val)
   { categorizer->add_distr(val); }
   virtual void add_distr(int label, Real delta)
   { categorizer->add_distr(label, delta); }

   virtual const Array<Real>& get_distr() const
   { return categorizer->get_distr(); }

   virtual void reset_original_distr()
   { categorizer->reset_original_distr(); }
   
   virtual void set_original_distr(const Array<Real>& dist)
   { categorizer->set_original_distr(dist); }

   virtual const Array<Real>& get_original_distr() const
   { return categorizer->get_original_distr(); }

   virtual Bool has_original_distr() const
   { return categorizer->has_original_distr(); }
   
   virtual Bool has_distr() const { return categorizer->has_distr(); }

   virtual Category majority_category(Category zeroBreaker =
				      FIRST_CATEGORY_VAL) const
   { return categorizer->majority_category(zeroBreaker); }
   
   virtual void set_used_attr(BoolArray& used) const
   { categorizer->set_used_attr(used); }
   
   virtual void project(const BoolArray& use)
   { categorizer->project(use); }

   virtual void project_schema(const BoolArray& mask)
   { categorizer->project_schema(mask); }

   virtual void short_display(ostream& stream) const
   { categorizer->short_display(stream); }

   virtual void display_struct(MLCOStream& stream, const DisplayPref& pref)
      const { categorizer->display_struct(stream, pref); }

   const MString& get_extra_viz_text() const
   { return categorizer->get_extra_viz_text(); }
   void set_extra_viz_text(const MString& txt)
   { categorizer->set_extra_viz_text(txt); }

   // Persistence
   virtual PerCategorizer_ *build_persistent_object() const
   { return new PerLeafCategorizer_; }
   virtual void make_persistent(PerCategorizer_& data) const
   { NodeCategorizer::make_persistent(data); }
};


#endif
