// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _LearnCurve_h
#define _LearnCurve_h 1

#include <LogOptions.h>
#include <Array.h>
#include <MRandom.h>
#include <MLCStream.h>
#include <MEnum.h>

class BaseInducer;
class InstanceList;

struct LearnCurveInfo {
   NO_COPY_CTOR(LearnCurveInfo);

   // Public member data
   int numTrainings;
   int trainSize;

   // Methods
   LearnCurveInfo()
      : numTrainings(UNDEFINED_INT), trainSize(UNDEFINED_INT) {}
   LearnCurveInfo& operator=(const LearnCurveInfo& rhs);
   Bool operator==(const LearnCurveInfo& rhs) const
      {return (numTrainings == rhs.numTrainings &&
	       trainSize == rhs.trainSize);}
   Bool operator!=(const LearnCurveInfo& rhs) const
      {return ! ((*this)== rhs);}
};

extern const MString DEFAULT_FILE_STEM;

//@@ DispLearnCurveType == DispConfMatType gives redefinition error
//
// enum DispLearnCurveType {no, ascii, scatterviz, both};
//
//const MEnum dispLearnCurveEnum = MEnum("no", no)
//   << MEnum("ascii", ascii)
//   << MEnum("scatterviz", scatterviz)
//   << MEnum("both", both);

const MString dispConfMatHelp = 
     "Display confusion matrix when displaying results on test";

class LearnCurve {
   LOG_OPTIONS;
   RAND_OPTIONS;
   NO_DEFAULT_OPS(LearnCurve);

   // Member data
   MString fileStem;
   Array<LearnCurveInfo> *info;
   PtrArray<Array<Real>*> *result;
public:
   virtual void OK(int level = 0) const;
   LearnCurve();
   virtual ~LearnCurve();

   virtual void set_sizes_and_trials (const Array<LearnCurveInfo>&
				      learnCurveInfoArray);
   virtual void set_sizes_and_trials (int numPoints,
				      int numTrials,
				      int min,
				      int max,
				      InstanceList& trainIL);
   virtual const MString& get_file_stem() const;
   virtual void set_file_stem(const MString& fileName);
   virtual void learn_curve(BaseInducer& inducer, InstanceList& trainIL);
   virtual Real get_error(int level, int trial) const;
   virtual void mean_error_sdv_and_confidence(int level,
					      Real& meanError,
					      Real& sdvError,
					      Real& confidence) const;
   virtual void display_ascii(MLCOStream& stream) const;
   virtual void display_scatterviz(MLCOStream& conf,
				   MLCOStream& data,
				   Bool interpolate) const;
   virtual void display(MLCOStream& stream = Mcout) const;
};


DECLARE_DISPLAY(LearnCurve)

#endif
