// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _LogOptions_h
#define _LogOptions_h 1

#include <DribbleEngine.h>

class LogOptions {
   int logLevel;      // higher is more log info.
   MLCOStream *stream;
   // default copy constructor OK
public:
   // Note that all functions are const because we do not want to disallow
   //   logging from const function members.
   //   However, callers should return the stream state to its original state.
   LogOptions();
   LogOptions(const MString& logOptionName);
   virtual ~LogOptions() {}
   void    set_log_options(const LogOptions& opt) const;
   virtual void set_log_level(int level) const;
   virtual int  get_log_level() const;
   virtual void set_log_stream(MLCOStream& strm) const;
   virtual MLCOStream& get_log_stream() const;
   virtual void set_log_prefixes(char *file, int line) const;
   virtual void set_log_prefixes(char *file, int line, 
				 int stmtLogLevel, int lgLevel) const;
};
   
// The following macro is meant to be put in the private section of a class.
// It provides a standard interface for set/get of logLevel and stream
#define LOG_OPTIONS \
public: \
   void set_log_level(int level) const {logOptions.set_log_level(level);} \
   int  get_log_level() const {return logOptions.get_log_level();} \
   void set_log_stream(MLCOStream& strm) const \
      {logOptions.set_log_stream(strm);} \
   MLCOStream& get_log_stream() const {return logOptions.get_log_stream();} \
   const LogOptions& get_log_options() const {return logOptions;} \
   void set_log_options(const LogOptions& opt) const \
      {logOptions.set_log_options(opt);} \
   void set_log_prefixes(char *file, int line,int lvl1, int lvl2) const \
     {logOptions.set_log_prefixes(file, line, lvl1, lvl2);} \
private: \
   LogOptions logOptions

void global_set_log_prefixes(MLCOStream& stream, char *file, int line);

// The following assumes you have the get/set log functions
// as defined by the LOG_OPTIONS macro
#undef IFLOG
#undef LOG
#define IFLOG(level, stmt) if (get_log_level() >= level){\
   MStreamOptions saveOptions = get_log_stream().get_options();\
   set_log_prefixes(__FILE__, __LINE__, level, get_log_level()); stmt;\
   get_log_stream().set_options(saveOptions);} else
#define LOG(level,out) IFLOG(level, get_log_stream() << out)

// The following macros use the global log levels.
#define IFGLOBLOG(level, stmt) if (GlobalOptions::logLevel >= level){\
   MStreamOptions saveOptions = (*globalLogStream).get_options();\
   global_set_log_prefixes(*globalLogStream, __FILE__, __LINE__); stmt;\
   (*globalLogStream).set_options(saveOptions);}else
#define GLOBLOG(level,out) IFGLOBLOG(level, *globalLogStream << out)

#define IFFLOG(level, stmt) if (logOptions.get_log_level() >= level) {\
   MStreamOptions saveOptions = logOptions.get_log_stream().get_options();\
   logOptions.set_log_prefixes(__FILE__, __LINE__, \
			       level, logOptions.get_log_level()); stmt;\
   logOptions.get_log_stream().set_options(saveOptions);} else
#define FLOG(level, out) IFFLOG(level, logOptions.get_log_stream() << out)   
									     
// End of LogOptions.

// "DRIBBLE("string")" is intended to be used to send a small amount of
//    information to Mcout.  This info is not intended to inform the user
//    what, why, or how things are occurring.  It is intended for use
//    when time-intensive proccessing is going on, to inform the user that
//    something big is going on, and to give occasional reassurance that
//    things haven't gotten stuck.  It would be used, for example, to
//    print:
//       Started search for 100,000,000th prime number.........
//       Looking at 10,000,000th.........20,000,000th.........30,000,000th
//       .........40,000,000th.........50,000,000th.........60,000,000th
//       .........70,000,000th.........80,000,000th.........90,000,000th
//       ......... done.
//  Better, it should be able to say what portion of the job is done, i.e.,
//    Discretizing 10 attributes: 1 2 3 4 ... 10 

#undef IFDRIBBLE
#undef DRIBBLE
#define IFDRIBBLE(stmt) if (GlobalOptions::dribble){stmt; Mcout << flush;} else
#define DRIBBLE(out) IFDRIBBLE(Mcout << out)

#undef IFDRIBBLEENG
#undef DRIBBLEABS
#undef DRIBBLEREL
#undef DRIBBLEMSG
#undef DRIBBLEDONE

#define IFDRIBBLEENG(stmt) if (GlobalOptions::dribble){stmt;} else
#define DRIBBLEABS(x) if (GlobalOptions::dribble) \
                        globalDribble->set_dribble_completion(x); else
#define DRIBBLEREL(x) if (GlobalOptions::dribble) \
			globalDribble->add_dribble_proportion(x); else
#define DRIBBLEMSG(x) if (GlobalOptions::dribble) \
			globalDribble->set_dribble_message(x); else
#define DRIBBLEDONE   if (GlobalOptions::dribble) \
			globalDribble->section_finished(); else
   
#endif
