// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _MajorityCategorizer_h
#define _MajorityCategorizer_h 1

#include <Categorizer.h>
#include <DisplayPref.h>
#include <InstanceList.h>


class PerMajorityCategorizer_ : public PerCategorizer_ {
   NO_DEFAULT_OPS(PerMajorityCategorizer_);
public:
   PerMajorityCategorizer_() {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);  
};


class MajorityCategorizer : public Categorizer {

private:
   NO_DEFAULT_OPS(MajorityCategorizer);
   LOG_OPTIONS;

protected:
   // Protected member data
   InstanceList *instList;

   // Protected method
   // helper functions
   virtual void del_instance(ILPix& pix); // pix of this->instList

public:
   // Sanity checking
   void OK(int level=0) const;

   // Constructors  
   MajorityCategorizer(const MString& dscr, const InstanceList& il,
		       Bool keepList=FALSE);
   MajorityCategorizer(const MajorityCategorizer&, CtorDummy);
   MajorityCategorizer(const PerMajorityCategorizer_& data,
		       const SchemaRC& schema);

   // Returns a pointer to a deep copy of this MajorityCategorizer
   virtual Categorizer* clone() const;
   virtual ~MajorityCategorizer();

   // Categorize, Scoring
   virtual AugCategory categorize(const InstanceRC&) const;
   virtual CatDist *score() const;
   virtual CatDist *score(const InstanceRC&) const;
   virtual Bool supports_scoring() const { return TRUE; }

   // Equality
   virtual Bool operator==(const Categorizer& cat) const ;

   // Incremental add and delete.
   virtual void add_instance(const InstanceRC& instance);
   virtual void del_instance(const InstanceRC& instance);

   // Returns the class ID
   virtual int class_id() const { return CLASS_MAJORITY_CATEGORIZER; }

   // Access functions
   virtual Bool has_inst_list() const { return (instList != NULL); }
   virtual const InstanceList& get_inst_list() const;
   virtual Category get_category() const; // useful for tester

   // Allow dropping of the testing instance list.  Useful for testers
   virtual void drop_list();
   
   // Output methods
   virtual void display(MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref,
			Bool showInstanceList = FALSE) const;
   virtual void display_struct(MLCOStream& stream,
			       const DisplayPref& dp) const
      { display(stream, dp, FALSE); }

   // Persistence
   virtual Bool supports_persistence() const { return TRUE; }
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerMajorityCategorizer_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   
};

// declares operator<< to call display
DECLARE_DISPLAY(MajorityCategorizer)

#endif
