// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _MultiSplitCat_h_
#define _MultiSplitCat_h_ 1

#include <RDGCat.h>
#include <Persistence.h>
#include <NodeCategorizer.h>

class RootedCatGraph;
class RDGCategorizer;

class PerMultiSplitCat_ : public PerNodeCategorizer_ {
public:
   NO_DEFAULT_OPS(PerMultiSplitCat_);
   
   Array<Real> *voteWeights;
   
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);

   PerMultiSplitCat_()
      : voteWeights(NULL) {}
};


class MultiSplitCat : public NodeCategorizer {
   NO_DEFAULT_OPS(MultiSplitCat);

   // Member data
   PtrArray<NodeCategorizer*> *splitCats;
   PtrArray<Array<MStringRC>*> *splitCatNames;
   Array<Real> *voteWeights;
   DBG_DECLARE(int numChildren;)

public:
   MultiSplitCat(int noCat, const MString& dscr, const SchemaRC& sch,
		 PtrArray<NodeCategorizer*>*& categorizers,
		 PtrArray<Array<MStringRC>*>*& names,
		 Array<Real>*& weights);
   MultiSplitCat(const MultiSplitCat& source, CtorDummy);
   MultiSplitCat(const PerMultiSplitCat_& data, const SchemaRC& schema);
   
   virtual void OK(int level = 0) const;
   virtual ~MultiSplitCat();

   AugCategory branch(const InstanceRC&) const;  // will abort
   AugCategory *get_edge_label(int branchNum) const; // @@ need to deal!
   
   virtual AugCategory categorize(const InstanceRC&) const;
   virtual CatDist *score(const InstanceRC& inst) const
   { return NodeCategorizer::score(inst); }
   virtual CatDist *score(const InstanceRC&, Bool addLoss) const;
   virtual Bool supports_scoring() const { return TRUE; }

   virtual void build_distr(const InstanceList& instList);

   virtual void distribute_instances(const InstanceList& il,
				     Real pruningFactor,
				     Real &pessimisticErrors,
				     TDDTInducer::LeafDistType ldType,
				     Real leafDistParameter,
				     const Array<Real> *parentWeightDist,
				     Bool saveOriginalDistr);

   virtual void display_struct(MLCOStream& stream,
			       const DisplayPref& dp) const;

   const Array<Real>& get_weights() const { DBG(OK()); return *voteWeights; }
   void assign_weights(Array<Real>*& weights);

   PtrArray<NodeCategorizer*> *release_categorizers();
   PtrArray<Array<MStringRC>*> *release_names();

   MStringRC choice_name(int choice) const;
   
   virtual Categorizer* clone() const;

   virtual int class_id(void) const { return CLASS_MULTI_SPLIT_CATEGORIZER; }
   
   virtual Bool operator==(const Categorizer& rhs) const;

   // @@ For the short term option nodes do not use any attributes
   //    and are unmodified by projections.  This may change.
   virtual void set_used_attr(BoolArray&) const {};
   virtual void project(const BoolArray&) {};

   // Persistence
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerMultiSplitCat_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   virtual Bool supports_persistence() const { return TRUE; }
};

#endif
