// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _MultiThreshCat_h
#define _MultiThreshCat_h 1

#include <NodeCategorizer.h>
#include <DisplayPref.h>
#include <Array.h>

class PerMultiThresholdCategorizer_ : public PerNodeCategorizer_ {
public:
   NO_DEFAULT_OPS(PerMultiThresholdCategorizer_);
   
   int attrNum;
   Array<Real> *thresholds;
   Array<MString> *descs;

   PerMultiThresholdCategorizer_()
      : attrNum(UNDEFINED_INT), thresholds(NULL), descs(NULL) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


class MultiThresholdCategorizer : public NodeCategorizer {
   // Member data
   int attrNum;
   Array<Real> *thresholds;
   Array<MStringRC> descs;

   // Private methods
   void make_descriptions();
   NO_COPY_CTOR(MultiThresholdCategorizer);

public:
   MultiThresholdCategorizer(const SchemaRC& schema, int attributeNum,
			     Array<Real>*& inThresholds,
			     const MString& dscr);
   MultiThresholdCategorizer(const MultiThresholdCategorizer& source,
			     CtorDummy);
   MultiThresholdCategorizer(const PerMultiThresholdCategorizer_& data,
			     const SchemaRC& schema);
   
   ~MultiThresholdCategorizer() { delete thresholds; }

   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp=defaultDisplayPref) const;
   // Returns a pointer to a deep copy of this MultiThresholdCategorizer
   virtual Categorizer* clone() const;

   virtual AugCategory branch(const InstanceRC& inst) const;
   virtual AugCategory *get_edge_label(int branchNum) const;

   // Returns class id
   virtual int class_id() const { return CLASS_MULTITHRESH_CATEGORIZER; }
   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);
   virtual void OK(int /* level */ = 0) const;
   virtual Bool operator==(const Categorizer& rhs) const ;
   int attr_num() const { return attrNum; }
   const Array<Real>& get_thresholds() const {return *thresholds;}
   virtual void change_threshold(int which, Real threshold);
   virtual Array<MStringRC>* real_edge_strings() const;

   // persistence
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerMultiThresholdCategorizer_; }
   virtual void make_persistent(PerCategorizer_& data) const;
   virtual Bool supports_persistence() const { return TRUE; }

   // for testers only
   virtual AugCategory tester_branch(const InstanceRC& inst) const
   { return branch(inst); }
};

#endif
