// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file "NaiveBayesInd.c".

#ifndef NaiveBayesInd_h
#define NaiveBayesInd_h


#include <Inducer.h>
#include <AugCategory.h>
#include <Instance.h>
#include <InstanceList.h>
#include <NaiveBayesCat.h>

class NaiveBayesCat;  // includes definition of struct NBNorm


class NaiveBayesInd : public Inducer {
   NO_DEFAULT_OPS(NaiveBayesInd);

   // Member data
   NaiveBayesCat * categorizer;

   // options
   Bool useLaplace;                             // set whether to use
                                                // laplace correction
   Real mEstimateFactor;                        // noise in laplace correction
   UnknownIsValueEnum unknownIsValue;
   Real klThreshold;
   Real noMatchesFactor;
   Bool useEvidenceProjection;
   Real evidenceFactor;   
public:
   NaiveBayesInd(const MString& description);
   NaiveBayesInd(const NaiveBayesInd& source, CtorDummy);
   virtual ~NaiveBayesInd();

   virtual int class_id() const { return NAIVE_BAYES_INDUCER; }
   
   virtual Bool was_trained(Bool fatalOnFalse = TRUE) const;
   virtual const Categorizer& get_categorizer() const;
   virtual Categorizer *release_categorizer();
   virtual void set_use_laplace(Bool laplace);
   virtual Bool get_use_laplace() const { return useLaplace; }
   virtual void set_m_estimate_factor(Real factor);
   virtual Real get_m_estimate_factor() const { return mEstimateFactor; }
   virtual void set_user_options(const MString& preFix);
   virtual void train();

   virtual Bool can_cast_to_incr_inducer() const {return FALSE;}
   virtual void display(MLCOStream& stream = Mcout,
			    const DisplayPref& dp = defaultDisplayPref) const;
   virtual Inducer* copy() const; 

   virtual void set_unknown_is_value(UnknownIsValueEnum unk) {
      unknownIsValue = unk;
   }
   virtual UnknownIsValueEnum get_unknown_is_value() const {
      return unknownIsValue;
   }
   virtual void set_kl_threshold(Real th) { klThreshold = th; }
   virtual Real get_kl_threshold() const { return klThreshold; }

   virtual void set_no_matches_factor(Real f) {
      noMatchesFactor = f; }
   virtual Real get_no_matches_factor() const { return noMatchesFactor; }

   virtual void set_use_evidence_projection(Bool b)
   { useEvidenceProjection = b;}
   virtual Bool get_use_evidence_projection() const
   { return useEvidenceProjection; }

   virtual void set_evidence_factor(Real f) {
      evidenceFactor = f; }
   virtual Real get_evidence_factor() const { return evidenceFactor; }
};

DECLARE_DISPLAY(NaiveBayesInd)
#endif









