// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _ODTCategorizer_h
#define _ODTCategorizer_h 1

#include <AugCategory.h>
#include <AttrOrder.h>
#include <DTCategorizer.h>
#include <MajorityCat.h>
#include <LogOptions.h>

class LeafCategorizer;

class PerODTCategorizer_ : public PerDTCategorizer_ {
   //void read(MInStore& in);
   //void write(MOutStore& out);
};


// In a given probability distribution when we look for the best category, we
// can have no weight, a tie, or a clear majority category.
enum BestCategoryCase { unknownCase = -2, zeroWeight, tie, clearMajority };

// Needed since we wish to return a set of values in a recursive function.
struct BestCategoryAndCase {
   BestCategoryCase case_id;
   Category category;
   Bool done;
   BestCategoryAndCase(BestCategoryCase ca, Category cat, Bool d)
            { case_id = ca; category = cat; done = d; }
};

class ODTCategorizer : public DTCategorizer {
   NO_COPY_CTOR(ODTCategorizer);
   LOG_OPTIONS;

public:

   // any change to this enum should be reflected in the MEnum in ODTInducer.c
   enum MajorityType { no_majority, first, local, global, local2 };

private:
   // Member Variables
   AttrOrder attrOrder;
   AugCategory augCateg; // Cache variable. See Comments section in .c file
   MajorityType tieBehavior;
   MajorityType zeroWeightBehavior;

   // helper functions
   BestCategoryAndCase find_best_category_by_behaviour(const Array<Real>
						       &distr) const;

   BestCategoryAndCase categorize_helper(const InstanceRC &inst,
				  const NodePtr &currentNode,
				  Array<Real> &distr) const;

   virtual MajorityCategorizer &
     find_or_create_path_to_leaf(const InstanceRC& inst, Bool add);
   NodePtr create_child(NodePtr parent, const AugCategory childEdgeLabel,
			InstanceRC inst, int attrNum, Bool isLeaf);
   LeafCategorizer* create_leaf_categorizer(InstanceList *&il) const ;

   void create_table_viz_files(MLCOStream& Hdr, MLCOStream& Body) const;
   void create_table_viz_data_file(const NodePtr &currentNode,
			   const MString soFar, MLCOStream &Out) const;

public:

   // Check invariants
   void OK(int level=0) const;

   // Constructor, Copy Constructor, Destructor
   ODTCategorizer(DecisionTree*& dt, const MString& dscr, int numCat,
		  const SchemaRC& schema, const AttrOrder &ao,
		  MajorityType defTieBehavior,
		  MajorityType defZeroWeightBehavior);
   ODTCategorizer(const ODTCategorizer& source, CtorDummy /*dummyArg*/);
   ODTCategorizer(const PerODTCategorizer_& data,
		  const SchemaRC& schema,
		  NodeInfo*& prototype,
		  const Array<NodeInfo *>& nodes);
   
   // Makes a deep copy of this ODTCategorizer
   virtual Categorizer* clone() const;   
   ~ODTCategorizer() {}

   // Categorize
   virtual AugCategory categorize(const InstanceRC& inst) const;

   // @@ Perhaps we should think of an IncrCategorizer class, to make explicit
   // the hidden relationship between an IncrInducer and its corresponding
   // categorizer.

   // Incremental add and delete
   virtual void add_instance(const InstanceRC &);   
   virtual void del_instance(const InstanceRC& instance);

   // Equality functions
   virtual Bool operator==(const Categorizer &rhs) const ;

   // Returns the class ID
   virtual int class_id() const { return CLASS_ODT_CATEGORIZER; }

   // Access functions
   MajorityType get_zero_weight_behavior() const { return zeroWeightBehavior; }
   void set_zero_weight_behavior(MajorityType type)
                                 { zeroWeightBehavior = type; }
   MajorityType get_tie_behavior() const { return tieBehavior; }
   void set_tie_behavior(MajorityType type) { tieBehavior = type; }

   // Output functions
   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp = defaultDisplayPref)
      const;

   // Persistence
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerRDGCategorizer_; }
   //virtual void make_persistent(PerCategorizer_& data) const;
   
};

#endif
