// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _OneRDiscretizor_h
#define _OneRDiscretizor_h 1

#include <RealDiscretizor.h>
#include <DynamicArray.h>
#include <Attribute.h>


class LabelValueCounts : private Array<Category> {
   NO_COPY_CTOR(LabelValueCounts);

   Bool tie;
   int maxValue;
   Category maxCategory;
   Category& index(int i) { return Array<Category>::index(i);}
   // The above is to avoid warnings since we need to override the const index
   // method
public:
   LabelValueCounts(int size);
   void increment(Category category);
   void reset();
   Bool tied_majority() const { return tie; }
   int majority_value() const; 
   Category majority_category() const;
   const Category& index(int i) const { return Array<Category>::index(i); } 
   int size() const { return Array<Category>::size(); }
   LabelValueCounts& operator= (const LabelValueCounts& rhs)
   { Array<Category>::operator=(rhs); tie = rhs.tie; maxValue=rhs.maxValue;
     maxCategory = rhs.maxCategory; return *this; }
   LabelValueCounts& operator+= (const LabelValueCounts&);
};

class AttrAndLabel {
   NO_COPY_CTOR(AttrAndLabel);
public:   
   Real attrVal;
   Category label;
   
   AttrAndLabel() :attrVal(-REAL_MAX),label(UNKNOWN_CATEGORY_VAL){}
   AttrAndLabel& operator=(const AttrAndLabel& rhs);
   // The following comparison operators return ints, so they should be
   //   used for sorting.  Also, there's no need for an operator!=.
   int operator<(const AttrAndLabel& item2) const
   {  DBG(ASSERT(label != UNKNOWN_CATEGORY_VAL && attrVal != -REAL_MAX));
      return (attrVal < item2.attrVal);}
   int operator==(const AttrAndLabel& rhs) const
   {  DBG(ASSERT(label != UNKNOWN_CATEGORY_VAL && attrVal != -REAL_MAX));
      return (attrVal == rhs.attrVal);}
};


class OneR : public RealDiscretizor{
   NO_COPY_CTOR(OneR);
   NO_OPERATOR_ASSIGN(OneR);
   
   int small;
protected:
   void build_thresholds(RealAndLabelColumn*& column, Real trim); 
public:
   static const int defaultSmall;
   OneR(int attrNum, const SchemaRC& schema, int small = defaultSmall);
   OneR(const OneR& source, CtorDummy dummyArg);
   int get_small() const { return small;}
   void set_small(int val) { small = val;}
   virtual Bool operator== (const RealDiscretizor& rhs) const;
   Bool operator!=(const RealDiscretizor& rhs) const
      {return ! ((*this)== rhs);}
   int class_id() const { return CLASSID_HOLTE1RDISC;}
   RealDiscretizor* clone() const;
};

PtrArray<RealDiscretizor*>*
create_OneR_discretizors(LogOptions& logOptions,
			 const InstanceList& instList,
			 const Array<int>& smallValues,
			 const Array<Real>& columnTrim);

PtrArray<RealDiscretizor*>*
create_OneR_discretizors(LogOptions& logOptions,
			 const InstanceList& instList,
			 int small = OneR::defaultSmall,
			 Real trim = 0);

#endif
