// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _OptionServer_h
#define _OptionServer_h 1

#include <StringMap.h>

// the following class contains a string and an integer.  The integer is
// to be used as an access count for options.
class OptionAccess_ {
public:
   MString value;
   int count;
   void display(MLCOStream& stream=Mcout) const {
      stream << value; } // required for templates
};
DECLARE_DISPLAY(OptionAccess_)


// the OptionServer class provides a layer which separates the other option
// classes (such as MOption) from the actual source of the options.
class OptionServer {

   NO_COPY_CTOR(OptionServer);
   NO_OPERATOR_EQ(OptionServer);
   StringMap<OptionAccess_> optionMap;
   Bool useEnv;   // TRUE = fall back on environment if option missing
   OptionServer *backup;  // backup server used for extra options if
                          // none found
   
public:
   static Bool displayUnused;

   OptionServer();
   ~OptionServer();
   
   void set_option(const MString& optName, const MString& optVal);
   void set_option(const MString& optPair);

   void include_options(MLCIStream& stream);
   void enable_environment(Bool useEnv);
   Bool get_environment_state(void) const { return useEnv; }
   void set_backup(OptionServer *backupServer);
   const OptionServer *get_backup() { return backup; }
   
   Bool get_option(const MString& optName, MString& optVal) const;
   void flag_use(const MString& optName, const MString& optVal);
   Array<MString> *get_option_names() const;
   
   void write_options(MLCOStream& stream) const;
   void display(MLCOStream& stream) const;
   void display_unused(MLCOStream& stream=Mcout) const;
   void display_used(MLCOStream& stream=Mcout) const;
   void display_debug(MLCOStream& stream=Mcout) const;
};

DECLARE_DISPLAY(OptionServer)
   
#endif

