// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _PartitionEvaluator_h
#define _PartitionEvaluator_h 1

#include <Attribute.h>
#include <Array2.h>
#include <LogOptions.h>
#include <SplitAttr.h>
#include <ValPartition.h>

class InstanceList;
class BagCounters;


class PartitionEvaluator {
   NO_DEFAULT_OPS(PartitionEvaluator);
   LOG_OPTIONS;

   // Member data
   Real minSetSize;
   NominalAttrInfo *attrInfo;           //@@ was reference--changed due to MSVC
   const BagCounters *testCounters;     //@@ was *const--changed due to MSVC
   const BagCounters *trainCounters;    //@@ was *const--changed due to MSVC
   int numInstances, numTestInstances;  //@@ was const--changed due to MSVC
   int numLabelVals;                    //@@ was const--changed due to MSVC
   SplitAttr& splitter;

private:
   // Private methods
   void find_counters(const ValuePartition& partition,
		      const BagCounters& counters, 
		      Array2<Real>& splitAndLabelDist) const;
   Real attr_set_count(const Array<NominalVal> &set,
		       const BagCounters& counters) const;
   Real attr_val_count(const Array<NominalVal> &set, NominalVal label,
		       const BagCounters& counters) const;

public:
   PartitionEvaluator(SplitAttr& split,
		      const InstanceList& instList,
		      const InstanceList *testSet = NULL);
   ~PartitionEvaluator() { delete attrInfo; }
   Real evaluate(Bool computeReal, Bool computeEstimated,
		 const ValuePartition& subset) const;

   Real evaluate_instances(const ValuePartition& partition,
			   const BagCounters& counters) const;

   int get_attr_num() const { return splitter.get_attr_num(); }
   const NominalAttrInfo& get_attr_info() const {
      ASSERT(attrInfo); return *attrInfo; }
   Bool has_train_counters() const { return trainCounters != NULL; }
   Bool has_test_counters() const { return testCounters != NULL; }
   const BagCounters& train_counters() const;
   const BagCounters& test_counters() const;
   void set_min_set_size(Real size) { minSetSize = size; }
   Real get_min_set_size() const { return minSetSize; }
   int  num_label_vals() const {return numLabelVals;}
   void reset() {splitter.reset();}
};

#endif

