// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _PartitionSplit_h
#define _PartitionSplit_h 1

#include <LogOptions.h>
#include <Attribute.h>

class InstanceList;
class PartitionEvaluator;

class PartitionSplit { // ABC
   NO_DEFAULT_OPS(PartitionSplit);
   LOG_OPTIONS;

   // Member data (also see protected and private data)
   Bool separateUnknowns;  // @@ move this to the search itself
protected:
   // Protected member data
   Array<NominalVal>* partition;
   Real partitionScore;

   // Protected method
   void reset() {delete partition; partition = NULL; 
                 partitionScore = -REAL_MAX;}
public:
   //Public data
   static const Bool defaultSeparateUnknowns;

   // Methods
   void OK() const;
   PartitionSplit() : partition(NULL), partitionScore(-REAL_MAX),
                      separateUnknowns(defaultSeparateUnknowns) { }
   virtual ~PartitionSplit() {DBG(OK()); reset();}
   // The evaluator knows about the instance list.
   virtual void subset_split(PartitionEvaluator& evaluator) = 0;
   virtual void set_user_options(const MString& prefix);
   void set_separate_unknowns(Bool separate) { separateUnknowns = separate; }
   Bool get_separate_unknowns() const { return separateUnknowns; }
   Bool exist_partition() const {return partition != NULL;}
   const Array<NominalVal>& get_partition() const;
   Array<NominalVal> *release_partition();
   Real partition_score() const {return partitionScore;}
   virtual void display(MLCOStream& stream = Mcout) const;
};

DECLARE_DISPLAY(PartitionSplit)

#endif
