// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _PerfData_h
#define _PerfData_h 1

#include <ErrorData.h>
#include <CatTestResult.h>

class CatTestResult;

class PerfData {
   NO_DEFAULT_OPS(PerfData);

   // Member data
   ErrorData error;
   ErrorData mse;
   ErrorData mae;
   ErrorData normalizedLoss;
   ErrorData loss;
   ErrorData logLoss;
   
   Array2<Real> *confusionMatrix;

   // cost is number of runs of inducer needed to get result
   int evalCost;

   void initialize_confusion_matrix(const Array2<Real>& confMatrix);
   void accumulate_confusion_matrix(const Array2<Real>& confMatrix);
   
public:
   virtual void OK(int level = 0) const;
   
   PerfData();
   virtual ~PerfData();
   
   void insert(const CatTestResult& item);
   void insert_error(Real err);
   void insert_cost(int cost) { evalCost += cost; }

   // set test set error information
   void set_test_set(const CatTestResult& item);
   void set_test_set(Real error, Real totalTestWeight);
   Bool has_test_set() const { return error.has_test_set(); }
   Bool has_perf_test_set() const { return mse.has_test_set(); }
   
   void display_conf_matrix(MLCOStream& stream, const SchemaRC& schema) const;
   void display(MLCOStream& stream = Mcout, Bool dispLogLoss = FALSE,
		Real trim = 0, int precision = ErrorData::defaultPrecision)
      const;
   void display_error(MLCOStream& stream = Mcout, Real trim = 0,
		      int precision = ErrorData::defaultPrecision) const;
   void display_non_error(MLCOStream& stream = Mcout,
			  Bool dispLogLoss = FALSE,
			  Real trim = 0,
			  int precision = ErrorData::defaultPrecision) const;
   Bool perf_empty() const { DBG(OK()); return mse.size() == 0; }
   Bool empty() const { return error.size() == 0; }

   virtual void append(const PerfData& other);
   virtual void clear();

   int size() const { DBG(OK()); return error.size(); }
   ErrorData& get_error_data() { return error; }

   const ErrorData& get_error_data() const { return error; }
   const ErrorData& get_mean_squared_error_data() const { return mse; }
   const ErrorData& get_mean_absolute_error_data() const { return mae; }
   const ErrorData& get_normalized_loss_data() const { return normalizedLoss; }
   const ErrorData& get_loss_data() const { return loss; }
   
   const Array2<Real>& get_confusion_matrix() const;
   
   int get_cost() const { return evalCost; }
   void set_cost(int cost) { evalCost = cost; }
};

DECLARE_DISPLAY(PerfData)

#endif

