// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _PerfEstState_h
#define _PerfEstState_h 1

#include <DynamicArray.h>
#include <PerfEstDispatch.h>
#include <PerfData.h>
#include <State.h>
#include <InstanceList.h>
#include <BaseInducer.h>

// Use an offset to avoid conflicts with both Categorizer.h and BaseInducer.h
#define PERF_EST_INFO 2001
#define C45AP_INFO   2002
#define DISC_INFO    2003
#define FSS_INFO     2004
#define ORDER_INFO   2005
#define WEIGHT_INFO  2006

class PerfEstInfo : public StateGlobalInfo {
   LOG_OPTIONS;
   
public:
   // Public member data
   //   for use by PerfEstState::eval().
   BaseInducer *inducer;
   InstanceList *trainList;
   InstanceList *testList;
   PerfEstDispatch perfEst;
   int seed;
   Bool useCompound;
   Real complexityPenalty;

   // Methods
   PerfEstInfo();

   // functions required by CompState (see CompState.c for details)
   virtual int lower_bound(int index) = 0;
   virtual int upper_bound(int index) = 0;
   virtual void display_values(const Array<int>& values, MLCOStream& out =
			       Mcout) const = 0;
   virtual Bool use_compound() { return useCompound; }

   virtual int class_id() const { return PERF_EST_INFO; }
};
   

class PerfEstState : public State<Array<int>, PerfEstInfo> {

   // Member data
   PerfData perfData;
   int numEvaluations;   
public:
   PerfEstState(Array<int>*& initialInfo, const PerfEstInfo& gI);
   virtual ~PerfEstState() { }

   virtual void set_final_state();

   virtual Real eval(PerfEstInfo *, Bool computeReal=TRUE,
		     Bool computeEstimated=TRUE);

   virtual void display_info(MLCOStream& stream = Mcout) const;
   virtual void display_stats(MLCOStream& stream,
			      PerfEstInfo *acInfo) const;
   virtual void display_for_graph(MLCOStream& stream = Mcout) const;   
   virtual Real get_error() const
   { return perfData.get_error_data().mean(); }
   virtual Real get_error_std_dev() const
   { return perfData.get_error_data().std_dev_of_mean(); }
   virtual Real get_test_set_fitness() const
   { return perfData.get_error_data().test_set_error(); }
   virtual Real get_theoretical_std_dev() const
   { return perfData.get_error_data().theo_std_dev(); }

   virtual void pre_eval(PerfEstInfo *) { };
   virtual void construct_lists(PerfEstInfo *, InstanceList *& trainList, 
				InstanceList *& testList)
      { (void)trainList;  (void)testList; }
   virtual void destruct_lists(PerfEstInfo *, InstanceList *trainList,
			       InstanceList *testList)
      { (void)trainList;  (void)testList; }
};


#endif





