// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _PerfEstimator_h
#define _PerfEstimator_h 1

#include <LogOptions.h>
#include <MRandom.h>
#include <ErrorData.h>

extern const MString DEFAULT_DATA_EXT;
extern const MString DEFAULT_NAMES_EXT;
extern const MString DEFAULT_TEST_EXT;

class BaseInducer;
class InstanceList;
class CatTestResult;
class PerfData;


class PerfEstimator {
   NO_COPY_CTOR(PerfEstimator);
   LOG_OPTIONS;
   RAND_OPTIONS;
protected:
   // Protected member data
   MString dumpStem;    // Don't dump if ""
   PerfData *perfData;

   // Protected methods
   // dumpSuffix is the file stem suffix (before .{names,data,test}
   // to add to dumpStem.  It will not dump if dumpStem is ""
   virtual void dump_data(const MString& dumpSuffix,
			  const MString& descr,
			  const InstanceList& trainList,
			  const InstanceList& testList);

   // NULL value for localErrorData means don't update error.
   //   virtual void update_acc_data(int numCorrect, int numIncorrect,
   //				ErrorData *localErrorData);

   // show how stratified this particular train/test pair are
   void show_stratification(const InstanceList& trainPart,
			    const InstanceList& testPart) const;
   
   // train_and_test should call dump_data
   // This is protected so we can test outside programs like C4.5
   virtual Real train_and_test(BaseInducer& inducer,
					 InstanceList* trainList,
					 const InstanceList& testList,
					 const MString& dumpSuffix,
					 PerfData *localPerfData);

public:
   PerfEstimator();
   virtual ~PerfEstimator();
   virtual MString description() const = 0;
   virtual Real estimate_performance(BaseInducer& inducer,
				  InstanceList& dataList) = 0;

   // estimate performance from a series of files
   virtual Real estimate_performance(BaseInducer& inducer,
				     const MString& fileStem) = 0;
   // Estimate performance from a fileStem with assumed suffixes
   //    -#.{names,data,test} where # denotes a sequence number.
   virtual Real estimate_file_performance(BaseInducer& inducer, int numFiles,
					  const MString& fileStem,
					  PerfData *localPerfData);
   // Estimate one file from a fileStem
   virtual Real single_file_performance(BaseInducer& inducer,
					const MString& fileStem,
					PerfData *localPerfData);

   virtual void check_error_data() const;
   virtual void check_perf_data() const;
   virtual const ErrorData& get_error_data() const;
   virtual const PerfData& get_perf_data() const;
   
   virtual void display_error_data(MLCOStream& stream = Mcout,
				   Real trim = 0,
				 int precision = ErrorData::defaultPrecision)
      const;
   virtual void display_perf_data(MLCOStream& out = Mcout,
				  Real trim = 0,
				  int precision = ErrorData::defaultPrecision)
      const;

   virtual Real error(Real trim = 0) const;
   virtual Real error_std_dev(Real trim = 0) const;
   
   virtual void dump_files(InstanceList& dataList, const MString& fileStem);

   virtual void display(MLCOStream& stream = Mcout, Bool descrip = FALSE,
			Real trim = 0.0, 
			int precision = ErrorData::defaultPrecision) const;
};

DECLARE_DISPLAY(PerfEstimator)

#endif
