// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _Persistence_h
#define _Persistence_h 1

#include <InstanceList.h>
#include <CatDist.h>

static const int MajorVersion = 2;
static const int MinorVersion = 4;


// MInStore is a class for reading from persistent storage.  Reading
// functions are defined for several basic types.
class MInStore {
   NO_DEFAULT_OPS(MInStore);

   // Member data
   MLCIStream in;
   int majorVersion, minorVersion;

   // Private method
   void get_header();   
public:
   MInStore(const MString& filename);

   // data types
   void get_int(int& i);
   void get_real(Real& r);
   void get_stored_real(StoredReal& r);
   void get_string(MString& str);
   void get_bool(Bool& b);
   void get_real_attr_value(AttrValue_& av);
   
   // complex types; these return the type
   SchemaRC get_schema();
   AttrInfo *get_attr_info();
   
   // separator--insures presence of separator in input
   void get_separator();

   // tell us what version we're dealing with
   int get_major_version() { return majorVersion; }
   int get_minor_version() { return minorVersion; }

   // return TRUE if the version is at least the one specified
   Bool version_at_least(int major, int minor) {
      return (majorVersion > major ||
	      (majorVersion==major && minorVersion >= minor));  }

   // Force the stream to close
   void close() { in.close(); }
};


// MOutStore defines an output mechanism for all persistent classes
class MOutStore {
   NO_DEFAULT_OPS(MOutStore);

   // Member data
   MLCOStream out;
   int majorVersion, minorVersion;

   // Private method
   void put_header();   
public:
   MOutStore(const MString& filename);

   // data types
   void put_int(int i);
   void put_real(Real r);
   void put_stored_real(StoredReal r);
   void put_string(const MString& str);
   void put_bool(Bool b);
   void put_real_attr_value(const AttrValue_ av);
   
   // complex types
   void put_schema(const SchemaRC& schema);
   void put_attr_info(const AttrInfo& info);
   
   // separator--useful for sanity checks
   void put_separator();
   void put_blank();

   // Force the stream to close
   void close() { out.close(); }
};


#endif
