// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _ProjGraph_h
#define _ProjGraph_h 1

#include <DblLinkList.h>
#include <LEDA/graph.h>
#include <MLCStream.h>
#include <DisplayPref.h>
#include <LogOptions.h>
#include <ProjLevel.h>

// LEDA functions
// This should be "const ProjSet*&" in Print but leda_access is
// returning non-const version.  This works well.
void Print(ProjSet*& nodeInfo, ostream& stream);

// Required for LEDA, not used.  Had compiler warnings when
//   it was in the .c file, so moved it here.
inline void Read(ProjSet*& /*nodeInfo*/, istream& /*stream*/)
{ err << "ProjGraph::Read: sorry, not implemented" << fatal_error;}

typedef node NodePtr;
typedef edge EdgePtr;

class ProjGraph : public GRAPH<ProjSet*, int> {
   NO_DEFAULT_OPS(ProjGraph);
   LOG_OPTIONS;

   // Member data
   DynamicArray<NodePtr> nodeArray;

   // Private methods
   void process_DotPoscript_preferences(MLCOStream& stream,
					const DisplayPref& pref) const;
protected:
   void free();
   virtual void process_XStream_display(const DisplayPref& dp) const;
   virtual void process_DotPostscript_display(MLCOStream& stream,
					      const DisplayPref& dp) const;
   virtual void process_DotGraph_display(MLCOStream& stream,
					 const DisplayPref& dp) const;
   virtual void convertToDotFormat(MLCOStream& stream,
				   const DisplayPref& pref)const;

   void init_coloring(int maxColors);
   void assign_color(NodePtr n, int color);
   NodePtr get_node_to_color();
   
   
public:
   ProjGraph();
   virtual ~ProjGraph();
   virtual MString graph_options(NodePtr v) const;

   // Graph Manipulation
   NodePtr create_node(ProjSet*& proj);
   void connect(NodePtr from, NodePtr to, int edge);
   void add_conflict_edges();

   // Accessor Functions.
   ProjSet& get_proj_set(NodePtr node) const;

   // Graph Coloring
   Bool color(int maxColors);

   // Display Functions
   virtual void display(MLCOStream& stream = Mcout,
                        const DisplayPref& dp = defaultDisplayPref) const;

   // Function to retrieve colored sets
   Array<ProjSet *> *get_colored_sets(int max);
};

DECLARE_DISPLAY(ProjGraph)


#endif
