// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _QuarkTable_h
#define _QuarkTable_h 1

#include <DynamicArray.h>
#include <UnivHashTable.h>
#include <Attribute.h>


class QuarkHashData {
public:
   int key;
   Category value;

   QuarkHashData() : key(-1), value(-1) { }
   QuarkHashData(int akey, Category avalue) :
      key(akey), value(avalue) { }
   QuarkHashData& operator=(const QuarkHashData& other) {
      key = other.key;  value = other.value;  return *this; }
   Bool contains_key(int akey) const { return key == akey;}
   int get_key() const { return key; }
   Category get_value() const { return value; }
   void set_value(int val) { value = val; }
   void display(MLCOStream& stream=Mcout) const;
};

DECLARE_DISPLAY(QuarkHashData)

class QuarkHashTable : public HashTable<int, QuarkHashData> {
public:
   QuarkHashTable(int sizeHint) :
      HashTable<int, QuarkHashData> (sizeHint) { }

   virtual ~QuarkHashTable() { }
   
   virtual int hash(const int& key) const {
      return key % size();
   }
};
   
   
class QuarkTable {
   NO_COPY_CTOR(QuarkTable);
   NO_OPERATOR_EQ(QuarkTable);
   QuarkHashTable table;
   
public:
   void OK() const;

   // Two constructors;  empty constructor, and constructor of
   // pre-built key/value pairs
   QuarkTable(int sizeHint = 512);
   QuarkTable(const Array<QuarkHashData>& pairs);

   // Add a mapping to the hash table.  Abort if the mapping was already
   // present.
   void add(int quark, Category value);

   // Retrieve the mapping for a quark
   Category get(int quark) const;

   // Retrieve a mapping.  Return TRUE if the mapping is present, FALSE if
   // the mapping is missing.  Return the value (on a TRUE return) through
   // the value parameter.
   Bool get(int name, Category& value) const;
};

     
#endif


