// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _RealDiscretization_h
#define _RealDiscretization_h 1

#include <InstanceList.h>
#include <Persistence.h>
#include <RealAndLabelColumn.h>
#include <class_id.h>

//Class ID's for discretizors
//These are global for the same reason Categorizer IDs are global -
//  see Categorizer.h for details.
//We start with 100 to avoid conflicting with Categorizer.
static const int CLASSID_ENTROPYDISC     = DISCRETIZOR_ID_BASE + 1;
static const int CLASSID_EQUALWIDTHDISC  = DISCRETIZOR_ID_BASE + 2;
static const int CLASSID_HOLTE1RDISC     = DISCRETIZOR_ID_BASE + 3;
static const int CLASSID_C45DISC         = DISCRETIZOR_ID_BASE + 4;
static const int CLASSID_T2DISC          = DISCRETIZOR_ID_BASE + 5;
static const int CLASSID_PERSISTDISC     = DISCRETIZOR_ID_BASE + 6;
static const int CLASSID_EQUALWEIGHTDISC = DISCRETIZOR_ID_BASE + 7;

class PerRealDiscretizor_ {
public:
   NO_DEFAULT_OPS(PerRealDiscretizor_);
   int attrNum;
   NominalAttrInfo *nai;
   RealAttrInfo *rai;
   MString descr;
   DynamicArray<RealAttrValue_> *thresholds;

   PerRealDiscretizor_()
      : attrNum(UNDEFINED_INT), nai(NULL), rai(NULL), descr(EMPTY_STRING),
      thresholds(NULL) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


class RealDiscretizor{  //ABC
   LOG_OPTIONS;
protected:
   NO_DEFAULT_OPS(RealDiscretizor);

   // Member data
   int attrNum;
   NominalAttrInfo* nai;
   RealAttrInfo *rai;
   MString descr;
   DynamicArray<RealAttrValue_>* thresholds;

   // Private methods
   void base_copy(const RealDiscretizor& source);
   void check_thresholds();
   virtual void build_thresholds(RealAndLabelColumn*& column,Real trim) = 0;
public:
   RealDiscretizor(int attrNum, const SchemaRC& schema);
   RealDiscretizor(const RealDiscretizor& source, CtorDummy);
   RealDiscretizor(const PerRealDiscretizor_& data);
   // Note:  The following, though pure virtual, has an implementation
   //   (defined in RealDiscretizor.c).  This implementation MUST be invoked
   //   by the operator== of all directly-derived classes.
   virtual Bool operator== (const RealDiscretizor& rhs) const = 0;
   Bool operator!= (const RealDiscretizor& rhs) const
      {return !operator==(rhs);}
   virtual int class_id() const = 0;
   virtual ~RealDiscretizor();
   virtual void OK(int /*level*/ = 0) const;
   virtual void create_thresholds(RealAndLabelColumn*& column, Real trim);
   virtual void set_thresholds(Array<Real>& newVals);   
   virtual Array<Real> *get_thresholds(void) const;
   virtual NominalAttrInfo* real_to_nominal_info() const;
   virtual NominalAttrValue_ discretize_real_val(const InstanceRC& inst)
     const;
   virtual void set_description(const MString& desc){descr = desc;}
   virtual MString get_description() const { return descr;}
   virtual void display(MLCOStream& stream = Mcout) const;
   virtual int num_intervals_chosen();
   virtual RealDiscretizor* clone() const = 0;
   virtual void util_display(MLCOStream& stream = Mcout) const;

   virtual void make_persistent(PerRealDiscretizor_& data) const;
   virtual void set_attr_num(int newNum) { attrNum = newNum; }
};


SchemaRC gen_discretized_schema(const InstanceList& instList,
				PtrArray<RealDiscretizor*>* const disc);

//@@ this should be a method in Instance.c called "discretize"
void discretize_instance(const PtrArray<RealDiscretizor*>* const disc,
			 const SchemaRC& discretizedSchema,
			 InstanceRC& instanceToBeDiscretized);

//@@ this should be a method of InstanceList.c called "discretize"
InstanceList* discretize_instance_list(const InstanceList& source,
				       PtrArray<RealDiscretizor*>*
				       const discretizors); 

DECLARE_DISPLAY(RealDiscretizor)
#endif



