// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _RipperInducer_h
#define _RipperInducer_h 1

#include <Inducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <TmpFileName.h>

// The first parameter should be set to the command line that
//   will execute RIPPER.
Bool run_ripper(const LogOptions& logOptions, const MString& ripperPgm, 
		InstanceList& trainingSet, InstanceList& testSet,
		Real& errorRate, int& numRules, int& numConditions);


// This version is intended to be used if you're running RIPPER a lot.
// Profiling showed that file creation (creat) and deletion (unlink) were
//   taking up to 40% of the time in some cases, so this version takes open
//   streams, writes to them, and truncates.
Bool run_ripper(const LogOptions& logOptions, const MString& ripperPgm,
		InstanceList& trainingSet,
		InstanceList& testSet, Real& errorRate, int& numRules,
		int& numConditions, MString& tmpFileStem,
		MLCOStream& namesStream,
		MLCOStream& dataStream, MLCOStream& testStream);

class RipperInducer : public BaseInducer {
   NO_DEFAULT_OPS(RipperInducer);

   // Member data (also see public data)
   MString pgmName;
   MString pgmFlags;

   PtrArray<TmpFileName *> *tempNames;
   MLCOStream* namesStream;
   MLCOStream* dataStream;
   MLCOStream* testStream;
   // @@ RIPPER accepts an optional grammar file - need to have
   // a good way to handle this.  The following stream is never
   // actually used in the current interface.
   // MLCOStream* gramStream;

   // Private methods
   void init_files();
   void end_files();
public:

   // Public data
   static const MString defaultPgmName;
   static const MString defaultPgmFlags;

   // Methods
   RipperInducer(const MString& description, 
		 const MString& theFlags = defaultPgmFlags,
		 const MString& aPgmName = defaultPgmName);
   virtual ~RipperInducer();
   virtual int class_id() const { return RIPPER_INDUCER; }

   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testSet);
};


#endif
