// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _RootedCatGraph_h
#define _RootedCatGraph_h 1

#include <CatGraph.h>

class PerRootedCatGraph_ : public PerCatGraph_ {
   NO_DEFAULT_OPS(PerRootedCatGraph_);
public:
   int rootIndex;

   PerRootedCatGraph_()
      : rootIndex(UNDEFINED_INT) {}
   void read(MInStore& in);
   void write(MOutStore& out);
};

class RootedCatGraph : public CatGraph {
   NO_COPY_CTOR2(RootedCatGraph, CatGraph());
   NO_OPERATOR_ASSIGN(RootedCatGraph);

   // Member data
   NodePtr root;
public:
   virtual void OK(int level = 0) const;
   RootedCatGraph(Bool isGraphSparse = FALSE);
   RootedCatGraph(CGraph& aGraph, Bool isGraphSparse = FALSE);
   RootedCatGraph(const RootedCatGraph& source, CtorDummy);
   RootedCatGraph(const PerRootedCatGraph_& data,
		  NodeInfo*& prototype,
		  const Array<NodeInfo *>& nodes,
		  Bool isGraphSparse = FALSE);

   virtual CatGraph *clone() const
   { return new RootedCatGraph(*this, ctorDummy); }
   
   virtual ~RootedCatGraph();
   void set_root(const NodePtr);
   NodePtr get_root(Bool abortOnNoRoot = FALSE) const;
   virtual void display(Bool hasNodeLoss, Bool hasLossMatrix,
			MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const;
   virtual void display(MLCOStream& stream = Mcout,
			const DisplayPref& dp = defaultDisplayPref) const
   { display(FALSE, FALSE, stream, dp);}
   void display_multi_split_differences(MLCOStream& stream) const;
   // trivial_edge returns TRUE if the edge is unknkown and leads
   //   to a node with no instances.  Such nodes are not displayed
   //   under certain displays and need not be counted.
   virtual Bool trivial_edge(EdgePtr e) const;
   // A NULL root means start at the graph's root
   virtual int num_nontrivial_nodes(NodePtr fromRoot = NULL) const;
   virtual int num_nontrivial_leaves(NodePtr fromRoot = NULL) const;
   virtual int longest_path_from_root() const
      {return longest_path(get_root(TRUE)); }

   virtual void rooted_make_persistent(PerRootedCatGraph_& data) const;
};

#endif
