// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.
/***************************************************************************
  Description  : StateSpaceSearch is the base class for search methods.
                   There are three class hierarchies in the search engine
		   classes.  The first is the Search heirarchy, of which
		   this class is the base.  The derived classes BFSearch,
		   HCSearch, and SASearch implement specific search methods
		   based on this framework.  The second in the State
		   hierarchy, which defines properties of a single state
		   in the search space.  PerfEstState, CompState, and other
		   state classes for specific operations are derived from
		   State.  A related class StateSpace stores a graph of
		   States which is used by the Search heirarchy.  The last
		   is the SearchInducer heirarchy.  SearchInducer abstracts
		   commonalities in a Wrapper Inducer based on performing
		   a search using another Inducer.
  Assumptions  :
  Comments     :
  Complexity   : Depends on space being searched.
  Enhancements :
  History      : Dan Sommerfield                                   12/12/94
                   Initial revision (.c)
***************************************************************************/
#include <basics.h>
#include <BFSearch.h>
#include <DblLinkList.h>
#include <GetOption.h>
#include <SSSearch.h>

// RCSID("MLC++, $RCSfile: SSSearch.c,v $ $Revision: 1.14 $")

/***************************************************************************
  Description : Sets user options common to all search methods
  Comments    : 
***************************************************************************/
template <class LocalInfo, class GlobalInfo>
void StateSpaceSearch<LocalInfo, GlobalInfo>::set_user_options(
   const MString& prefix)
{
   set_show_test_set_perf(
      get_option_enum(prefix + "SHOW_TEST_SET_PERF", showTestSetPerfEnum,
	      get_show_test_set_perf(), SHOW_TEST_SET_PERF_HELP, TRUE));
   // don't prompt for eval limit because it is specific to whatever
   // is using the StateSpaceSearch (e.g. FSS)
}

/***************************************************************************
  Description : Resets all options to their default values
  Comments    : 
***************************************************************************/
template <class LocalInfo, class GlobalInfo>
void StateSpaceSearch<LocalInfo, GlobalInfo>::set_defaults()
{
   set_show_test_set_perf(DEFAULT_SHOW_TEST_SET_PERF);
   set_eval_limit(DEFAULT_MAX_EVALS);
}


