// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SSSearch_h
#define _SSSearch_h 1

#include <State.h>
#include <StateSpace.h>
#include <search_enums.h>

template <class LocalInfo, class GlobalInfo> 
class StateSpaceSearch {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(StateSpaceSearch);

protected:
   // Protected member data
   StateSpace< State<LocalInfo, GlobalInfo> >* graph;
   int numExpansions;
   int evalLimit;
   ShowTestSetPerf showTestSetPerf;

   // Protected method
   virtual Bool terminate() = 0;
public:   
   StateSpaceSearch() { graph = NULL; showTestSetPerf = showBestOnly;}
   virtual ~StateSpaceSearch() { delete graph; }
   virtual void set_user_options(const MString& prefix);
   virtual void set_defaults();
   virtual const State<LocalInfo, GlobalInfo>&
      search(State<LocalInfo, GlobalInfo>* initialState, GlobalInfo *gInfo,
	     const MString& displayFileName = EMPTY_STRING) = 0;

   // Accessor functions.
   virtual Bool has_graph() const { return graph != NULL; }
   virtual const StateSpace< State<LocalInfo, GlobalInfo> >& get_graph() const
      { return *graph; }
   virtual StateSpace< State<LocalInfo, GlobalInfo> >& get_graph()
      { return *graph; }
   virtual int get_num_expansions() const { return numExpansions; }

   virtual void set_show_test_set_perf(ShowTestSetPerf sra) {
      showTestSetPerf = sra; }
   virtual ShowTestSetPerf get_show_test_set_perf() const {
      return showTestSetPerf; }

   virtual void set_eval_limit(int newEvalLimit) {
      evalLimit = newEvalLimit; }
   virtual int get_eval_limit() {
      return evalLimit; }
};

#ifdef COMPILE_TIME_TEMPLATES
#include "SSSearch.ct"
#endif

#endif

