// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SearchDispatch_h
#define _SearchDispatch_h 1

#include <LogOptions.h>
#include <SSSearch.h>
#include <search_enums.h>
#include <HCSearch.h>
#include <SASearch.h>

template <class LocalInfo, class GlobalInfo>
class SearchDispatch {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(SearchDispatch);

   // Member data
   // StateSpaceSearch/general options
   SearchMethod searchMethod;
   ShowTestSetPerf showTestSetPerf;
   int evalLimit;

   // Hill Climbing (HCSearch) options
   /* None */

   // Best-first search (BFSearch) options
   int maxStale;
   Real epsilon;

   // Simulated Annealing (SASearch) options
   /* maxStale and epsilon same as BFSearch */
   int maxEvals;
   int minExpEvals;
   Real lambda;
   int sasSeed;   
public:
   SearchDispatch() { set_defaults(); }

   void set_defaults();
   void set_user_options(const MString& prefix = EMPTY_STRING);

   StateSpaceSearch<LocalInfo, GlobalInfo> *build_search() const;

   // set/get functions for all options
   void set_search_method(SearchMethod method) { searchMethod = method; }
   SearchMethod get_search_method() const { return searchMethod; }

   void set_show_test_set_perf(ShowTestSetPerf show)
     { showTestSetPerf = show; }
   ShowTestSetPerf get_show_test_set_perf() const
     { return showTestSetPerf; }

   void set_eval_limit(int limit);
   int get_eval_limit() const { return evalLimit; }

   void set_max_stale(int max);
   int get_max_stale() const { return maxStale; }

   void set_epsilon(Real eps);
   Real get_epsilon() const { return epsilon; }

   void set_max_evals(int evals);
   int get_max_evals() const { return maxEvals; }

   void set_min_exp_evals(int minExp);
   int get_min_exp_evals() const { return minExpEvals; }

   void set_lambda(Real aLambda);
   Real get_lambda() const { return lambda; }

   void set_sas_seed(int seed);
   int get_sas_seed() const { return sasSeed; }
   
};

#ifdef COMPILE_TIME_TEMPLATES
#include "SearchDispatch.ct"
#endif

#endif



