// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SearchInducer_h
#define _SearchInducer_h 1

#include <Inducer.h>
#include <SSSearch.h>
#include <MRandom.h>
#include <PerfEstState.h>
#include <ProjectInd.h>
#include <SearchDispatch.h>

class SearchInducer : public Inducer {
   RAND_OPTIONS;
   NO_DEFAULT_OPS(SearchInducer);

   // Member data (also see public and protected data)
   MString dotFileName;     // name of dot file for graph output

   // Private method
   virtual Real search(InstanceList* trainingSet);
protected:

   // Protected member data
   // we do own this but it may be NULL
   Array<int> *finalStateInfo;

   SearchMethod searchMethod;
   Categorizer *categorizer;
   BaseInducer *baseInducer;

   // Protected method
   SearchDispatch<Array<int>, PerfEstInfo> searchDispatch;   
public:
   // Public member data
   PerfEstInfo *globalInfo; // global search information.
                           // must be set in derived class constructor

   // Methods
   // Get's ownership of given inducer.  Defaults to environment
   //   variable <prefix>INDUCER if NULL
   SearchInducer(const MString& description, BaseInducer* ind = NULL);
   virtual ~SearchInducer();
   virtual Bool has_global_info(Bool fatalOnFalse = TRUE) const;
   virtual void set_user_options(const MString& prefix);
   
   virtual void train();
   virtual Bool was_trained(Bool fatalOnFalse = TRUE) const;
   virtual const Categorizer& get_categorizer() const;
   virtual Categorizer *release_categorizer();
   
   Real train_and_test_files(const MString& fileStem,
			     const MString& namesExtension = DEFAULT_NAMES_EXT,
			     const MString& dataExtension = DEFAULT_DATA_EXT,
			     const MString& testExtension = DEFAULT_TEST_EXT);

   void display(MLCOStream& stream = Mcout) const;
   virtual CatTestResult *train_and_perf(InstanceList* trainingSet,
					 const InstanceList& testList);
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testList);
   virtual Bool can_cast_to_inducer() const;
   virtual Bool supports_full_testing();
   
   // new virtuals
   virtual PerfEstInfo *create_global_info() const = 0;
   virtual Array<int> *create_initial_info(InstanceList* trainingSet) = 0;
   virtual PerfEstState *create_initial_state(Array<int> *&initialInfo,
					      const PerfEstInfo& gI) const =0;
   virtual Categorizer *state_to_categorizer(const Array<int>& stateInfo)
      const;
   const Array<int>& get_final_state_info() const;

   // options setting functions;  use search_dispatch() to access
   // search options and perf_est_dispatch() to access performance
   // estimation options.  The rest can be set directly.
   SearchDispatch<Array<int>, PerfEstInfo>& search_dispatch()
      {return searchDispatch; }
   PerfEstDispatch& perf_est_dispatch()
      {DBG(has_global_info()); return globalInfo->perfEst; }

   void set_seed(int seed);
   int get_seed() const;

   void set_use_compound(Bool comp);
   Bool get_use_compound() const;

   void set_cmplx_penalty(Real penalty);
   Real get_cmplx_penalty() const;
         
};

DECLARE_DISPLAY(SearchInducer)

#endif



