// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SearchSplit_h
#define _SearchSplit_h 1

#include <PartitionSplit.h>
#include <BFSearch.h>
#include <LogOptions.h>

class InstanceList;
class PartitionEvaluator;
class ValuePartition;
class ValSetState;

// Derivation is virtual for CartSearch multiple inheritance
class SearchSplit : virtual public PartitionSplit {
   NO_DEFAULT_OPS(SearchSplit);
public:
   enum SearchDirectionType { forwardSearch, backwardSearch };
private:
   // Member data (also see public data)
   BFSearch<ValuePartition, PartitionEvaluator> searcher;
   SearchDirectionType searchDirection;
   Bool bidirectionalSearch;

   // Private methods
   void set_final_partition(const State<ValuePartition, 
	PartitionEvaluator>& state, const Array<Real>& attrCounts, 
	PartitionEvaluator& evaluator,
        const DynamicArray<int>& extras);

   const State<ValuePartition, PartitionEvaluator>&
      search_from_node(ValSetState *state, PartitionEvaluator& evaluator);

   ValSetState *build_root_state(const Array<NominalVal>& values,
				 const PartitionEvaluator& eval,
				 const NominalAttrInfo& attrInfo) const;
   
   void reverse_search_direction();
public:
   // Public data
   static const SearchDirectionType defaultSearchDirection;
   static const Bool defaultBidirectionalSearch;

   // Methods
   SearchSplit();
   ~SearchSplit() { }
   
   void subset_split(PartitionEvaluator& evaluator);
   void rooted_search(PartitionEvaluator& evaluator, Array<int>& root);
   void set_user_options(const MString& prefix);
   void set_search_direction(SearchDirectionType dir)
   { searchDirection = dir; }
   SearchDirectionType get_search_direction() const
   { return searchDirection; }
   void set_bidirectional_search(Bool bi) { bidirectionalSearch = bi; }
   Bool get_bidirectional_search() const { return bidirectionalSearch; }
};

#endif
