// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SplitInfoCache_h 
#define _SplitInfoCache_h 1

#include <Attribute.h>
#include <LogOptions.h>

// @@ this == -REAL_MAX, given that Real doesn't have 306 digits of precision
const Real UNSET_INFO_VAL = -REAL_MAX + 1;

class SplitInfoCache;
class InstanceList;

// This class was introduced to speed up SplitInfoCache.
class CacheInfo {
   NO_COPY_CTOR(CacheInfo);
   NO_OPERATOR_ASSIGN(CacheInfo);
public:   
   SplitInfoCache* sic;
   Real infoValue;
   Real numInstances;   //@@ this is really the weight
   NominalVal splitVal;

   CacheInfo() { init(); }
   void init() {
       sic = NULL; infoValue = UNSET_INFO_VAL; 
       splitVal = UNKNOWN_NOMINAL_VAL - 1;
       numInstances = -1;
   } 
   CacheInfo(Real infoVal, NominalVal val) {
      sic = NULL;
      infoValue = infoVal;
      splitVal = val;
      numInstances = -1;
   }
   ~CacheInfo();
   Bool operator==(const CacheInfo& rhs) const; 
};

//ABC
class SplitInfoCache {
   NO_COPY_CTOR(SplitInfoCache);
   NO_OPERATOR_ASSIGN(SplitInfoCache);
   LOG_OPTIONS;
   InstanceList *instanceList;
   Real numInstances;  //@@ this is really the weight
protected:

   PtrArray< Array<CacheInfo> *> infoMatrix; // sparse matrix
   virtual void build_info_values(int attrNum) = 0;
   virtual InstanceList* split_child(int attrNum, NominalVal ourVal,
				       NominalVal splitVal) = 0;
public:
   virtual void OK(int level = 0) const;
   SplitInfoCache(InstanceList*& il);
   SplitInfoCache(const SplitInfoCache& sic, CtorDummy);
   virtual ~SplitInfoCache();

   const InstanceList& get_instance_list()
      {ASSERT(instanceList != NULL); return *instanceList;}
   Real num_instances() const { return numInstances; }   //@@ this is a weight
   Real info_value(int attrNum, NominalVal val) const;   
   // non-const because it returns a pointer to non-const member.
   virtual SplitInfoCache& child_cache(int attrNum, NominalVal val) = 0;
   virtual const InstanceList& get_instance_list() const
      {return *instanceList; }
   virtual void delete_small_caches(Real minInstances); //@@ minInstance is weight
   Real num_instances(int attrNum, NominalVal val) const; //@@ this is a weight
   Bool operator== (const SplitInfoCache& rhs) const;
};




#endif
