// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SplitSelector_h
#define _SplitSelector_h 1

#include <DblLinkList.h>
#include <SplitScore.h>
#include <LogOptions.h>

class LeafGenerator;
class LeafCategorizer;
class NodeCategorizer;
class SubtreeInfo;
class SplitSuggestion;

class SplitSelector {
   LOG_OPTIONS;
   NO_DEFAULT_OPS(SplitSelector);
   
   LeafGenerator *leafGenerator;
   const SplitScore::SplitScoreCriterion splitCriterion;
protected:
   Real find_mean_mutual_info(const DblLinkList<SplitSuggestion*>&
			      suggestions,
			      const Array<Bool>& multiValAttr) const;
public:
   SplitSelector(LeafGenerator*& lGen,
		 SplitScore::SplitScoreCriterion criterion);
   virtual ~SplitSelector();

   LeafCategorizer *create_leaf_categorizer(SubtreeInfo& info) const;
   
   virtual NodeCategorizer *
   choose_categorizer(SubtreeInfo& info,
		      const DblLinkList<SplitSuggestion*>& suggestions) const;

   const LeafGenerator& get_leaf_generator() const { return *leafGenerator; }
   SplitScore::SplitScoreCriterion get_split_criterion() const
   { return splitCriterion; }
};

#endif
