// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _StackingInd_h
#define _StackingInd_h 1

#include <Inducer.h>

class StackingCat;

class StackingInd : public Inducer {
   NO_DEFAULT_OPS(StackingInd);

   // Member data (also see public data)
   StackingCat *categorizer;
   PtrArray<Inducer*> *indArray;
   Inducer *stackedInd;
   Bool stackWithInst;
   
   static const int minNumInducers;
   static const int maxNumInducers;

   // Private methods
   Bool is_initialized(Bool fatalOnFalse = TRUE) const;
   InstanceList *create_stacking_TS(PtrArray<Categorizer*> *catArray) const;
   SchemaRC create_meta_schema(int numInducers) const;
   void init_inducer_array(const PtrArray<Inducer*>* inducers);   
public:
   // Public data
   static const int defaultNumInducers;
   static const Bool defaultStackWithInst;

   // Methods
   virtual void OK(int level = 0) const;

   StackingInd(const MString& description);
   StackingInd(const StackingInd& source, CtorDummy);
   virtual ~StackingInd();

   virtual void train(void);
   virtual void train_cat(PtrArray<Categorizer*> &catArray);
   virtual void train_meta(PtrArray<Categorizer*> *&catArray);
   virtual Bool was_trained(Bool fatalOnFalse = TRUE) const;
   virtual const Categorizer& get_categorizer(void) const;
   virtual Categorizer *release_categorizer();
   virtual Inducer* copy(void) const;

   virtual int class_id(void) const { return STACKING_INDUCER; }

   virtual void set_user_options(const MString& prefix);
   void set_inducers(PtrArray<Inducer*>*& inducers);
   void set_stacked_inducer(Inducer*& stackInducer);
   void set_stack_with_inst(Bool stackInst);
};

#endif
