// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef StatData_h
#define StatData_h 1

#include <DynamicArray.h>
#include <class_id.h>

// Define class IDs for StatData-derived classes.
#define CLASS_STAT_DATA  STAT_DATA_ID_BASE + 1
#define CLASS_ACC_DATA   STAT_DATA_ID_BASE + 2
#define CLASS_ERROR_DATA STAT_DATA_ID_BASE + 3

class StatData{
   NO_COPY_CTOR(StatData);

   // Member data
   DynamicArray<Real> items;
public:
   // Note that we must initialize the DynamicArray with 0 because if we
   //   initialize it with a high number, we'll have zeroes cropping in when
   //   the array is sorted.
   StatData(): items(0) {}
   virtual ~StatData(){}
   Real operator [](int index) const;
   void insert(Real item);
   int size() const { return items.size(); }
   Real mean(Real trim = 0) const;
   Real variance(Real trim = 0) const;
   Real std_dev(Real trim = 0) const;
   Real variance_of_mean(Real trim = 0) const;
   Real std_dev_of_mean(Real trim = 0) const;
   // percentile gives the confidence interval with the given probability
   void percentile(Real confIntProb, Real& low, Real& high) const;
   void display(MLCOStream& stream = Mcout) const;
   void display_math_histogram(Real columnWidth, int precision,
			       MLCOStream& stream = Mcout);
   virtual void append(const StatData& other);
   
   virtual void clear();
   virtual int class_id() const;
   StatData& operator=(const StatData& rhs);
   virtual Bool operator==(const StatData& rhs) const;
   Bool operator!=(const StatData& rhs) const
      {return ! ((*this) == rhs);}
   Real squared_error(Real trueVal) const;
};

DECLARE_DISPLAY(StatData)

#endif
