// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _StringMap_h
#define _StringMap_h 1

#include <DynamicArray.h>
#include <UnivHashTable.h>

template <class Elem>
class StringHashData {
   MString key;
   Elem value;

public:
   StringHashData(const MString& aname, const Elem& aval) :
      key(aname), value(aval) { }
   StringHashData<Elem>& operator=(const StringHashData<Elem>& other) {
      key = other.key;  value = other.value;  return *this; }
   Bool contains_key(const MString& akey) const { return key == akey; }
   const MString& get_key() const { return key; }
   const Elem& get_value() const { return value; }
   void set_value(const Elem& val) { value = val; }
   void display(MLCOStream& stream=Mcout) const;
};

template <class Elem>
DECLARE_DISPLAY(StringHashData<Elem>)


template <class Elem>
class StringMap {
   NO_COPY_CTOR(StringMap);
   NO_OPERATOR_EQ(StringMap);
   UniversalHashTable<MString, StringHashData<Elem> > table;
   DynamicArray<MString> names;
   
public:
   void OK() const;

   StringMap(int sizeHint = 512);

   // Add a mapping to the hash table.  Abort if the mapping was already
   // present.
   void add(const MString& name, const Elem& value);

   // Set the mapping for a name.  Abort if the mapping is missing
   void set(const MString& name, const Elem& value);

   // Establish a mapping;  Add the mapping if its missing, otherwise change
   // the existing mapping
   void map(const MString& name, const Elem& value);

   // Retrieve the mapping for a name.  Abort if the mapping is missing
   const Elem& get(const MString& name) const;

   // Retrieve a mapping.  Return TRUE if the mapping is present, FALSE if
   // the mapping is missing.  Return the value (on a TRUE return) through
   // the value parameter.
   Bool get(const MString& name, Elem& value) const;

   // Return the number of items in the table
   int size() const { return names.size(); }

   // Return the ith name, subject to range checking
   const MString& index(int i) const { return names.index(i); }

   // display everything in the table
   void display(MLCOStream& stream = Mcout) const;
};

template <class Elem>
DECLARE_DISPLAY(StringMap<Elem>)

// use the following define to set the required loadFactor and
// SET_DLLPIX_CLEAR for use of StringMap.  This must be done in the
// c file which uses the mapper.
#define DEF_STRING_MAP(Elem) \
  const Real HashTable<MString, StringHashData<Elem> >::loadFactor = 1; \
  SET_DLLPIX_CLEAR(StringHashData<Elem>, NULL)

#ifdef COMPILE_TIME_TEMPLATES
#include "StringMap.ct"
#endif
     
#endif


