// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _SubtreeInfo_h
#define _SubtreeInfo_h 1

#include <RealAndLabelColumn.h>

class CGraph;
class InstanceList;

// This class contains no virtual functions. Do not derive from it unless you
// take care to change overridden functions (including the destructor) to
// virtual.

class SubtreeInfo {
   NO_DEFAULT_OPS(SubtreeInfo);

   // items used during induction, to pick splits, etc.
   CGraph& graph;
   const InstanceList& instanceList;
   const Array<int>* tieBreakingOrder;  // we do not own this pointer
   const Array<Real>& parentWeightDistribution;
   const Array<Bool>& multiValAttributes;
   
   PtrArray<RealAndLabelColumn*> *realColumns;

   int level;

   Real totalWeight;  // weight of the whole training set

   Real minSplit;
   Real nominalMinSplit;

   Bool dribble;

   // options that we copy down the tree, because the functors need them
   // @@ alternatively, we could put these options into the functors, since
   // they don't change as we progress (probably the way to go).
   const Real pruningFactor;  // needed to accumulate info at the leaves
   const Bool debugDisplay;
   
   // items determined during induction: accumulated error, loss, etc.
   Real subtreeError;
   Real pessimisticSubtreeError;
public:   
   SubtreeInfo(CGraph& cGraph, const InstanceList& il,
	       const Array<int>* order, const Array<Bool>& multiVal,
	       Real weight, Real pruneFactor, Bool debugDisp);
   SubtreeInfo(const SubtreeInfo& prototype, const InstanceList& il);
   SubtreeInfo(const SubtreeInfo& rhs, CtorDummy);
   ~SubtreeInfo();

   void find_real_columns();
   
   CGraph& get_graph() const { return graph; }
   const InstanceList& instance_list() const { return instanceList; }

   void set_tiebreaking_order(const Array<int>* newOrder);
   const Array<int>& get_tiebreaking_order() const;
   void delete_tiebreaking_order();
   
   const Array<Real>& parent_weight_distribution() const
   { return parentWeightDistribution; }
   const Array<Bool>& multi_val_attributes() const
   { return multiValAttributes; }

   void set_min_split(Real min) { minSplit = min; }
   Real min_split() const { return minSplit; }

   void set_nominal_min_split(Real min) { nominalMinSplit = min; }
   Real nominal_min_split() const { return nominalMinSplit; }

   Real total_weight() const { return totalWeight; }

   int get_level() const { return level; }
   void set_level(int l);

   RealAndLabelColumn& labelled_real_column(int attrNum);

   Bool get_dribble() const { return dribble; }
   void set_dribble(Bool dr) { dribble = dr; }

   Real get_pruning_factor() const { return pruningFactor; }

   Bool get_debug_display() const { return debugDisplay; }
   
   Real get_subtree_error() const { return subtreeError; }
   void set_subtree_error(Real err) { subtreeError = err; }

   Real get_pessimistic_subtree_error() const
   { return pessimisticSubtreeError; }
   void set_pessimistic_subtree_error(Real err)
   { pessimisticSubtreeError = err; }
};


#endif
