// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _T2Inducer_h
#define _T2Inducer_h 1

#include <Inducer.h>
#include <SchemaRC.h>
#include <MLCStream.h>
#include <DecisionTree.h>
#include <TmpFileName.h>

// These are useful as global functions.  In fact, they provide
//   more functionality than the inducer above.
// The first parameter should be set to the command line that
//   will execute T2.
Bool run_t2(const LogOptions& logOptions, const MString& t2Pgm, 
 	    const InstanceList& trainingSet, const InstanceList& testSet,
	    int& treeSize, int& numErr, Real& expAcc);

// This version is intended to be used if you're running T2 a lot.
// Profiling showed that file creation (creat) and deletion (unlink) were
//   taking up to 40% of the time in some cases, so this version takes open
//   streams, writes to them, and truncates.
Bool run_t2(const LogOptions& logOptions, const MString& t2Pgm,
	    const InstanceList& trainingList, const InstanceList& testList,
	    int& treeSize, int& numErr, Real& expAcc,
	    const MString& tmpFileStem, MLCOStream& namesStream,
	    MLCOStream& dataStream, MLCOStream& testStream);

class T2Inducer : public BaseInducer {
   NO_DEFAULT_OPS(T2Inducer);

   // Member data (also see public data)
   MString pgmName;
   MString pgmFlags;

   PtrArray<TmpFileName *> *tempNames;
   MLCOStream* namesStream;
   MLCOStream* dataStream;
   MLCOStream* testStream;

   // Private methods
   void init_files();
   void end_files();
public:
   // Public data
   static const MString defaultPgmName;
   static const MString defaultPgmFlags;

   // Methods
   T2Inducer(const MString& description, 
	     const MString& theFlags1 = defaultPgmFlags,
	     const MString& aPgmName1 = defaultPgmName);
   virtual ~T2Inducer();
   virtual Real train_and_test(InstanceList* trainingSet,
			       const InstanceList& testList);
   virtual int class_id() const { return T2_INDUCER; }
};


#endif








