// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _ThresholdCat_h
#define _ThresholdCat_h 1

#include <NodeCategorizer.h>
#include <DisplayPref.h>

// Persistent helper for threshold categorizer
class PerThresholdCategorizer_ : public PerNodeCategorizer_ {
public:
   NO_DEFAULT_OPS(PerThresholdCategorizer_);
   
   int attrNum;
   Real thresholdVal;

   PerThresholdCategorizer_()
      : attrNum(UNDEFINED_INT), thresholdVal(UNDEFINED_REAL) {}
   virtual void read(MInStore& in);
   virtual void write(MOutStore& out);
};


class ThresholdCategorizer : public NodeCategorizer {
   NO_DEFAULT_OPS(ThresholdCategorizer);

   // Member data
   const AttrInfo& attrInfo;
   int attrNum;
   Real thresholdVal;
   MStringRC LTEDscr;  // "less than or equal to" description
   MStringRC  GTDscr;  // "greater than" description

public:
   ThresholdCategorizer(const SchemaRC& schema, int attributeNum,
                        Real threshold, const MString& dscr);
   ThresholdCategorizer(const ThresholdCategorizer& source,
                        CtorDummy);
   ThresholdCategorizer(const PerThresholdCategorizer_& data,
			const SchemaRC& schema);
   
   virtual void display_struct(MLCOStream& stream = Mcout,
			       const DisplayPref& dp=defaultDisplayPref) const;
   // Returns a pointer to a deep copy of this ThresholdCategorizer
   virtual Categorizer* clone() const;

   virtual AugCategory branch(const InstanceRC& inst) const;
   virtual AugCategory *get_edge_label(int branchNum) const;

   // Returns class id
   virtual int class_id() const { return CLASS_THRESHOLD_CATEGORIZER; }
   virtual Bool operator==(const Categorizer &rhs) const ;
   int attr_num() const { return attrNum; }
   Real threshold() const {return thresholdVal;}
   virtual void change_threshold(Real threshold);
   virtual Array<MStringRC>* real_edge_strings() const;
   
   virtual void set_used_attr(BoolArray& usedAttr) const;
   virtual void project(const BoolArray& mask);

   // make persistent
   virtual PerCategorizer_ *build_persistent_object() const {
      return new PerThresholdCategorizer_; }
   virtual void make_persistent(PerCategorizer_& data) const;

   virtual Bool supports_persistence() const { return TRUE; }

   AugCategory tester_branch(const InstanceRC& inst) const // for testers only
   { return branch(inst); }
};

#endif

