// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file "TmpFileName.c".

#ifndef _TmpFileName_h
#define _TmpFileName_h 1

#include <Array.h>

// This class, based on MString, stores a temporary file name.
// All registration (register and unregister) is taken care of automatically
// through this class.
class TmpFileName {
   NO_DEFAULT_OPS(TmpFileName);

   friend PtrArray<TmpFileName *> *gen_temp_file_names(const Array<MString>&);

   // Member data
   MString name;
   MString stem;

   // Private methods
   TmpFileName(const MString& base, const MString& suffix);
   void build_name(const MString& base, const MString& suffix);   
public:
   TmpFileName(const MString& suffix);
   ~TmpFileName();

   const MString& get_stem() const { return stem; }
   operator const char *() const { return (const char *)name; }
   operator const MString& () const { return (const MString&)name; }

   static Bool generate_fixed_names;
   static const char *fixed_name_stem;
   static MString generate_temp_name();
};


class TmpDirectoryName {
   NO_DEFAULT_OPS(TmpDirectoryName);

   // Member data
   MString currentDirectory;
   MString name;   
public:
   TmpDirectoryName();
   ~TmpDirectoryName();
   void enter_temp_directory();
   void exit_temp_directory();
};


// friend function for creation of temporary file names
PtrArray<TmpFileName *> *gen_temp_file_names(const Array<MString>& suffixes);


#endif


