// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _TmpFileRegistry_h
#define _TmpFileRegistry_h 1

#include <DynamicArray.h>

// The TmpFileRegistry class stores a raw list of filenames of known
// temporary files currently in use by MLC++.  The TmpFileRegistry
// can clean itself out at any time by deleting these files if need
// be.

class TmpFileRegistry : private DynamicArray<MString> {
   // Member data
   DynamicArray<Bool> dir;

   // Private methods
   void verify_empty();
   void register_name(const MString& name, Bool directory);
   void unregister_name(const MString& name, Bool directory);
public:
   TmpFileRegistry();
   ~TmpFileRegistry();
   
   void register_file(const MString& fileName) {
      register_name(fileName, FALSE); }
   void register_directory(const MString& directoryName) {
      register_name(directoryName, TRUE); }
   void unregister_file(const MString& fileName) {
      unregister_name(fileName, FALSE); }
   void unregister_directory(const MString& directoryName) {
      unregister_name(directoryName, TRUE); }
   void clean();
   void list_files(ostream& out);
};

#endif


