// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _ValSetState_h
#define _ValSetState_h 1

#include <State.h>
#include <ValPartition.h>
#include <PartitionEval.h>
#include <SearchSplit.h>

typedef State<ValuePartition, PartitionEvaluator> PartitionSearchState;

class ValSetState : public PartitionSearchState {
   NO_DEFAULT_OPS(ValSetState);

public:
   enum SuccessorOperatorType { forwardSuccessor, backwardSuccessor,
				bothSuccessor };

private:
   // Member data (also see public data)
   SuccessorOperatorType successorOperator;
   DBG_DECLARE(int initialNumVals;)

   // Private methods
   void gen_all_forward(DblLinkList<PartitionSearchState*> *successors);

   void gen_all_succ(DblLinkList<PartitionSearchState*> *successors);
   
   void gen_forward(DblLinkList<PartitionSearchState*> *successors,
		    int swapIndex = 0, int baseSetNum = 0);
   
   void gen_backward(DblLinkList<PartitionSearchState*> *successors);
   
   void insert_new_set(NominalVal newValue,
		       DblLinkList<PartitionSearchState*>& setList,
		       const Array<NominalVal>& baseSet,
		       int baseSetNum) const;

   void insert_value_into_sets(NominalVal newValue,
			       DblLinkList<PartitionSearchState*>& setList,
			       const Array<NominalVal>& baseSet,
			       int baseSetNum) const;

   ValSetState *create_merged_state(int set1, int set2) const;
   void merge_first_set(DblLinkList<PartitionSearchState*>& successors);

public:
   // Public data
   static const SuccessorOperatorType defaultSuccessorOperator;

   // Methods
   void OK(int level = 0) const;
   ValSetState(ValuePartition*& initialInfo,
	       const PartitionEvaluator& gI);
   ~ValSetState() { DBG(OK()); }

   Real eval(PartitionEvaluator *, Bool computeReal = TRUE,
	     Bool computeEstimated = TRUE);

   DblLinkList<PartitionSearchState*>*
   gen_succ(PartitionEvaluator *globalInfo,
	    StateSpace<PartitionSearchState> *space,
	    Bool computeReal = TRUE);

   DBG_DECLARE(void set_num_vals(int num) { initialNumVals = num; })

   void set_successor_operator(SuccessorOperatorType op)
   { successorOperator = op; }
   SuccessorOperatorType get_successor_operator() const
   { return successorOperator; }
   
   void display_for_graph(MLCOStream& stream) const;

   void display_info(MLCOStream& stream) const;
   void display_stats(MLCOStream& stream,
		      PartitionEvaluator *) const;
};

#endif
