// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _error_h
#define _error_h 1

#include <setjmp.h>

extern MString fatal_expected;
extern jmp_buf backFromFatal;

MLCOStream& fatal_error(MLCOStream&);
ostream& fatal_abort(ostream&);
Bool is_valid_pointer(void *ptr, Bool fatalOnFalse = FALSE);
void install_signal_handler(void);
void install_math_exception_handler(void);

const int ERRMESSAGEBUF_SIZE = 1024;

// @@ remove this one because everybody should be using NO_DEFAULT_OPS
#define NO_OPERATOR_EQ(X) X& operator=(const X&)

// leave it undefined to get link error if attempting to use.
#define NO_COPY_CTOR(X) X(const X&)
#define NO_OPERATOR_ASSIGN(X) X& operator=(const X&)
#define NO_DEFAULT_OPS(X) \
  NO_COPY_CTOR(X); \
  NO_OPERATOR_ASSIGN(X)

// for templates, use the format below with X not containing <T>
// otherwise g++ complains
#define NO_COPY_CTOR_TEMP(X,Y) X(const Y&)

// The second argument is to handle base class constructors we must specify
// the argument is always called x
// The statement "(void)x;" prevents warnings about x not being used
// This problem was originally solved using the comma operator, but that
// became a problem in CatGraph.h, where the arguments were too complex
#define NO_COPY_CTOR2(X,ctors) X(const X& x) : ctors \
    {(void)x; err << "No copy constructor for " #X << fatal_error;}

#define NO_COPY_CTOR3(X, ctor1, ctor2) X(const X& x) : ctor1, ctor2 \
    {(void)x; err << "No copy constructor for " #X << fatal_error;}

#define NO_COPY_CTOR4(X, ctor1, ctor2, ctor3) \
    X(const X& x) : ctor1, ctor2 , ctor3 \
    {(void)x; err << "No copy constructor for " #X << fatal_error;}

#define NO_COPY_CTOR5(X, ctor1, ctor2, ctor3, ctor4) \
    X(const X& x) : ctor1, ctor2 , ctor3, ctor4 \
    {(void)x; err << "No copy constructor for " #X << fatal_error;}


#define NOT_IMPLEMENTED \
    {err << "Unimplemented function called " << __FILE__ << " at line " \
         << __LINE__ << fatal_error;}
     
// MLC++ exception mechanism
// MLCTRY and MLCCATCH must occur together, in that exact order.

#define MLCTRY(stmt)                                                   \
  if(setjmp(mlcExceptionBuf) == 0) {                                   \
    mlcExceptionSet = TRUE;                                            \
    stmt                                                               \
  }

#define MLCCATCH(stmt)				                   \
  else {					                   \
    if(!mlcExceptionSet)			                   \
      err << "MLCCATCH: attempting to use MLCCATCH without MLCTRY" \
	  << fatal_error;			                   \
    mlcExceptionSet = FALSE;			                   \
    stmt					                   \
  } 						                   \
  mlcExceptionSet = FALSE;

MString get_mlc_error_message();
extern jmp_buf mlcExceptionBuf;
extern Bool mlcExceptionSet;

#endif
