// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  It provides a macro and a variable for use in
//   testers--TEST_ERROR and memcheck.

/***************************************************************************
  Description  : Provide primitive error catching capability.
  Comments     : To use this in testers,
                   Use the macro TEST_ERROR(msg, stmts) where
                     msg is part of the expected error message, and stmt
                     is some set of statements that generate the expected
                     error.
		   Use 'if memCheck' to check and branch on the condition
		     of the environment variable MLC_MEMCHECK.
  Enhancements : 
  History      : Ronny Kohavi                                       7/17/93
                   Initial revision

***************************************************************************/


#ifndef _errorUnless_h
#define _errorUnless_h 1

#include <error.h>
#include <setjmp.h>
#include <MString.h>
#include <GetOption.h>
#include <OptionServer.h>

static Bool get_mem_check()
{
   MString mCheck;
   Bool mCheckBool = FALSE;
   // If the compile-time directive has this set, it's true
#ifdef MLC_MEMCHECK
   mCheckBool = TRUE;
#endif

   if (optionServer->get_option("MLC_MEMCHECK", mCheck)) {
      mCheck = mCheck.to_upper();
      if (mCheck  == "TRUE" || mCheck == "YES")
	 mCheckBool = TRUE;
      else if (mCheckBool == TRUE)
	 err << "Warning: compile time directive MLC_MEMCHECK on "
	        " while environment variable MLC_MEMCHECK is set but "
	    "not to TRUE/YES" << fatal_error;
   }
   // side effect--disable use of environment options here--
   // keep them out of testers!
   optionServer->enable_environment(FALSE);

   return mCheckBool;
}


Bool memCheck = get_mem_check();

#ifndef FAST
#define TEST_ERROR(msg,stmts) if (GlobalOptions::debugLevel) { \
   Bool saveDribble = GlobalOptions::dribble;\
   fatal_expected = msg; \
   if (setjmp(backFromFatal) == 0) \
      {stmts; err << "TEST_ERROR at " << __FILE__ << ':' << __LINE__ << \
              " did NOT generate expected fatal_error" << fatal_error;}\
   else /* back from longjmp */ \
      GlobalOptions::dribble = saveDribble;}
#else
#define TEST_ERROR(msg,stmts)
#endif

#endif
