// MLC++ - Machine Learning Library in -*- C++ -*-
// See Descrip.txt for terms and conditions relating to use and distribution.

// This is an include file.  For a full description of the class and
// functions, see the file name with a "c" suffix.

#ifndef _MACHINE_h
#define _MACHINE_h 1

#define FUNCPROTO 1

#ifdef MIPS_PRO
#define FASTINLINE
#endif
 
// For gnu:           #if defined(GNU)
// For MIPS_PRO n32:  #if (_MIPS_SIM ==  _ABIN32)

#if (_MIPS_SIM ==  _ABIN32)
#  define N32_COMPILE 1
#else
#  define N32_COMPILE 0
#endif


void overlap_byte_copy(char* dest, char* src, int);
int  str_case_cmp(const char *s1, const char *s2);

// Read some standard value names here.
// On UNIX systems, we can use values.h.
// On the PC, we need to define these ourselves
// values.h is needed for MINDOUBLE, MAXDOUBLE, etc.

#ifdef PC_MSVC
#include <float.h>
#define _DEXPLEN        11
#define _FEXPLEN        8

// the PC lacks M_LOG2E.  Define it right here
#define M_LOG2E  1.4426950408889634074

#else
#include <values.h>
#endif


// the PC lacks the getopt function
#ifdef PC_MSVC
extern int getopt(int, char *const *, const char *);
#endif

// filename conversion--does nothing under UNIX
int convert_filename_to_native(const char *src, char *dest);

// byte ordering conversion--universal hash tables and binary storage expect
// ints and longs in network byte order.  Use these functions to convert.
#ifdef PC_MSVC
#define NEED_BYTE_SWAP
#else
#undef NEED_BYTE_SWAP
#endif

#ifdef NEED_BYTE_SWAP

inline unsigned short host_to_net_16(unsigned short val)
{ return (val << 8) | (val >> 8); }

inline unsigned long host_to_net_32(unsigned long val)
{ return ((val << 24) | ((val & 0xFF00) << 8) |
	 ((val >> 8) & 0xFF00) | (val >> 24)); }

// WARNING: this function is UNSAFE!!  You need to make sure "val"
//  actually points to an 8 byte quantity!!
inline void host_to_net_64(unsigned char *val)
{
   char tmp[8];
   tmp[0] = val[0];
   tmp[1] = val[1];
   tmp[2] = val[2];
   tmp[3] = val[3];
   tmp[4] = val[4];
   tmp[5] = val[5];
   tmp[6] = val[6];
   tmp[7] = val[7];

   // swap here
   val[0] = tmp[7];
   val[1] = tmp[6];
   val[2] = tmp[5];
   val[3] = tmp[4];
   val[4] = tmp[3];
   val[5] = tmp[2];
   val[6] = tmp[1];
   val[7] = tmp[0];
}

extern void host_to_net_abort(char *, int, char *);

template <class T>
inline T host_to_net(T val)
{
  if(sizeof(T) == 1)
    return val;
  else if(sizeof(T) == 2) {
    unsigned short v = *((unsigned short *)&val);
    v = host_to_net_16(v);
    return *((T *)&v);
  }
  else if(sizeof(T) == 4) {
    unsigned long v = *((unsigned long *)&val);
    v = host_to_net_32(v);
    return *((T *)&v);
  }
  else if(sizeof(T) == 8) {
    T val2 = val;
    unsigned char *v2 = (unsigned char *)(&val2);
    host_to_net_64(v2);
    return val2;
  }
  else {
     // Can't use err, which isn't defined at this stage
     host_to_net_abort("host_to_net: order conversion is only supported for "
		       "8, 16, 32, or 64-bit quantities.  Your type has ", 
		       sizeof(T) * 8, " bits");
  }
}


inline unsigned short net_to_host_16(unsigned short val)
{ return host_to_net_16(val); }

inline unsigned long net_to_host_32(unsigned long val)
{ return host_to_net_32(val); }  

inline void net_to_host_64(unsigned char *val)
{ host_to_net_64(val); }

template <class T>
inline T net_to_host(T val)
{ return host_to_net(val); }

#else

inline unsigned short host_to_net_16(unsigned short val)
{ return val; }

inline unsigned long host_to_net_32(unsigned long val)
{ return val; }

inline void host_to_net_64(unsigned char *val)
{ (void)val; }

inline unsigned short net_to_host_16(unsigned short val)
{ return val; }

inline unsigned long net_to_host_32(unsigned long val)
{ return val; }

inline void net_to_host_64(unsigned char *val)
{ (void)val; }

extern void host_to_net_abort(char *, int, char *);
template <class T>
inline T host_to_net(T val)
{
  if(sizeof(T) != 1 && sizeof(T) != 2 && sizeof(T) != 4 &&
     sizeof(T) != 8)
     // Can't use err, which isn't defined at this stage
      host_to_net_abort("host_to_net: order conversion is only supported for "
      "8, 16, 32, or 64-bit quantities.  Your type has ", 
      sizeof(T) * 8, " bits");

  return val;
}

template <class T>
inline T net_to_host(T val)
{ return host_to_net(val); }

#endif

class MString;

int run_dotty(const MString& argument);

int run_treeviz(const MString& argument);

int run_eviviz(const MString& argument);

int run_scatterviz(const MString& argument);

int run_ruleviz(const MString& argument);

#endif







